function plotall(varargin)
% function plotall(label, level, dday, nday, year_base)
% label is a plot label (defaults to cruiseid (subdirectory name))
% level is the index of depth to use (defaults to give 50m depth for NBPalmer)
% dday is the decimal day to start plotting on (defaults to first day)
% nday is the number of days to plot (defaults to all; ignored if < 2)

% originated as the UH gplotadcp.m
% ClH 9/19/99 Matlab 5.2
% 2000/12/19 : unstable, no time to debug: deleting everything except vector and contour

%______________________________________________________________________
% You may wish to change these settings.
%______________________________________________________________________

% Contour plots of absolute velocity can be plotted vs. time, lat, or
% lon.  Set contourx to be 'time', 'lat', or 'lon'.

contourx = 'time';               

% Set the default path to read the velocity data from.  The choices
% are '/contour' or '/vector'.  If set to contour, current vectors
% appear more densely on the plot than if set to vector.  

HH.default_path=[pwd, '/vector'];             % vector or contour
nbins = 50;                                   %this has to match the data
def_year_base = datestr(datenum(date),10);        %defaults to current year


%______________________________________________________________________
% >>>>>>>>>>>>>>>>>>End of user-configurable section<<<<<<<<<<<<<<<<<<<
%______________________________________________________________________

%% recommend do not plot these - haven't debugged yet. JH 2000/12/20
mk_percentgoodplot = 0;
mk_ampplot = 0;
mk_attitudeplot = 0;

% Parse the input arguments.

% Defaults:

[fpath,fname,fext,fver] = fileparts(pwd);
label =[' ', fname];
if isempty(findstr('vector', HH.default_path))
  level = 3;
else
  level = 2;
end
dday = NaN;
nday = NaN;
year_base = NaN;

% Get the user's settings:

if (nargin >= 1)
  label = [' ',varargin{1}(1:end)];
end
if (nargin >= 2)
  level = varargin{2}(:);
end
if (nargin >= 3)
  dday = varargin{3}(:);
end
if (nargin >= 4)
  nday = varargin{4}(:);
end
if (nargin >= 5)
  year_base = varargin{5}(:);
end

if (nargin > 5)
  disp(['Usage: ',mfilename,'(label, level, dday, nday, year_base)']);
end

%______________________________________________________________________
% globals are for webvectt and webcont

global MAX_INCHES 
MAX_INCHES = 7;

set(0,'defaulttextfontsize',14);
set(0,'defaultaxesfontsize',14);
set(0,'defaulttextfontweight','bold');
set(0,'defaultaxesfontweight','bold');

ifig = 0;
close all;  %start with no plots on the board

%______________________________________________________________________
% Initialize fields in HH.

HH.split_longitude=0;
HH.comment=label;
HH.vector_this_level=level;
if (length(varargin)==0)
  [HH.start_dd HH.end_dd HH.year_base] = get_dday;
else %we have arguments
  if ~isnan(dday)
    HH.start_dd = dday;
  end
  if ~isnan(nday)
    HH.end_dd = HH.start_dd + nday - 1;
  end
  if ~isnan(year_base)
    HH.year_base = def_year_base;
  end
end

fprintf('start_dd = %d, end_dd = %d\n',HH.start_dd,HH.end_dd)
paused(2)

%______________________________________________________________________
% Load default file for vector/contour plots.

% defaultname must be 'vect', or 'cont', but we can get the string
% from HH.default_path which is already set.

slashes = findstr('/',HH.default_path);
lastslash = slashes(end);
defaultname = HH.default_path((lastslash+1):(lastslash+4))
file = sprintf('%s/%s_uv.mat',HH.default_path,defaultname);

if (exist(file))
     [fpath,fname,fext,fver] = fileparts(file);
     rootname=fname(1:end-3);
	  
     eval(sprintf('load %s/%s_uv.mat',HH.default_path,rootname))
     eval(sprintf('load %s/%s_xy.mat',HH.default_path,rootname))
	  
     disp(sprintf('\n%s/%s_uv.mat\n%s/%s_xy.mat\n',...
		  HH.default_path,rootname,HH.default_path,rootname));
	  
     HH.x=xyt(1,:)';                    %Nx1 vectors
     HH.x=rem(HH.x+360-HH.split_longitude,360)+HH.split_longitude; %split
     HH.y=xyt(2,:)';                    %Nx1 vectors
     HH.dday=xyt(3,:)';                 %Nx1 vectors
     HH.zc=zc;
     HH.z=z;
     HH.u=uv(:,1:2:end);                %ndepths x Nvelocities
     HH.v=uv(:,2:2:end);                %ndepths x Nvelocities

	 save HH  HH

else
  disp(['Default vector/contour file ',file,' doesn't exist\n']);
end

%_______________________________________________________________________
% Now that initialization is done, call vector plotting routine.
fprintf('making vector plot')

  if (length(zc)==length(z))
	zc=mid(z);
  end
  
  badi=find(isnan(HH.x));
  HH.x(badi)=[];HH.y(badi)=[];HH.dday(badi)=[];HH.u(:,badi)=[];HH.v(:,badi)=[];

  %let's fake it a bit - replot vectors twice (since we don't have colors)

  TT=-99+0*HH.x;
  xyt=[HH.x HH.y HH.dday]';
  uvm=[HH.u(HH.vector_this_level,:)'  HH.v(HH.vector_this_level,:)']; %let's try this
  %these have to be the levels surrounding the average (zc"minmax"???)
  zcm=[HH.z(HH.vector_this_level) HH.z(HH.vector_this_level+1)];  

  ifig = ifig + 1;
  vect_fig = ifig;
  figure(ifig);
  clf;
  webvectt(HH.x, HH.y, HH.u(HH.vector_this_level,:)', ...
                       HH.v(HH.vector_this_level,:)', ...
                       xyt, zcm, uvm, TT, label );
  orient tall
  print -dpsc vector.ps

%_______________________________________________________________________
% Second figure is done by mk_adcpcontour.
% Files were loaded by call_init.
fprintf('making contour plot')

  gridlength=100;

  %set things up for calling webcont

  if (strcmp(contourx,'lon'))
        arange=[min(HH.x) max(HH.x)];
	abcissca_grid=[min(arange):diff(arange)/gridlength:max(arange)];
	d=1;
  elseif (strcmp(contourx,'lat'))
        arange=[min(HH.y) max(HH.y)];
	abcissca_grid=[min(arange):diff(arange)/gridlength:max(arange)];
	d=2; 
  elseif (strcmp(contourx,'time'))
        arange=[min(HH.dday) max(HH.dday)];
	abcissca_grid=[min(arange):diff(arange)/gridlength:max(arange)];
	d=3;
  end

  %set up xyt
  xyt=[HH.x HH.y HH.dday]';
  uv=0*[HH.u HH.v];
  uv(:,1:2:end)=HH.u;
  uv(:,2:2:end)=HH.v;

  ifig = ifig + 1;
  cont_fig = ifig;
  figure(ifig);
  clf;
  webcont(abcissca_grid, xyt, d, HH.zc, uv, label)

  orient tall
  print -dpsc contour.ps

%_______________________________________________________________________
if (mk_percentgoodplot)

	% Third figure is the percent good figure.
	fprintf('making percent good plot')

	pgstruct.start_dd = HH.start_dd;
	pgstruct.end_dd = HH.end_dd;
	pgstruct.year_base=HH.year_base;
	pgstruct.comment = label;
	pgstruct.dbpath = [pwd, '/adcpdb/'];
	dbname = dir([pgstruct.dbpath,'*dir.blk']);
	pgstruct.dbname = dbname(1).name(1:(end-7));
	%optional fields (these are the defaults)
	
	pgstruct.nbins = nbins;       %this may be a problem
	pgstruct.PGOOD_MIN=0;      
	pgstruct.PGOOD_MAX=100;    
	pgstruct.icolor=1;         
	pgstruct.reduce_res=0;     
	pgstruct.interval_dd=4;    
	pgstruct.days_per_panel=1; 
	
	save pgstruct  pgstruct
	
	ifig = ifig + 1;
	figure(ifig);
	clf;
	gplotpg(pgstruct);
	orient tall
	print -dpsc percentgood.ps
	
	while(HH.end_dd - pgstruct.start_dd + 1 > pgstruct.interval_dd)
	pgstruct.start_dd = pgstruct.start_dd + pgstruct.interval_dd;
	ifig = ifig + 1;
	figure(ifig);
	clf;
	gplotpg(pgstruct);
	orient tall
	print -append -dpsc percentgood.ps
	end
end


%_______________________________________________________________________
if (mk_ampplot)

	% Fourth figure is the amp
	  fprintf('making amplitude plot')
	
	  ampstruct.start_dd = HH.start_dd;
	  ampstruct.end_dd = HH.end_dd;
	
	  ampstruct.dbpath = [pwd, '/adcpdb/'];
	  dbname = dir([ampstruct.dbpath,'*dir.blk']);
	  ampstruct.dbname = dbname(1).name(1:(end-7));
	  ampstruct.year_base=HH.year_base;
	  ampstruct.cruiseid = label;
	
	  %optional fields (these are the defaults)
	
	  ampstruct.nbins = nbins;       %this may be a problem
	  ampstruct.AMP_MIN=0;      
	  ampstruct.AMP_MAX=256;    
	  ampstruct.icolor=1;         
	  ampstruct.reduce_res=0;     
	  ampstruct.interval_dd=4;    
	  ampstruct.days_per_panel=1; 
	  ampstruct.pg_cutoff = 30;
	 
	  save ampstruct   ampstruct 
	  
	  ifig = ifig + 1;
	  figure(ifig);
	  clf;
	  plotamp(ampstruct);
	  orient tall
	  print -dpsc amplitude.ps
	  while(HH.end_dd - ampstruct.start_dd + 1 > ampstruct.interval_dd)
	    ampstruct.start_dd = ampstruct.start_dd + ampstruct.interval_dd;
	    ifig = ifig + 1;
		fprintf('opening figure %d',ifig);
	    figure(ifig);
	    clf;
	    plotamp(ampstruct);
	    orient tall
	    print -append -dpsc amplitude.ps
	  end
end

%_______________________________________________________________________

if (mk_attitudeplot)
	% Finally, attitude figures.
	  fprintf('making attitude plots (there are 3)\n')
	
	  attfig1num = ifig + 1;
	  attfig2num = ifig + 2;
	  attfig3num = ifig + 3;
	
	  %let's force this to be integer days.
	
	  attfiles = dir([pwd, '/nav/ub*.mat']);
	  m = size(attfiles,1);
	  found = 0;
	  while (m > 0 & ~found)
	    fdd = sscanf(attfiles(m).name, 'ub%03d%03d');
	    if (fdd(1) <= HH.start_dd & fdd(2) > HH.end_dd)
	      found = 1;
	    else
	      m = m - 1;
	    end
	  end
	  if (m == 0)
	    attfile = sprintf('%s/nav/%s', pwd, attfiles(end).name);
	  else
	    attfile = sprintf('%s/nav/%s', pwd, attfiles(m).name);
	  end
	  [fpath, fname, fext, fver] = fileparts(attfile);
	  attstruct.attfile = sprintf('%s/%s',fpath,fname);
	  attstruct.start_dd  = floor(HH.start_dd);
	  attstruct.year_string=sprintf('%d',HH.year_base);
	  attstruct.title = label;
	
	  save attstruct  attstruct
	  gattplot(attstruct);
	
	  figure(attfig1num); orient tall; print  -dpsc attfig1.ps
	  figure(attfig2num); orient tall; print  -dpsc attfig2.ps
	  figure(attfig3num); orient tall; print  -dpsc attfig3.ps
	
	  while(ceil(HH.end_dd) - (attstruct.start_dd) > 3)
	    attstruct.start_dd = attstruct.start_dd + 3;
	    gattplot(attstruct);
	    figure(attfig1num); orient tall; print -append -dpsc attfig1.ps
	    figure(attfig2num); orient tall; print -append -dpsc attfig2.ps
	    figure(attfig3num); orient tall; print -append -dpsc attfig3.ps
	  end
	
	  figure(cont_fig)
	  figure(vect_fig)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                              functions                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%------------------------------

function [start_dday, end_dday, year_base]=get_dday;

% sets the year_base as the year as the first file in the list
  
  raw_files = dir('/home/data/adcp/adcp*.*');

  %get the first file name
  ifile = raw_files(1).name;
  [fpath, fname, fext, fver] = fileparts(ifile);
  year_day = fext(2:end);
  year_base = fname(5:8);
  start_dday = sscanf(year_day,'%d') - 1;
  start_year = sscanf(year_base,'%d');

  %get the last file name
  ifile = raw_files(end).name;
  [fpath, fname, fext, fver] = fileparts(ifile);
  year_day = fext(2:end);
  year_base = fname(5:8);
  end_dday=sscanf(year_day,'%d') - 1;
  end_year = sscanf(year_base,'%d');
  end_date = to_date(end_year, end_dday, 'n');
  full_end_dday = to_day(start_year,end_date);

  year_base = start_year;
  
%------------------------------

function midz=mid(z)
  
  if (length(z)>1)
	midz= (z(1:end-1)+z(2:end))/2;
  else
	midz=z;
  end
  
%------------------------------


