/***************************************************************

   DPMASK.H

   Assigns meaning to the bit positions in the data
   processing mask, for ADCP data.

   Provides a name list for these bit positions.

   Eric Firing
   Wed  09-07-1988

***************************************************************/
#ifndef misc_included
#include "misc.h"                /* NAME_LIST_ENTRY_TYPE */
#endif

#ifndef dpmask_included
#define dpmask_included

#define TIME_CORRECTED           0
#define TRANSDUCER_ORIENTATION   1  /* ROTATE.EXE */
#define COMPASS_ERROR            2
#define TRANSDUCER_BEAM_ANGLES   3
#define SOUND_SPEED_VEL_COR      4
#define SOUND_SPEED_DEPTH_COR    5
#define GLITCHES_REMOVED         6   /* BADBIN.EXE */
#define GLITCHES_CORRECTED       7
#define BOTTOM_SOUGHT            8   /* dbupdate; formerly BOTMPAS3.EXE */
#define BOTTOM_MGB               9   /* set_lgb; formerly LAST_85.EXE */
#define MGB_SET                 10
#define DEPTH_RANGE_COR         11
#define RELATIVE_PROFILES       16
#define REFERENCE_VELOCITIES    17
#define VEL_TO_ABSOLUTE         18
#define VEL_TO_REF_LAYER        19
#define POSITIONS_OPTIMIZED     20
#define NAV_SOURCE_GPS          21
#define NAV_SOURCE_TRANSIT      22
#define NAV_SOURCE_LORAN        23
#define NAV_SOURCE_OMEGA        24
#define NAV_SOURCE_RADAR        25

#define NOT_TIME_CORRECTED           100
#define NOT_TRANSDUCER_ORIENTATION   101
#define NOT_COMPASS_ERROR            102
#define NOT_TRANSDUCER_BEAM_ANGLES   103
#define NOT_SOUND_SPEED_VEL_COR      104
#define NOT_SOUND_SPEED_DEPTH_COR    105
#define NOT_GLITCHES_REMOVED         106
#define NOT_GLITCHES_CORRECTED       107
#define NOT_BOTTOM_SOUGHT            108
#define NOT_BOTTOM_MGB               109
#define NOT_MGB_SET                  110
#define NOT_DEPTH_RANGE_COR          111
#define NOT_RELATIVE_PROFILES        116
#define NOT_REFERENCE_VELOCITIES     117
#define NOT_VEL_TO_ABSOLUTE          118
#define NOT_VEL_TO_REF_LAYER         119
#define NOT_POSITIONS_OPTIMIZED      120
#define NOT_NAV_SOURCE_GPS           121
#define NOT_NAV_SOURCE_TRANSIT       122
#define NOT_NAV_SOURCE_LORAN         123
#define NOT_NAV_SOURCE_OMEGA         124
#define NOT_NAV_SOURCE_RADAR         125

NAME_LIST_ENTRY_TYPE dpmask_list[] =
{
   {"TIME_CORRECTED",         0},
   {"TRANSDUCER_ORIENTATION", 1}, /* ROTATE.EXE */
   {"COMPASS_ERROR",          2},
   {"TRANSDUCER_BEAM_ANGLES", 3},
   {"SOUND_SPEED_VEL_COR",    4},
   {"SOUND_SPEED_DEPTH_COR",  5},
   {"GLITCHES_REMOVED",       6},  /* BADBIN.EXE */
   {"GLITCHES_CORRECTED",     7},
   {"BOTTOM_SOUGHT",          8},  /* BOTMPAS3.EXE */
   {"BOTTOM_MGB",             9},  /* LAST_85.EXE */
   {"MGB_SET",               10},
   {"DEPTH_RANGE_COR",       11},
   {"RELATIVE_PROFILES",     16},
   {"REFERENCE_VELOCITIES",  17},
   {"VEL_TO_ABSOLUTE",       18},
   {"VEL_TO_REF_LAYER",      19},
   {"POSITIONS_OPTIMIZED",   20},
   {"NAV_SOURCE_GPS",        21},
   {"NAV_SOURCE_TRANSIT",    22},
   {"NAV_SOURCE_LORAN",      23},
   {"NAV_SOURCE_OMEGA",      24},
   {"NAV_SOURCE_RADAR",      25},

   {"NOT_TIME_CORRECTED",         100},
   {"NOT_TRANSDUCER_ORIENTATION", 101},
   {"NOT_COMPASS_ERROR",          102},
   {"NOT_TRANSDUCER_BEAM_ANGLES", 103},
   {"NOT_SOUND_SPEED_VEL_COR",    104},
   {"NOT_SOUND_SPEED_DEPTH_COR",  105},
   {"NOT_GLITCHES_REMOVED",       106},
   {"NOT_GLITCHES_CORRECTED",     107},
   {"NOT_BOTTOM_SOUGHT",          108},
   {"NOT_BOTTOM_MGB",             109},
   {"NOT_MGB_SET",                110},
   {"NOT_DEPTH_RANGE_COR",        111},
   {"NOT_RELATIVE_PROFILES",      116},
   {"NOT_REFERENCE_VELOCITIES",   117},
   {"NOT_VEL_TO_ABSOLUTE",        118},
   {"NOT_VEL_TO_REF_LAYER",       119},
   {"NOT_POSITIONS_OPTIMIZED",    120},
   {"NOT_NAV_SOURCE_GPS",         121},
   {"NOT_NAV_SOURCE_TRANSIT",     122},
   {"NOT_NAV_SOURCE_LORAN",       123},
   {"NOT_NAV_SOURCE_OMEGA",       124},
   {"NOT_NAV_SOURCE_RADAR",       125}
};

#endif /* ifndef dpmask_included */

