function adcpcal(cal_, label, trange, dvrange)
% function adcpcal(cal, label, trange, dvrange)
%  input:
%     cal     = the output filename from timslip, or output matrix
%		   which has been loaded into matlab
%     label   = text for plot titles
%  optional arguments:
%     trange  = a vector [t0 t1] with the time range of the
%        calibration points to be used.
%     dvrange = a vector [umin umax vmin vmax] with the
%        range of acceptance of velocity component differences,
%        post-acceleration minus pre-acceleration.
%
%  This function calculates statistics for the
%  amplitude and phase calibration factors, and
%  the seconds to be added to the recorded time to
%  maximize agreement with the navigation, and
%  plots them as time series and as histograms.
%
%  Statistics are appended to the file adcpcal.out.

%       Eric Firing
%       Fri  02-17-1989; major mod Fri  12-15-1989
% 97/05/15 JF: modified to read in cal matrix from timslip output file
%	       directly without stripping off header


if (nargin==2)
   trange = [];
   dvrange = [];
end

% >>>>>>>>>>>>>>>>>>>> user must edit the following lines >>>>>>>>>>>>>>>>>>>>
                     % phase seems more often to contain a trend than amplitude,
                     % so it may be justified to clip phase less stringently
clip_ph  = 3;        % degrees on either side of median
clip_amp = 0.04;     % eliminate anything over x percent from median
clip_var = 0.05;     % maximum variance of ref layer velocity
clip_dt  = 60;       % maximum time slip beyond median
% >>>>>>>>>>>>>>>>>>>>>>>>>>>>> end editing >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

config.clip_ph   = clip_ph  ;
config.clip_amp  = clip_amp ;
config.clip_var  = clip_var ;
config.clip_dt   = clip_dt  ;


config.trange = trange;
config.dvrange = dvrange;
if (nargin >=2), config.label = label; end
if (nargin >=2), config.cal_ = cal_; end

if (~ismember(nargin, [2 3 4]))
   help adcpcal
   error('must specify cal output file name and plot label')
end

adcpcal1(config)