#! /usr/bin/env python

''' link Hdss files to the current directory with new names by:
         - replacing all square brackets and dots with underscores
         - adding ".raw" to the end

    Usage: link_hdss.py sourcedir [globstr]

    if "globstr" is not specified, links everything in the directory
    NOTE: "globstr" _MUST_ be quoted or wildcard expansion will fail

'''

import sys, os, os.path, re, string, glob, shutil
from stat import *

def newname(filename):

   bigpieces = filename.split('[')
#   print bigpieces

   prefix = bigpieces[0]
   lastbit = bigpieces[1]
   timepieces = lastbit.replace('[', '_').replace('.', '_').replace(']','') .split('_')

#   print 'prefix = <%s>' % (prefix,)
#   print 'lastbit = <%s>' % (lastbit,)
#   print timepieces
            
   yr = '20%02d' % (int(timepieces[-2]))
   mo = timepieces[-3]
   day = timepieces[-4]
   frac = timepieces[-1]
   linkto = '%s.raw' % ('_'.join([prefix, yr, mo, day, frac]))

   return linkto


if __name__ == '__main__':

   if len(sys.argv) == 1:
      print 'usage:'
      print __doc__
      sys.exit()

   if len(sys.argv) > 1:
      sourcedir = sys.argv[1]
      if not os.path.isdir(sourcedir):
         print "source directory %s does not exist" % (sourcedir)
         sys.exit()

   if len(sys.argv) == 3:
      globstr = sys.argv[2]
   else:
      globstr = "*"
   print "getting files from %s" % (os.path.join(sourcedir, globstr))
   filelist = glob.glob(os.path.join(sourcedir, globstr))

   if len(filelist) == 0:
      print 'no files found in %s with globstr "%s"' % (sourcedir, globstr)
      sys.exit()

   for filename in filelist:
      linkname = newname(os.path.basename(filename))
      if os.path.islink(linkname):
         print 'symbolic link %s exists' % (linkname)
      else:
         ss= os.stat(filename)
         if ss[ST_SIZE] == 0L:
            print 'not linking empty file %s' % (filename)
         else:
            #print 'linking %s to %s' % (filename, linkname)
            print '.',
            os.symlink(filename, newname(os.path.basename(filename)))

      
   
