#!/usr/bin/env python
'''
link_uhdaslegs.py
------------------

link directories (cruise legs) to one cruise location by component:
    i.e. link each file in raw/*/*, rbin/*/*, gbin/*/*
'''

import os, sys, string

usage = string.join(["usage: python link_uhdaslegs.py fromdir todir",
                     "   eg.",
                     "       python link_uhdaslegs.py  w0605a_legs/w0605a_1 w0605a",
                     ], '\n')


if  len(sys.argv) < 3:
   print __doc__
   print usage
   sys.exit()

fromdir = sys.argv[1]
todir   = sys.argv[2]


# link contents of 'from' to 'to'


if not os.path.isdir(todir):
   os.mkdir(todir)
   
for subdir in ('raw', 'rbin', 'gbin'):
   subdir_path = os.path.join(todir, subdir)
   if not os.path.isdir(subdir_path):
      os.mkdir(subdir_path)
      print 'making directory %s' % (subdir_path)
         
# do raw:
dirlist =  os.listdir(os.path.join(fromdir, 'raw'))
instdirs = [d for d in dirlist if d not in ('config', 'log')]
print '------raw-----\n'
for instdir in instdirs:
   frompath = os.path.join(fromdir, 'raw', instdir)
   topath   = os.path.join(todir,   'raw', instdir)
   if not os.path.isdir(topath):
      os.mkdir(topath)
      print 'making directory %s' % (topath)
   filelist = os.listdir(frompath)
   print 'linking  %s/* to %s/*\n' % (frompath, topath)

   for fname in filelist:
      fromname = os.path.abspath(os.path.join(frompath, fname))
      toname   = os.path.abspath(os.path.join(topath,   fname))
      if not os.path.isfile(toname):
         os.symlink(fromname, toname)
      else:
         print '%s exists.  Not linking' % (toname)      

# do rbin:
print '------rbin-----\n'
instdirs = os.listdir(os.path.join(fromdir, 'rbin'))
for instdir in instdirs:
   frompath = os.path.join(fromdir, 'rbin', instdir)
   topath   = os.path.join(todir,   'rbin', instdir)
   if not os.path.isdir(topath):
      os.mkdir(topath)
      print 'making directory %s' % (topath)
   filelist = os.listdir(frompath)
   print 'linking %s/* to %s/*\n' % (frompath, topath)
   
   for fname in filelist:
      fromname = os.path.abspath(os.path.join(frompath, fname))
      toname   = os.path.abspath(os.path.join(topath,   fname))
      if not os.path.isfile(toname):
         os.symlink(fromname, toname)
      else:
         print '%s exists.  Not linking' % (toname)      
      

# do gbin:
print '------gbin-----\n'
adcpdirs = os.listdir(os.path.join(fromdir, 'gbin'))
for adcpdir in adcpdirs:
   topath   = os.path.join(todir,   'gbin', adcpdir)
   if not os.path.isdir(topath):
      os.mkdir(topath)
      print 'making directory %s' % (topath)

   instdirs = os.listdir(os.path.join(fromdir, 'gbin', adcpdir))
   for instdir in instdirs:
      frompath = os.path.join(fromdir, 'gbin', adcpdir, instdir)
      topath   = os.path.join(todir,   'gbin', adcpdir, instdir)
      if not os.path.isdir(topath):
         os.mkdir(topath)
         print 'nmaking directory %s' % (topath)
      filelist = os.listdir(frompath)
      print 'linking %s/* to %s/*\n' % (frompath, topath)
      
      for fname in filelist:
         fromname = os.path.abspath(os.path.join(frompath, fname))
         toname   = os.path.abspath(os.path.join(topath,   fname))
         if not os.path.isfile(toname):
            os.symlink(fromname, toname)
         else:
            print '%s exists.  Not linking' % (toname)      
            
