#!/usr/bin/python
##to be imported by quick_adcp.py
#
# NOTES
# Each python subroutine should bear the name of the function contained within

import sys, string, os, time, re, shutil, commands, glob
from types import *
from call_matlab import MatlabEngine
from quick_subs import quickFatalError, system_cmd

#from call_matlab import matlab


# -------------------------
def user_defined_headfunc(opts):

   matstr = '''
   %% this file is generated by quick_adcp.py (quick_user.py)
   %% It is easiest (syntactically) if the function called
   %%   is just a name

   vmd_gen_ang;

'''
   matfile = 'user_def_headfunc_tmp.m'
   mat = open(matfile, 'w')
   mat.write(matstr)
   mat.close()

   ## run the string
   ## opts['M'].matlab(matstr, opts['matpathlist'], verbose=1)

   ## call as a file
   try:
      (runstr, output) = opts['M'].matlab(matfile[:-2])
   except:
      if opts['verbose']:
         logappend(opts['logfile'], (runstr, output))


#------------------------

def get_gyrodh(opts):

   #This routine produces an *.ang file of they same type as produced by ashrot
   # should have already changed dir to cal/rotate

   # (1) list heading for scan_time_range so we can get a set of
   #     ensemble endday -- output is scn.hdg

   # (2) set up and run matlab mk_gyrodh


   if opts['dday_bailout'] == '':
      opts['bailoutstr'] = "'dh_ddrange', [%(thisload_startdd)f %(thisload_enddd)f], " % opts
   else:
      opts['bailoutstr'] = "'dh_ddrange', [%(thisload_startdd)f %(dday_bailout)f], " % opts
      print 'bailoutstr is <%s>' % (opts['bailoutstr'])


   matstr = '''

   %(displaystr)s
   set(0,'defaulttextinterpreter','none')

   mk_gyrodh(                '%(hcorr_inst)s', ...
   'cfgbase'          ,      '%(cfgbase)s',...
   'cfgpath'          ,      '%(cfgpath)s',...
   'yearbase'         ,      %(yearbase)d,    %(bailoutstr)s     ...
   'cruiseid'         ,      '%(cruisename)s',...
   'endddayfile'      ,      'scn.hdg',...
   'time_angle_file'  ,      '%(time_angle_file)s',...
   'ascfile'          ,      '%(time_angle_base)s.asc');


''' %opts

   mat = open('gyrodh_tmp.m', 'w')
   mat.write(matstr)
   mat.close()

   ## run the string
   ## opts['M'].matlab(matstr, opts['matpathlist'], verbose=1)

   ## call as a file
   try:
      (runstr, output) = opts['M'].matlab('gyrodh_tmp', verbose=1)
   except:
      if opts['verbose']:
         logappend(opts['logfile'], (runstr, output))

   # clean up opts
   del opts['bailoutstr']

## end get_gyrodh

#####################################################################3

## Let's hope noone  has to use this...

def get_seapath11dh(opts):
   #This routine produces an *.ang file of they same type as produced by ashrot
   # should have already changed dir to cal/rotate

   # (1) list heading for scan_time_range so we can get a set of
   #     ensemble endday -- output is scn.hdg


   cntfile = 'lst_hdg.tmp'

   lsthdg_cnt = '''
   dbname:         ../../adcpdb/%(dbname)s
   output:           scn.hdg
   step_size=        1
   year_base=        %(yearbase)d
   time_ranges:
        %(scn_time_range)s
   ''' %opts

   print lsthdg_cnt
   lhdg = open(cntfile, 'w')
   lhdg.write(lsthdg_cnt)
   lhdg.close()

   syscmd = '%s %s' %  (os.path.join(opts['bindir'], 'lst_hdg'), cntfile)
   try:
      system_cmd(syscmd)
   except:
      msg = 'cannot list heading in ' + os.path.getcwd()
      raise quickFatalError, msg

   # (2) set up and run maglab mk_gyrodh

   matstr = '''

   %% %(diarystr)s
   cfg.toofarsecs = 30;
   timerange =   [%(scn_startdd)s %(scn_enddd)s];

   inst_info = {'gyro','hdg'};
   gyrolist = guess_rfilelist(cfg, timerange, inst_info, 'rbin');

   inst_info = {'seapath1','sea'};
   otherlist = guess_rfilelist(cfg,timerange, inst_info, 'rbin');

   end_dday = read_asc('scn.hdg',3,1);  %%run by get_seapath11dh.py

   mk_gyrodh(gyrolist, otherlist, 'end_dday', end_dday,...
            'timerange', timerange, 'outfile', '%(time_angle_file)s',...
            'othertype', 'seabin');


''' %opts

   mat = open('gyrodh_tmp.m', 'w')
   mat.write(matstr)
   mat.close()

   opts['M'].matlab(matstr, opts['matpathlist'], verbose=1)


## end get_seapath11dh

##################################################################

def plot_lastensq(ldict):


   ###### build matlab call
   matstr = r'''

   set(0, 'defaulttextinterpreter', 'none')

   unix('/bin/rm lastens.diary')
   diary lastens.diary

   path


   set(0,'defaultfigureposition',[0 0 1 1])
   set(0,'defaultfiguremenubar','none')
   set(0,'defaultfiguretoolbar','none')


   cruiseid = '%(cruiseid)s';
   instclass = %(instclass)s;
   instname = '%(instname)s';
   pingtype = '%(pingtype)s';
   procdirname = '%(procdirname)s';

   kt_vec_instrument = '%(kt_vec_instrument)s';
   kt_vec_pingtype   = '%(kt_vec_pingtype)s';
   kt_vec_depthstr =  %(kt_vec_depthstr)s;

   cfg =get_uhdascfg('cfgbase', cruiseid,...
                     'cfgpath'  , '%(cfgpath)s',...
                     'instclass', instclass,...
                     'instname',  instname, ...
                     'pingtype', pingtype);

   logfile = '%(logfile)s';
   fid = fopen(logfile,'w');
   if fid == -1
     fid = 1;
   else
      fprintf(fid, '%%s\n', nowstr);
   end
   lastensfile = fullfile(cfg.uhdas_dir, 'proc', procdirname, 'load', 'lastens.mat')

   try
      ar = load(lastensfile);
      [rr , config] = restruct(instclass, ar.r);
      rr.vel = ar.r.vel;
      rr.amp = ar.r.amp;
      if isfield(ar.r, 'cor'); rr.cor = ar.r.cor;  end

      if isfield(rr.params, 'ducer_depth')
         rr.depth = rr.depth + rr.params.ducer_depth;
      end

      amprange = [config.minclipval.amp1  config.maxclipval.amp1];
      plot_ens(rr, config, cfg, 'hcorr_inst', cfg.hcorr_inst,...
                                  'amprange', amprange);

      fprintf(fid, '%%s ran lastensq plot for %%s\n',...
                   nowstr, procdirname)
      psfile = sprintf('%%s_lastens', procdirname);
      printit(psfile, 'orient', 'portrait');
      paused(1)
      close
    catch
      fprintf(fid, '%%s cannot run lastens plot for %%s\n', ...
                   nowstr, procdirname);
      return
   end


   if ~isempty(kt_vec_instrument)
      if strcmp(kt_vec_instrument, instname)
         pingtype = kt_vec_pingtype;
         if ~isempty(rr)
            figure
            plot_ktvec(rr, config, cfg, ...
               'hcorr_inst', cfg.hcorr_inst,...
               'depthrange', kt_vec_depthstr);

           printit('dayvec', 'orient', 'portrait');
           paused(1)
           close


           figure
           plot_ktvec(rr, config, cfg, ...
               'nightview', 1,...
               'hcorr_inst', cfg.hcorr_inst,...
               'depthrange', kt_vec_depthstr);

           printit('nightvec', 'orient', 'tall');
           paused(1)
                  close
         end
      end
   end


   if fid ~= 1
     fclose(fid);
   end

   ''' % ldict


   #----- main script----

   matfile =  'last%s_tmp.m' % (ldict['instname'])


   mat = open(matfile, 'w')
   mat.write(matstr)
   mat.close()

   ## call as a file
   try:
      #   M = MatlabEngine('matdirs')
      M = MatlabEngine()
      (runstr, output) = M.matlab(matfile[:-2], verbose=0)

   except:
      raise


## end plot_ensq

##################################################################

# This may be only a temporary location for this.

try:

   # try to get around possible bad X interaction
   import matplotlib
   matplotlib.use('Agg')

   ## adding masks
   import matplotlib.numerix.ma as M
   from matplotlib.font_manager import fontManager, FontProperties

   import pylab as P
   def plot_hcorr(hcorr_filename, n_ens=0,
                     titlestring='Heading Correction',
                     outfilename='./hcorr_plot.png',
                     dpi=100):
      '''
      dpi and outfilename can be either matching lists or
      single values.
      '''

      ## Crude method for reading part of the ascii file into an array:        
      # lines = open(hcorr_filename).readlines()[4:]                            
      # nrows = len(lines[0].split())                                           
      # lines = lines[-n_ens:-1]                                                
      # nlines = len(lines)                                                     
      # hcorr = array(map(float, ''.join(lines).split()))                       
      # hcorr = reshape(hcorr, (nlines, nrows))                                 

      ## "shape" is specific to pylab
      hcorr = P.load(hcorr_filename)
      (nrows, ncols) = P.shape(hcorr)

      dday = hcorr[:,0]                                                       
      hd_mean = hcorr[:,1]                                                    
      hd_last = hcorr[:,2]                                                    
      dh_mean = hcorr[:,3]                                                    
      dh_std  = hcorr[:,4]                                                    
      n_used = hcorr[:,5]                                                     
      badmask = hcorr[:,6]                                                    
      if ncols == 8:                                                          
         Npersist = hcorr[:,7]                                                
      else:                                                                   
         Npersist = P.zeros(P.size(dday))                                         
                                                                              
      # don't plot bad stuff                                                  
      dh_meanarray = M.array(dh_mean)                                         
      Npts_array   = M.array(n_used)                                          
      Pmask        = M.array(Npersist)                                        
      badmask      = M.array(badmask)                                         
                                                                              
      #good values only:                                                      
      dh_mean_maskedB = M.masked_where(badmask == 0, dh_meanarray)            
      NptsB           = M.masked_where(badmask == 0, Npts_array)              
                                                                              
      # Bug in 0.82: subplot(3,1,1) != subplot(311)                           
      ax1 = P.subplot(311)                                                      
      ll1 = P.plot(dday, hd_mean, 'ro', dday, hd_last, 'go')                    
      P.ylabel('Gyro Heading (deg)')                                            
      P.legend(('Mean', 'Last'),loc = 'upper left')                             
      P.setp( ax1.get_xticklabels(), visible=False)                             
      P.title(titlestring)                                                      
                                                                              
                                                                              
      ax2 = P.subplot(312, sharex=ax1)                                          
      ll2 = P.plot(dday, dh_mean, 'g',                                          
                 dday, dh_mean_maskedB, 'r.',                                 
                 dday, dh_std, 'go')                                          
      P.ylabel('Correction (deg)')                                              
      P.legend(('Mean(all)', 'Mean(persist)',  'StdDev'),loc = 'upper left')    
      ylow = P.median(dh_mean) - 2*P.median(dh_std)                               
      yhi = P.median(dh_mean) + 2*P.median(dh_std)                                
      ax2.set_ylim(min(0,ylow), yhi)                                          
      P.setp( ax2.get_xticklabels(), visible=False)                             
                                                                              
                                                                              
      ax3 = P.subplot(313, sharex=ax1)
      ll3 =P.plot(dday, n_used, 'go', dday, NptsB, 'ro')
      P.ylabel('N')
      P.legend(('N', 'bad;Used Persist'),loc = 'upper left')
      ax3.xaxis.set_major_formatter(P.ScalarFormatter(useOffset=False))
      P.xlabel('Decimal day')

      P.setp([ll1, ll2, ll3], markersize=4)

      if not P.iterable(dpi):
         dpi = [dpi]
         outfilename = [outfilename]
      for d, outf in zip(dpi, outfilename):
         P.savefig(outf, dpi=d)

except:
   plot_hcorr = False
   ## for debugging
   #  raise


## turn matplotlib off here
##plot_hcorr = False   


def get_hcorrstats(opts):

   if opts['skip_avg'] == 0:
      opts['numdays'] = 365
   else:
      opts['numdays'] = 1

   if opts['hcorr_inst'] == '':
      hcorrstr = 'no heading correction instrument selected'
   else:
      if opts['ping_headcorr']:
         hcorrstr = 'heading from %s applied to ensembles' % \
                 (opts['hcorr_inst'])
      elif opts['post_headcorr']:
         hcorrstr = 'heading from %s applied by "rotate rotate.tmp"' % \
                 (opts['hcorr_inst'])
      else:
         hcorrstr = 'heading from %s not applied' % (opts['hcorr_inst'])

   opts['commentstr'] = "comment = '%s';" % (hcorrstr)


   ## (1) generate statistics

   matstr = '''

   %% create  a text file with statistics from the ping_headcor correction
   %% call it from load, read hcorr.asc (post_headcorr) or ens_hcorr.asc
   %%


   %(diarystr)s
   %(displaystr)s

   yearbase = %(yearbase)d;

   %(commentstr)s

   hcorfile = '%(time_angle_base)s.asc';
   hcorr_inst = '%(hcorr_inst)s';

   if exist(hcorfile)
      print_hcorstats('yearbase', yearbase,...
                      'stats_ddrange', %(numdays)s,...
                      'hcorfile', hcorfile,...
                      'comment', comment,...
                      'outfile', 'hcorr_stats.txt');
   else
     fid = fopen('hcorr_stats.txt', 'w');
     fprintf(fid, 'no heading correction applied \\n')
     fclose(fid)
   end

''' % opts

   matfile = 'print_hcorrstats_tmp.m'
   mat = open(matfile, 'w')
   mat.write(matstr)
   mat.close()

   try:
      (runstr, output) = opts['M'].matlab(matfile[:-2])
   except:
      if opts['verbose']:
         logappend(opts['logfile'], ['CANNOT FINISH HCORR_STATS',])
      raise

   ## (2) now plot hcorrstats (try for matplotlib if possible; else use matlab

   if plot_hcorr:

      titlestring = "%(cruisename)s %(procdir)s" % opts
      hcorfile = '%(time_angle_base)s.asc' % opts
      plot_hcorr(hcorfile, n_ens=300, titlestring=titlestring,
                  outfilename=['./hcorr_plot.png', './hcorr_plot_thumb.png'],
                  dpi=[100, 30])

   else:
      matstr = '''

      %(diarystr)s
      %(displaystr)s

      hcorfile = '%(time_angle_base)s.asc';
      yearbase = %(yearbase)d;
      cruiseid = '%(cruisename)s';

      if exist(hcorfile)
      plot_hcorstats('comment', cruiseid,...
                     'yearbase', yearbase,...
                     'plot_ddrange', %(numdays)s,...
                     'plottype','fancy',...
                     'hcorfile', hcorfile,...
                     'gps_attname', '%(hcorr_inst)s',...
                     'cruiseid', cruiseid,...
                     'outfilebase', 'hcorr');
      end
   ''' % opts


      if opts['logfile']:
         logw = open(opts['logfile'], 'a')
         logw.write('about to run\n' + matstr + '\n')
         logw.close()

      matfile = 'plot_hcorrstats_tmp.m'
      mat = open(matfile, 'w')
      mat.write(matstr)
      mat.close()

      try:
         (runstr, output) = opts['M'].matlab(matfile[:-2])
      except:
         if opts['verbose']:
            logappend(opts['logfile'], ['CANNOT PLOT HCORR STATS',])
         raise



   del opts['commentstr']
   del opts['numdays']
## end hcor_stats

##################################################################


def xducerxy(opts):

   xducerxy_file = 'call_xducerxy.m'

   # ADD MATLAB COMMENTS IN PAIRS OF %%
   call_xducerstr = '''
      %%%% call_xducerxy being generated by quick_adcp.py
      %%  call the engine, xducerxy_core
      xducerxy_core('yearbase',%(yearbase)d,'dy',%(xducer_dy)d,...
             'dx',%(xducer_dx)d, 'enl', %(ens_len)d, ...
             'fixfile', '%(fixfile)s');
      ''' % opts

   print 'about to run xducerxy'
   print call_xducerstr

   xydf = open(xducerxy_file, 'w')
   xydf.write(call_xducerstr)
   xydf.close()

   try:
      (runstr, output) = opts['M'].matlab('call_xducerxy', verbose=1)
   except:
      if opts['verbose']:
         logappend(opts['logfile'], ['CANNOT FINISH CALL_XDUCERXY',])
      raise



## end xducerxy

##################################################################


