
import struct, sys, os, os.path
from ascfile import ascfile

class nb_ping:
   def __init__(self):
      self.offV = None  # sanity check flag
      self.data = None

   def decode_header(self, s):
      ''' Get the offsets from a header.'''
      (nb, nL, nV, nSW, nA, nPG, nS) = struct.unpack('>7H', s)
      self.offV = 14 + nL
      self.nV = nV
      self.offSW = self.offV + nV
      self.nSW = nSW
      self.offA = self.offSW + nSW
      self.nA = nA
      self.offPG = self.offA + nA
      self.nPG = nPG
      self.offS = self.offPG + nPG
      self.nS = nS

   def set_data(self, s):
      self.data = s
      if self.offV is None:
         self.decode_header(s[0:14])

   def get_V(self):
      return self.data[self.offV:(self.offV + self.nV)]

   def get_V_stream(self, s):
      '''Decode header and get vel; for streams where header may change.
      '''
      self.data = s
      self.decode_header(s[0:14])
      return self.data[self.offV:(self.offV + self.nV)]

class nbfile_log(ascfile):
   def __init__(self, filename):
      ascfile.__init__(self, filename)
      self.rawfilename = os.path.splitext(filename)[0] # chop off the .log
      self.rawfile = open(self.rawfilename, 'rb')

   def __del__(self):
      if self.rawfile:
         self.rawfile.close()
      ascfile.__del__(self)


   def next(self):
      #(udday, offset, nbytes, ens_num, inst_dday) = ascfile.next(self)
      fields = ascfile.next(self)
      (udday, offset, nbytes, ens_num) = fields[:4]
      self.rawfile.seek(int(offset))
      ens = self.rawfile.read(int(nbytes))
      #print int(ens_num)
      return ens


if __name__ == "__main__":
   from nbspeed import get_speed
   nb = nbfile_log(sys.argv[1])
   if len(sys.argv) > 2:
      H = float(sys.argv[2])
   else:
      H = 0.0
   nbp = nb_ping()

   count = 0
   for record in nb:
      nbp.set_data(record)
      v = nbp.get_V()
      (U, V, n) = get_speed(v, 0, 50, H)
      print U, V, n
      count += 1
      if count == 5:
         break

