'''Pathname operations beyond os.path.
'''
import os.path
def filename_base(pathname):
   head, tail = os.path.split(pathname)
   return tail.split('.')[0]


def corresponding_pathname(pathname, dir, ext):
   '''Generate a new path preserving a base filename.

   The base filename is the filename up to the first dot.

      pathname: path containing the original filename
      dir: directory path of new file
      ext: new extension, including the leading dot

   Note that ext differs from the output of os.path.splitext;
   the latter never includes a dot other than the leading dot.
   '''
   fnbase = filename_base(pathname)
   return os.path.join(dir, fnbase + ext)

def get_common_filelist(*filelists):
   '''Generate a list of file base names common to a set of lists.
   The input lists are the arguments.
   '''
   common_fl = list()
   for f in filelists[0]:
      fb0 = filename_base(f)
      common = True
      for flist in filelists[1:]:
         flist_b = [filename_base(ff) for ff in flist]
         if fb0 not in flist_b:
            common = False
            break
      if common:
         common_fl.append(fb0)
   return common_fl



