#!/usr/bin/env python

## JH adapted for UHDAS; 2004/09/15

'''
show long listing of last n files (ascii order):
raw logging                 -l     2
rbin directories            -r     2
gbin directories            -g     2
rootdir (optional) is the root directory containing raw/, rbin/ or gbin/

      usage: showlast.py  [-l num] [-r num] -g num] rootdir

'''
# JH 2003/07/01
import sys, os, string, re, glob, shutil, getopt, pwd, getopt
from commands import getoutput

def listfiles(basedirname, msglist, instname,  numfiles = 0, verbose = 0 ):
   # list raw logging
   startdir = os.getcwd()
   dirname = os.path.join(basedirname, instname)
   if os.path.exists(dirname):
      os.chdir(dirname)
      if msglist is None:
         globstr = '*'
         files = glob.glob(globstr)
         files.sort()
         lastfiles = files[-numfiles :]
         print '\n------- %s: -------\n' %  instname
         if (len(files) > 0):
            command = 'ls -l %s' % string.join(lastfiles,' ')
            print getoutput(command)
      else:
         for msg in msglist[instname]:
            globstr = '*.%s.*' % (msg)
            files = glob.glob(globstr)
            if len(files) > 0:
               files.sort()
               lastfiles = files[-numfiles :]
               print '\n------- %s: -------\n' %  instname
               command = 'ls -l %s' % string.join(lastfiles,' ')
               print getoutput(command)
   os.chdir(startdir)


# --- end of listfiles


def usage():
   print __doc__

# end of usage   ----------


###-----------------------------------------------------------


# get the options
try:
   options, args = getopt.getopt(sys.argv[1:], 'l:r:g:hv',
             ['num_log=', 'num_rbin=', 'num_gbin=', 'help', 'verbose' ])
except getopt.GetoptError:
   usage()

help = 0
verbose = 0
num_log = 0
num_rbin = 0
num_gbin = 0


for o, a in options:
   if o in ('-l', '--logging'):
      num_log = string.atoi(a)
   if o in ('-r', '--rbin'):
      num_rbin = string.atoi(a)
   if o in ('-g', '--gbin'):
      num_gbin = string.atoi(a)
   elif o in ('-h', '--help'):
      usage()
   elif o in ('-v', '--verbose'):
      verbose = 1

### main

if (num_log + num_rbin + num_gbin == 0):
   usage()

if len(args) == 1:
   rootdir = args[0]
   print 'root directory is %s\n' % (rootdir)
else:
   rootdir = '/home/adcp/cruise'

if not os.path.exists(rootdir):
   print 'raw root directory %s does not exist' % (rootdir)
   sys.exit()

rawdir      = os.path.join(rootdir, 'raw')
rbindir     = os.path.join(rootdir, 'rbin')
gbindirbase = os.path.join(rootdir, 'gbin')  # prefix instrument


instnames  = ['nb150', 'os38']
rbinnames  = ['gpsnav', 'gyro', 'ashtech', 'posmv', 'sndspd', 'simrad', 'seapath']
gbinnames  = ['gpsnav', 'gyro', 'ashtech', 'posmv', 'sndspd', 'simrad', 'seapath', 'time']


msglist = {}      ## initialize
# list possible suffixes
msglist['time']    = ('best', 'tim')
msglist['nb150']   = ('raw', 'log')
msglist['os38']    = ('raw', 'log')
msglist['gpsnav']   = ('gps',)
msglist['gyro']    = ('hdg',)
msglist['ashtech']    = ('gps', 'adu')
msglist['posmv']   = ('gps', 'pmv')
msglist['simrad']  = ('gps', 'pmv')
msglist['seapath'] = ('gps', 'gps_sea', 'psxn20', 'psxn23', 'sea')
msglist['sndspd']  = ('spd',)



# list raw logging

if (num_log > 0):
   print "===== raw data (%s) : last few files ==============" % (rawdir)
   for instname in instnames:
      listfiles(rawdir, None , instname,  num_log, verbose)
   print ''
   for serdir in rbinnames:
      listfiles(rawdir, None , serdir, num_log, verbose)
   print ''


if (num_rbin > 0):
   print "===== rbin data (%s) : last few files ==============" % (rbindir)
   for serdir in rbinnames:
      listfiles(rbindir, msglist, serdir, num_rbin, verbose)
   print ''


if (num_gbin > 0):
   for instname in instnames:
      print "===== gbin data (%s) : last few files ==============" % \
            (os.path.join(gbindirbase, instname),)
      for gbindir in gbinnames:
         listfiles(os.path.join(gbindirbase, instname),
                   msglist, gbindir, num_gbin, verbose)



