#!/usr/bin/env python
# EF 11/2000
#
#split up all the .aux file written by ue4 to have useful columns
#output is .hdg with the same prefix as the aux file had
#
#
#290.309792 207 3564 199579.0 24.60 2.40 1.13 -3.79 0 1.761 4.877 -0.076 0.114 10
#290.309815 208 4455 199581.0 24.19 2.82 2.04 -2.83 0 1.872 4.605 -0.959 0.082 10
#290.309826 209 5346 199582.0 23.89 2.46 0.00 0.00 1 1.902 4.576 0.028 0.051 10
#
#  JH documentation added



import sys, os, os.path, glob, string, time

auxlist = glob.glob('*.aux')


os.environ['TZ'] = 'GMT'  # so mktime will work with gmt
yearbase = 2001
base_sec = time.mktime((yearbase, 1, 1, 0, 0, 0, -1, 1, 0))

def degmin_to_dec(dm, NSEW):
   #sign
   sign = {'N': 1, 'E': 1, 'S': -1, 'W': -1}
   #the number (2222.2108 N is 22deg, 22.2108 min N)
   dm = float(dm)
   # degrees (whole number)
   dg = int(dm/100)
   # decimal minutes
   mn = dm - dg * 100
   # decimal lon or lat
   dec = sign[NSEW] * (dg + mn / 60.0)
   return dec

def hms_to_dday(hms, dday_estimate):
   #first two digits
   hh = float(hms[0:2])
   #second two digits
   mm = float(hms[2:4])
   #third two digits
   ss = float(hms[4:])
   # dday fraction of 1
   dday_frac = hh / 24.0 + mm / 1440.0 + ss / 86400.0
   # actual dday given the estimate
   dday = int(dday_estimate) + dday_frac
   if dday - dday_estimate > 0.5:
      dday = dday - 1.0
   if dday - dday_estimate < -0.5:
      dday = dday + 1.0
   return dday

def gga_to_txy(gga, dday_estimate):
   # Could add checksum check, of course.
   #skip, hrs_min_ssec, lat_decimal_latmin, NS, lon_decimal_lonmin, EW, the_rest
   (s, hms, y, NS, x, EW, rem) = string.split(gga, ',', 6)
   tt = hms_to_dday(hms, dday_estimate)
   xx = degmin_to_dec(x, EW)
   yy = degmin_to_dec(y, NS)
   return (tt, xx, yy)

def unixs_to_dday(s):
   dday = (float(s) - base_sec) / 86400.0
   return dday

scn = open('aux_.scn', 'w')

# auxlist = auxlist[0:1]

for auxname in auxlist:
   #for a file named "16196126.aux", base will be "16196126"
   base = os.path.splitext(auxname)[0]
   # open a file with the same prefix; suffix "hdg" for writing
   hdg = open(base + '.hdg', 'w')
   # open the aux file; for reading
   aux = open(auxname, 'r')

   # ?? no idea what this is
   scn.write(base)
   # start_date comes from the first day of the file
   start_date = aux.readline()
   # skip the next line
   junk = aux.readline()

   # First time estimate: (may not need it)
   # (this comes from the filename base)
   dday = unixs_to_dday(float(os.path.split(base)[1]) * 60)
   first_gga = 1
   #pairs of lines sare written with GGA first.
   while 1:
      #read a record
      rec = aux.readline()
      # skip it if it's blank
      if rec == '':
         break
      #if it's a $GPGGA match
      if rec[0:7] == '$GPGGA,':
         gga_rec = rec
         #it's the first GGA record.  get txy and write to .scn
         if first_gga:
            (t, x, y) = gga_to_txy(gga_rec, dday)
            scn.write(' %12.6f %12.6f %12.6f' % (t, x, y))
            first_gga = 0
            #don't do anything more
         continue  # For now, leave most GPGGA reading to nmea_gps

      fields = string.split(rec)
      #it's not a GGA record.  get dday from first field (unix seconds)
      dday = unixs_to_dday(fields[0])
      if len(fields) == 16:
         #write RDIday  and the [4:end]
         hdg.write('%12.6f ' % dday)
         hdg.write(fields[1] + ' ' + string.join(fields[4:]) + '\n')


   # end of loop as shown by indentation!!      
   aux.close()
   hdg.close()

   #we're out of the loop: now write down the last gga txy
   if first_gga == 0:
      (t, x, y) = gga_to_txy(gga_rec, dday)
      scn.write(' %12.6f %12.6f %12.6f\n' % (t, x, y))

scn.close()


