#!/usr/bin/env python

## JH adapted for UHDAS; 2004/09/15

'''
show long listing of last n files (ascii order), or last few lines of files
ascii output                -a     2
raw logging                 -l     2
rbin directories            -r     2
gbin directories            -g     2
rootdir (optional) is the root directory containing raw/, rbin/ or gbin/

      usage: showlast.py  [-a num]  [-l num] [-r num] -g num] rootdir

'''
# JH 2003/07/01
import sys, os, string, re, glob, shutil, getopt, pwd, getopt
from commands import getoutput

def tailfiles(basedirname, msglist, instname,  numlines = 0, verbose = 0 ):
   # tail of ascii files (number refers to number of lines of each type)
   startdir = os.getcwd()
   dirname = os.path.join(basedirname, instname)
   if os.path.exists(dirname):
      os.chdir(dirname)
      if msglist == 'raw.log': # raw
         globstr = '*%s' % (msglist,)
         files = glob.glob(globstr)
         files.sort()
         lastfile = files[-1]
         print '\n------- %s: -------\n' %  instname
         command = 'tail -%d %s' % (numlines, lastfile)
         print command
         print getoutput(command)
      else: # ascii
         globstr = '*'
         files = glob.glob(globstr)
         files.sort()
         lastfile = files[-1]
         print '\n------- %s: -------\n' %  instname
         command = 'tail -%d %s' % (2*numlines, lastfile)
         print getoutput(command)

   os.chdir(startdir)

def listfiles(basedirname, msglist, instname,  numfiles = 0, verbose = 0 ):
   # list logged files (number refers to number number of files of each type)
   startdir = os.getcwd()
   dirname = os.path.join(basedirname, instname)
   if os.path.exists(dirname):
      os.chdir(dirname)
      if msglist is None:
         globstr = '*'
         files = glob.glob(globstr)
         files.sort()
         lastfiles = files[-numfiles :]
         print '\n------- %s: -------\n' %  instname
         if (len(files) > 0):
            command = 'ls -l %s' % string.join(lastfiles,' ')
            print getoutput(command)
      elif msglist is 'raw':
         rawext = ['raw', 'log', 'bin', 'err']
         print '\n------- %s: -------\n' %  instname
         for ext in rawext:
            globstr = '*.%s' % (ext,)
            files = glob.glob(globstr)
            files.sort()
            lastfiles = files[-numfiles :]
            if (len(files) > 0):
               command = 'ls -l %s' % string.join(lastfiles,' ')
               print getoutput(command)
               print ' '
      else:
         filenames = os.listdir('./')
         msglist = []
         for f in filenames:
            fields = f.split('.')
            if len(fields) == 3:
                msg = fields[1]
                if msg not in msglist:
                    msglist.append(msg)
         for msg in msglist:
            globstr = '*.%s.*' % (msg)
            files = glob.glob(globstr)
            if len(files) > 0:
               files.sort()
               lastfiles = files[-numfiles :]
               print '\n------- %s: -------\n' %  instname
               command = 'ls -l %s' % string.join(lastfiles,' ')
               print getoutput(command)
   os.chdir(startdir)


# --- end of listfiles


def usage():
   print __doc__

# end of usage   ----------


###-----------------------------------------------------------


# get the options
try:
   options, args = getopt.getopt(sys.argv[1:], 'a:l:r:g:hv',
      ['num_asc=', 'num_log=', 'num_rbin=', 'num_gbin=', 'help', 'verbose' ])
except getopt.GetoptError:
   usage()

help = 0
verbose = 0
num_asc = 0
num_log = 0
num_rbin = 0
num_gbin = 0


for o, a in options:
   if o in ('-l', '--logging'):
      num_log = string.atoi(a)
   if o in ('-a', '--ascii'):
      num_asc = string.atoi(a)
   if o in ('-r', '--rbin'):
      num_rbin = string.atoi(a)
   if o in ('-g', '--gbin'):
      num_gbin = string.atoi(a)
   elif o in ('-h', '--help'):
      usage()
   elif o in ('-v', '--verbose'):
      verbose = 1

### main

if (num_asc + num_log + num_rbin + num_gbin== 0):
   usage()

if len(args) == 1:
   rootdir = args[0]
   print 'root directory is %s\n' % (rootdir)
else:
   rootdir = '/home/adcp/cruise'

if not os.path.exists(rootdir):
   print 'raw root directory %s does not exist' % (rootdir)
   sys.exit()

rawdir      = os.path.join(rootdir, 'raw')
rbindir     = os.path.join(rootdir, 'rbin')
gbindirbase = os.path.join(rootdir, 'gbin')  # prefix instrument

instnames = os.listdir(gbindirbase)
if len(instnames) == 0:
   print 'no gbin data yet; cannot determine instruments\n'
   sys.exit()
rbinnames = os.listdir(rbindir)
gbinnames = os.listdir(os.path.join(gbindirbase, instnames[0]))


# list raw logging

if (num_asc > 0):
   print "===== ascii data (%s) : last few lines ==============" % (rawdir)
   for instname in instnames:
      if os.path.exists(os.path.join(rootdir, rawdir, instname)):
         tailfiles(rawdir, 'raw.log' , instname,  num_asc, verbose)
         print ''
   for serdir in rbinnames:
      if os.path.exists(os.path.join(rootdir, rawdir, serdir)):
         tailfiles(rawdir, None , serdir, num_asc, verbose)
   print ''


if (num_log > 0):
   print "===== raw data (%s) : last few files ==============" % (rawdir)
   for instname in instnames:
      if os.path.exists(os.path.join(rootdir, rawdir, instname)):
         listfiles(rawdir, 'raw' , instname,  num_log, verbose)
         print ''
   for serdir in rbinnames:
      if os.path.exists(os.path.join(rootdir, rawdir, serdir)):
         listfiles(rawdir, None , serdir, num_log, verbose)
   print ''


if (num_rbin > 0):
   print "===== rbin data (%s) : last few files ==============" % (rbindir)
   for serdir in rbinnames:
      if os.path.exists(os.path.join(rootdir, rbindir, serdir)):
         listfiles(rbindir, 'rbin', serdir, num_rbin, verbose)
   print ''


if (num_gbin > 0):
   for instname in instnames:
      if os.path.exists(os.path.join(gbindirbase, instname)):
         print "===== gbin data (%s) : last few files ==============" % \
               (os.path.join(gbindirbase, instname),)
         for gbindir in gbinnames:
            if os.path.exists(os.path.join(gbindirbase, instname, gbindir)):
               listfiles(os.path.join(gbindirbase, instname),
                         'gbin', gbindir, num_gbin, verbose)



