function newtimes = fix_timestamps(bindata, efid, ploterr);
% newtimes = fix_timestamps(bindata, efid, ploterr);
% something can make two writes take place with same time stamp
% move the time stamp if it has this signature
% bindata: nrows x ntimes; first row is used as timestamp to fix
% efid: fiel id to write errors to, if nonzero
% ploterr: (plot busy cpu instances)

ddd = bindata(1,:);

if isemptys(find(diff(ddd)) <= 0)
   newtimes = ddd;
   return
end

if nargin == 2
   if efid ~= 0
      writerr = 1;
   else
      writerr = 0;
   end
else
   writerr = 0;
end

if nargin == 3
   ploterr = 1;
else
   ploterr = 0;
end
  

%% need to deal with first time being repeated
repi = find(ddd == ddd(1));
if length(repi) > 1       %should match the first value: are there others?
   if max(diff(repi)) > 1
      fprintf('fix_timestamps: nonadjacent times match the first time\n')
      fprintf('this is unusual and very bad.\n')
      newtimes=[];
      return
   end
   repi(1)=[];   
   index = [1; repi(end)+1];
   ddd(repi) = table1([index ddd(index)'], repi);
   fprintf('count = 0, ---->  fixing %d  busy cpu instances\n',...
           length(repi))
   if (writerr == 1)
      fprintf(efid, 'count = 0, ---->  fixing %d  busy cpu instances\n',...
              length(repi))
   end
   if ploterr == 1
      figure
      plot(ddd,'.')
      hold on
      plot(repi, ddd(repi),'ko')
   end
end

iters = 0;

if (min(diff(ddd)) <= 0)
   busycpu = 1;
else
   busycpu = 0;
end


while busycpu
   origddd = ddd;
   iters = iters+1;
   dddd = diff(ddd)*86400;
   [junk1, junk2, junk3, med] = mstdgap(dddd);
   s_threshold = .1*med;
   si_1 = find(dddd < s_threshold);              % too close together
   si_2 = find(diff(dddd) < -s_threshold*2);     % too far apart
   si_0 = si_1(ismember(si_1, si_2+1)) -  1;
   
   if (isemptys(si_0))
      if si_1(1) == 1
         ddd(2) = (ddd(1) + ddd(3))/2;
      else
         ddd(si_1) = (ddd(si_1 - 1) + ddd(si_1 + 1))/2;
         fprintf('count = %d, warning: fix_time faked it\n', iters)
      end      
   else
      ddd(si_0 + 1) = (ddd(si_0) + ddd(si_0 + 2))/2;
   end
   
   if (length(si_0) > 0)
      
      fprintf('count = %d, ---->  fixing %d  busy cpu instances\n', ...
              iters, length(si_0))
      if (writerr == 1)
         fprintf(efid, 'count = %d, ---->  fixing %d  busy cpu instances\n', ...
                 iters, length(si_0));
      end
      if (iters > 4)
         newtimes=[];
         return
      end
   end


   if ploterr == 1
      figure
      colors = ['rgbk'];
      plot(86400*diff(origddd),'k+');hold on
      plot(86400*diff(ddd),'.');hold on
      plot(86400*diff(ddd),'color', colors(rem(iters,4) + 1))
      paused
   end
   
   if (min(diff(ddd)) > 0)
      break
   end
   
end


newtimes = ddd;

