function [inst_besttime, gga_besttime] = ...
    get_besttime(instrument_dday, logdata, cfg, trinfo, errfile)
% [inst_besttime, gga_besttime] = ...
%             get_besttime(instrument_dday, logdata, cfg, trinfo, errfile)
%
% use raw times, logger times, and gga times to get a best time grid
% called by mk_gbin
% trinfo is from tr2ilist
% data is from read(instclass, rawfile)
% logdata is from loading the corresponding log file
%

% JH 2004/09/22 This was Really Ugly and had alot of legacy code.
% rewriting to 
%  - provoke error message if log file is not 5-columns with ensemble number as #4
%  - use unscootched instrument corrected to GGA as timestamp
%  - ignore instrument timestamps (except to count how many)
%  - assume time is at worst nondecreasing (may be nonmonotonic)


gga_besttime = [];
inst_besttime = [];
logger_dday = logdata(:,1)';            % PC wrote $UNIXD

%get the logger time info from the gps files
logger_range = [logger_dday(1) logger_dday(end)];
%bindata is [logger_dday, fullggadday, lon, lat]


[bindata, binstruct] = tr2ilist(trinfo, 'timerange', logger_range);
if isemptys(bindata)
   fprintf('get_besttime: no data found in this time range\n')
   fprintf('check rbin yearbase??')
   return
end

u_dday = bindata(r_row(binstruct,'u_dday'),:);


if (isemptys(bindata))
   fprintf('no data found in meas_logger range %f-%f\n',...
           logger_range(1), logger_range(2))
   berrfid = fopen(errfile,'a');
   if berrfid == -1
      fprintf('could not open log file %s\n', errfile)
   else
      fprintf(berrfid, 'note added at at %s\n', nowstr);
      fprintf(berrfid,'no data found in meas_logger range %f-%f\n',...
              logger_range(1), logger_range(2));
      fclose(berrfid);
   end
   return
end


%% (1)  make sure gga fraction can be interpolated over
if ismember('dday', binstruct.rows)
   dday = bindata(r_row(binstruct, 'dday'),:);
   dt = rem(u_dday - dday + .5,1) - .5;
elseif ismember('dday_frac', binstruct.rows)
   tt = bindata(r_row(binstruct, 'dday_frac'),:);
   tt = rem(bindata(2,:),1);    %% make it between -1 and 1 
   tt = rem(tt + 1, 1);         %% make it between 0 and 1
   dt = rem(u_dday - tt + .5,1) - .5;
   dday = u_dday - dt;
else
   fprintf('ERROR: no gga message in this serial %s data.  \n', ...
           rstruct.messagename)
   fprintf(' choose  another data stream for besttime\n')
   besttime = [];
   return;
end

logdays = logdata(:,[1 5])';
logstruct.rows = {'u_dday', 'adcp_dday'};
%% use this for future calls to tablelegs
gga_besttime = [bindata(1,:); dday; 86400*dt]; 

%test it
%test it
if isemptys(gga_besttime)
   fprintf('best gga times empty.  skipping\n')
   efid = fopen(errfile,'a');
   fprintf(efid,'best gga times times.  skipping\n');
   fclose(efid);
   gga_best_time = [];
   inst_besttime = [];
   return
elseif  min(diff(dday)) <= 0 
   fprintf('nonmonotonic gga times.  skipping\n')
   efid = fopen(errfile,'a');
   fprintf(efid,'nonmonotonic gga times.  skipping\n');
   fclose(efid);
   gga_best_time = [];
   inst_besttime = [];
   return
end


logger_utc = tablelegs(logdays, logstruct, gga_besttime, errfile);
inst_besttime =  [logdata(:,1)'; logger_utc ; 86400*(logger_dday - logger_utc)];

if cfg.verbose

   figure(1);clf
   set(gcf,'defaulttextinterpreter','none')
   fprintf('\nPC time range, adcp pings :\n %s - %s\n',...
           to_date(cfg.yearbase,logger_range(1)),...
           to_date(cfg.yearbase,logger_range(2)))
   fprintf('PC time range, adcp pings :\n %10.7f - %10.7f\n\n',...
           logger_range(1), logger_range(2))
   subplot(211)
   plot(logger_utc, logdata(:,1)','ro')
   hold on
   plot(dday, bindata(1,:),'g.')
   title('gga(green), adcp (red) corrected to UTC')

   subplot(212)
   plot(dday,  86400*dt,'k')
   title('correction to PC clock: pc_seconds_minus_utc')
   
   drawnow
   
   paused(1)
end


%test it
if isemptys(logger_utc)
   fprintf('logger_utc empty.  skipping\n')
   efid = fopen(errfile,'a');
   fprintf(efid,'logger_utc times.  skipping\n');
   fclose(efid);
   gga_best_time = [];
   inst_besttime = [];
   return
end

