function [trinfo, gbinstruct, qcmask] = gtr2ilist(arg1, varargin)
% TWO modes: (1) % trinfo = gtr2ilist(arg1, varargin)
%                  return cellaray with file and time info
%
%            (2)  [trinfo, gbinstruct, qcmask] = gtr2ilist(arg1, varargin)
%                return data and structure from rbin read; 
%                data have qc applied or not; mask has 1 for bad values
% [trinfo, gbinstruct, qcmask] = gtr2ilist(arg1, varargin)
% (1)
% trinfo = gtr2ilist(filelist, varargin)
% input: filelist from dirs('*.tim.gbin','fullfile',1)
% returns a cellarray with each row containing:
%    filename, [startdd enddd], [starti endi], numpts_in_tr
%    (for files that need to be read)
%    [startdd enddd] are decimal day ranges of bestdday from file
%    [starti endi] are index range from this file for time range 
%
% (2)
% subset = gtr2ilist(trinfo, varargin)
% input: trinfo is from a previous call of gtr2ilist
%        Uses info from trinfo to output a subset of the data
%          that is inside timerange
%
%
% varargin
%
%   name                    what                        default
%  ---------          --------------                --------------
%   'timerange'       dday time range of file             'all'  
%                  string 'all' (returns all)          
%                     [dday1 dday2]
%   
%   'step'           take every N values (only call #2))   1
%   'sort'           how to sort rows         
%                   'none' (as is)                     'none'
%                   'time' (startdd)
%  'verbose'         [0,1] print messages                0
%
%% (do "help r_row" to access rows using structure)



% jh

cfg.sort = 'none';
cfg.timerange = 'all';
cfg.rowname = 'bestdday';
cfg.verbose = 0;
cfg.step = 1;


cfg = fillstruct(cfg, varargin, 0);

trinfo = [];
gbinstruct = [];
qcmask = [];

if isemptys(arg1)
   return
end


if  ~isstruct(arg1) %sending in a filelist
   fprintf('%s requires a filelist as input\n', mfilename)
   trinfo = [];
   return
end

if isemptys(arg1)
   fprintf('%s returning: (filelist was empty)\n', mfilename)
   return
end

badi = [];
filelist = arg1;

for ii=1:length(filelist)
   filename = filelist(ii).name;
   if (~exist(filename,'file'))
      badi = [badi; ii];
      fprintf('could not find file %s\n', filename)
   end
end
filelist(badi)=[];


for ifile=1:length(filelist)
   [tmp, gbinstruct]=read_bin(filelist(ifile).name,'last',1);
   if (~isemptys(tmp))
      fclose(gbinstruct.fid);
      continue
   end
end


rownum = find(strcmp(gbinstruct.rows, cfg.rowname));
if (isemptys(rownum))
   fprintf('\n\n%s error: you chose rowname %s\n', mfilename, cfg.rowname)
   fprintf('must choose rowname from among these\n')
   gbinstruct.rows
   trinfo = [];
   gbinstruct = [];
   return
end

badi = [];
% go through and fill the cellarray; delete unused files later
trinfo = cell(length(filelist),4);
for filei = 1:length(filelist)
   filename = filelist(filei).name;
   data = read_bin(filename);         
   
   if (isemptys(data))
      trinfo{filei,1} = filename;
      trinfo{filei,2} = [];
      trinfo{filei,3}=  [];
      trinfo{filei,4}=  [];
      badi = [badi; filei];
   else

      dday = data(rownum,:);
            
      if (isstr(cfg.timerange))
         trange = [min(dday), max(dday)];
      else
         trange = cfg.timerange;
      end
      
      %check whether the file is completely outside the timerange
      if (min(dday) > trange(2))               % data(1) after trange
         badi = [badi; filei];
      elseif (max(dday) < trange(1))
         badi = [badi; filei];
      else
         ii = find(...
            (dday >= trange(1)) & (dday <= trange(2)));
         trinfo{filei,1} = filename;
         trinfo{filei,2} = [min(dday) max(dday) ];
         trinfo{filei,3}=  [min(ii) max(ii)];
         trinfo{filei,4}=  [max(ii) - min(ii)+1];            
      end
   end
end

if (~isemptys(badi))
   trinfo(badi,:) = [];
end

      
      



