function smallfilelist = guess_rfilelist(cfg, files, inst_info, bintype)
% smallfilelist = guess_rfilelist(arg1, files, inst_info, bintype)
% arg is a structure with fields 
%           rbindir (or gbindir)
%           toofarsecs
% -or-  the name of the rbin root directory (or gbin root directory)
%       (if not using a structure, toofarsecs is 30)
%
% files is full of rawfilebase or could be a ddrange
% inst_info should be 1x2 instrument directory and suffix 
%         eg. {'adu2', 'pos'} or {'time','tim'}
% bintype is 'rbin' or 'gbin'
%
%assuming files are called prefix_ddd_seconds, fixed character format
% use rfilebase to guess what rfiles to use
%
% or if a ddrange comes in, make the same assumption about file
% names and guess which ones to use
%
%
% assumes the path information is in cfg in the form of cfg.rbindir
% assumes an acceptable time separation in the form of cfg.toofarsecs

smallfilelist = [];

% JH 2003, 7/17 -- needs to be cleaned up.
% called by mk_gbin and get_rndata


if strcmp(bintype, 'rbin')
   if (isstruct(cfg)) %cfg.rbindir
      bindir = cfg.rbindir;
   else
      bindir = cfg;  % no structure, just the file name
   end
elseif strcmp(bintype, 'gbin')
   if (isstruct(cfg)) %cfg.rbindir
      bindir = cfg.gbindir;
   else
      bindir = cfg;
   end
else
   error('must set "bintype" oas ''rbin'' or ''gbin''')
end


if (isstruct(cfg))
   if isfield(cfg, 'toofarsecs')
      toofarsecs = cfg.toofarsecs;
   else
      toofarsecs = 30;
   end      
else
   toofarsecs = 30;
end


if isstruct(files)
   if (length(find(files(1).name == filesep)) ~= 0)
      %these are full file names, not relative file names
      % manually strip off everything prior to the last filesep
      for filei = 1:length(files)
         sepi = find(files(filei).name == filesep);
         if (~isemptys(sepi))
            files(filei).name(1:sepi(end)) = [];
         end
      end
   end

   underscores = find(files(1).name == '_');
   prefix = files(1).name(1:underscores(1)-1);
   dayi = underscores(1)+1 : underscores(2)-1;
   seci =  underscores(2)+1 : underscores(2)+5;
   alldays = ones(length(files),1);
   allsecs = ones(length(files),1);
   for ii=1:length(alldays);
      alldays(ii) = str2num(files(ii).name(dayi));
      allsecs(ii) = str2num(files(ii).name(seci));
   end
   allddays = alldays + allsecs/86400;  %dday of raw files
      
   istimerange = 0;

else
   allddays = files; %it's a dday range
   istimerange = 1;
   timerange = files;
end


%now list all the raw files of one type for a given instrument
% use the times of the raw files to get the nearest rbin files
% return rfile filelist

fileglob = sprintf('%s/*.%s.%s', ...
                   fullfile(bindir, inst_info{1}), ...
                   inst_info{2}, bintype);
rfilelist = dirs(fileglob,'fullfile',1);


if isemptys(rfilelist)
   fprintf('%s:\n', mfilename)
   fprintf('no %s files found\n', bindir);
   return
end
   
if length(rfilelist) <= 3
   fprintf('only a few files: returning all\n')
   smallfilelist = rfilelist;
else
   
   if (~exist('underscores', 'var'))
      tryfile = rfilelist(1);
      sepi = find(tryfile.name == filesep);
      if (~isemptys(sepi))
         tryfile.name(1:sepi(end)) = [];
      end
      dots = find(tryfile.name == '.');
      barename = tryfile.name(1:dots(1)-1);
      underscores = find(barename == '_');
      dayi = underscores(1)+1 : underscores(2)-1;
      seci =  underscores(2)+1 : length(barename);
      prefix = barename(1:underscores(1)-1);
   end
   
   ralldays = ones(length(rfilelist),1);
   rallsecs = ones(length(rfilelist),1);
   for ii=1:length(ralldays);
      [rpath, rfile] = fileparts(rfilelist(ii).name);
      ralldays(ii) = str2num(rfile(dayi));
      rallsecs(ii) = str2num(rfile(seci));
   end
 
   rallddays = ralldays + rallsecs/86400;  %dday of raw files
   okrfiles = ones(size(rfilelist));
   
   if (length(rallddays) == 1)
      smallfilelist = rfileslist;
   else
      if istimerange
         
         dt = max(2/24, median(diff(rallddays)));
         % get everything in between best files (with one extra outside)
         okrfiles( (rallddays < timerange(1)-dt) | ...
                   (rallddays > timerange(2)+dt)) = 0;
      else
         for rfilei = 1:length(okrfiles)
            minsecs = min(abs(rallddays(rfilei) - allddays));
            if (minsecs(1) > toofarsecs/86400) %there is no matching raw file
               okrfiles(rfilei) = 0;
            end
         end
      end
   end
   
   smallfilelist = rfilelist(find(okrfiles));
end


if (isemptys(smallfilelist))
   fprintf('%s:\n', mfilename)
   fprintf('no %s files found within %s minutes of requested files\n',...
           bindir , toofarsecs/(24*60))
   fprintf('filename time range:    %7.5f to %7.5f\n', ...
           rallddays(1), rallddays(end))
   if (exist('timerange','var'))
      fprintf('time range requested:   %7.5f to %7.5f\n', ...
              timerange(1), timerange(2))
   else
      fprintf('time range of listed files:   %7.5f to %7.5f\n', ...
              allddays(1), allddays(2))
   end
end



