function mk_gbin(instclass, yearbase, varargin)
% mk_gbin(instclass, yearbase, varargin)
% mk_gbin makes gbin files 
%           time stamp: "time/*.tim"
%           best (position, attitude): "time/*.best"
%           for each instrument (eg. seapath1/*.sea, pcode/*.gps ...)
%
% do this from adcpr, or where raw/ and rbin/ and gbin/ are 
% works on a list of raw files; will not overwrite files unless set 'redo'=1
%
% 
%% varargin                  default 
%  
%  log_yearbase             : logging yearbase (default: same as yearbase)
%
%                           ==> choose files this way:
%  startfilei               : start with this filenum (partway through the list)
%  rawglob                      '*.raw'  (eg. could choose specific days)
%
%                           ==> or this way
%  filelist                 : cellarray of base filenames
% 
%                           ==> info about files to write
%
%  redo                     :default:0 (do not rewrite gbin files
%                           :       else do rewrite time and best
%
%  mkgbin                   : cellarray of instruments make gbin files for
%                           : default is to make no extra files
%                           : same format as msg_info (use rows from msg_info)
%
%                           ==>  info about instruments and messages  
%                           ==> type "mk_gbin" for more help <==
%
%  msg_info                 : structure of cellarrays with info about 
%                           :    where to find logged data  REQUIRED
%
%  best                     : structure of cellarrays with rules:    REQUIRED
%
%                           : other arguments 
%  instname                 : (must specify)
%  rawdir                   : default:    'raw';
%  rbindir                  : default:    'rbin';
%  gbindir                  : default:    'gbin';
%  logfpath                 : default:    'log'
% gbinlogfile               : default:    'mk_gbin.log'
%  verbose                  : default:    0           (don't say stuff)
%  debug                    : default:    0           (don't drop into keyboard)

%
%%% JH 2002/08/18: 
%%% vg0202
%%% start in /home/jules/vg0202
%%% "rbin" and "gbin" directories and subdirectories already exist
%%%  rbin files already exist
%%%  new version!! for sontek and Kilomoana logging
%%%  new (improved?) speedier? replace mk_gbin, [xxx,time,best]2gbin
%%% revise mk_gbin: remove multiple rules, streamline besttime generation

cfg.log_yearbase = [];
cfg.startfilei  = 1;
cfg.filelist = [];
cfg.redo = 0;
cfg.mkgbin = [];
cfg.instname = '';
cfg.rawdir    =    'raw';
cfg.rbindir   =    'rbin';
cfg.gbindir   =    'gbin';
cfg.logfpath  =    'log';
cfg.gbinlogfile = 'mk_gbin.log';
cfg.msg_info  = [];
cfg.best      =   [];              %structure of cellarrays for best2gbin
cfg.verbose  = 0;
cfg.debug    = 0;

cfg.help = [];

cfg = fillstruct(cfg, varargin, 0);

if isemptys(cfg.instname)
   fprintf('must specify instrument name\n')
   return
end

%update with whatever fields were put in varargin 'best'
if (nargin == 0)
   besthelp
   fprintf('\n\nmust specify both ''best'' and ''msg_info'' \n')
   fprintf('type ''help mk_gbin'' for more help\n')
   return
end


%% getting time from gps instrument
gps_rule = getfield(cfg.best,'gps_rule');
%% getting position from same source as time
posinst = gps_rule{1};  

if (isemptys(cfg.best))
   help(mfilename)
   error('must specify rules with ''best''')
else
   cfg = fillstruct(cfg, cfg.best, 1);
end
cfg = rmfield(cfg,'best');


if (isemptys(cfg.msg_info))
   help(mfilename)
   error('must specify raw logging file info with ''msg_info''')
end   

% other constants
cfg.rawfpath  =    fullfile(cfg.rawdir,cfg.instname);
cfg.timeoutpath = fullfile(cfg.gbindir,'time');
cfg.time_fext = 'gps';             % get info from gps files
cfg.rawglob = '*.raw';
cfg.toofarsecs = 2*60*60 + 10;   % 2hrs,10sec (scope of search for rbinfiles)


% for later, get the subset being used for gbin files; 
if (isemptys(cfg.mkgbin))
   gbinnums= [];
else
   gbinnums= find(ismember(cfg.msg_info(:,1), cfg.mkgbin));
end

if isemptys(gbinnums)
   gbin_info = [];
else
   gbin_info = cfg.msg_info(gbinnums,:)
end


%%-- error checking --:

if ((nargin ==1) | (nargin == 2))
   help mk_gbin
   error('must set instrument class and yearbase')
end
cfg.yearbase= yearbase;


if (~isemptys(cfg.help))
   besthelp
   return
end


%%% check if raw directory exist
checkdirs = {cfg.rawdir, cfg.rawfpath};
for dirnum = 1:length(checkdirs)
   if (~exist(checkdirs{dirnum}, 'dir'))
      fprintf('directory %s does not exist\n', checkdirs{dirnum})
      return
   end
end


fprintf('checking gbin dirs from %s\n', pwd)
%%% check if rbin, gbin, log directories exist
if isemptys(cfg.logfpath)
   mkdirs = {cfg.rbindir, cfg.gbindir};
else
   mkdirs = {cfg.rbindir, cfg.gbindir, cfg.logfpath};
end
for dirnum = 1:length(mkdirs)
   dirname = mkdirs{dirnum};
   if (~exist(dirname, 'dir'))
      fprintf('%s directory does not exist, making directory.\n',dirname)
      eval(sprintf('!mkdir %s', dirname))
   end
end

%%% check if output directories exist
mkdirs = unique(cfg.msg_info(:,1));
mkdirs{length(mkdirs)+1} = 'time';
for dirnum = 1:length(mkdirs)
   dirname = fullfile(cfg.gbindir, mkdirs{dirnum});
   if (~exist(dirname, 'dir'))
      fprintf('%s directory does not exist, making directory:\n%s\n',...
              mkdirs{dirnum}, dirname)
      eval(sprintf('!mkdir %s', dirname))
   end
end


% look for input files:
if (isemptys(cfg.filelist))
   files = dirs(fullfile(cfg.rawfpath, cfg.rawglob));
   fprintf('found %d files ', length(files))
   fprintf('starting at file number %d\n', cfg.startfilei)
   if (cfg.startfilei > 1)
      files(1:cfg.startfilei) = []
   end
else
   if (iscell(cfg.filelist))
      files = cell2struct(cfg.filelist, 'name', 1);
   elseif isstruct(cfg.filelist)
      files = cfg.filelist;
   end
end
   
if (length(files) > 0)
   fprintf('... last file is  %s', files(end).name)
end
fprintf('\n')
paused(1)


% get timerange info for all files listed, just once for 
%   each instrument that  might be called

for insti = 1:length(cfg.msg_info(:,1))
   guessed_rfiles = guess_rfilelist(cfg, files, cfg.msg_info(insti,:),'rbin');
   fprintf('getting time ranges from %d rfiles for %s/*.%s\n', ...
           length(guessed_rfiles), ...
           cfg.msg_info{insti,1},   cfg.msg_info{insti,2})

   if (isemptys(guessed_rfiles))
      fprintf('no file within %d seconds\n\n', cfg.toofarsecs)
      tmpcfg = cfg; tmpcfg.toofarsecs = 86500;
      fprintf('trying a larger interval (%d secs):\n', tmpcfg.toofarsecs)
      guessed_rfiles = guess_rfilelist(tmpcfg, files, ...
                                       cfg.msg_info(insti,:),'rbin');
      fprintf('getting time ranges from %d rfiles for %s/*.%s\n', ...
              length(guessed_rfiles), ...
              cfg.msg_info{insti,1},   cfg.msg_info{insti,2})
   end              
   if (isemptys(guessed_rfiles))
      fprintf('WARNING: still no files found\n')
      fprintf(' ---> check whether rfiles are up to date.\n')
      fprintf(' ---> check whether instruments are properly configured.\n')

   end

   trinfoc{insti} =  tr2ilist(guessed_rfiles);
end

%set up structure (add names from cfg.rules to lon and lat)
%number of columns beyond the time stamp
numbest = 2*find(strcmp(cfg.rules, 'gps')) + ...   %lon and lat
          length(find(strcmp(cfg.rules, 'heading'))) + ...
          length(find(strcmp(cfg.rules, 'pitch')))   + ...
          length(find(strcmp(cfg.rules, 'roll')))    + ...
          length(find(strcmp(cfg.rules, 'heave')))   + ...
          length(find(strcmp(cfg.rules, 'sndspd')))  ;


errfile = fullfile(cfg.logfpath, cfg.gbinlogfile);
efid = fopen(errfile,'w');
   
if efid<0
   fprintf('cannot write errors to logfile %s\n',errfile)
   return
end


fprintf('\n\nwriting errors to %s\n\n',errfile);
fprintf(efid,'----------------------------------------------------------\n\n');
for ii=1:length(cfg.rules)
   rulename = sprintf('%s_rule', cfg.rules{ii});
   rule_value = getfield(cfg, rulename);
   fprintf(efid, '%20s: best comes from %s\n', ...
          rulename, rule_value{1});
end
fprintf(efid,'\n');
fclose(efid);



%these are full file names, not relative file names
if (length(find(files(1).name == filesep)) ~= 0)
   % manually strip off everything prior to the last filesep
   sepi1 = find(files(1).name == filesep);
   rawdirname = files(1).name(1:sepi1(end)-1);
   for filei = 1:length(files)
      sepi = find(files(filei).name == filesep);
      if (~isemptys(sepi))
         if ~strcmp(rawdirname, files(filei).name(1:sepi(end)-1))
            error('raw files must all come from the same directory')
         end
         files(filei).name(1:sepi(end)) = [];
      end
   end
end



% now loop through files
for filei = 1:length(files)
   rawfile = files(filei).name;
   rawfilebase = rawfile(1:end-4);

   % we're expecting filenames with prefix_ddd_secs
   fprintf('\nprocessing file %s, %d out of %d\n', rawfile, ...
           filei, length(files))
   
   % (1) get time files
   %  time and positions come from 'best' gps_rule
   timeoutfile =     fullfile(cfg.timeoutpath, [rawfilebase, '.tim.gbin']);
   bestoutfile =     fullfile(cfg.timeoutpath, [rawfilebase, '.best.gbin']);

   fullrawfile = fullfile(cfg.rawfpath, [rawfilebase, '.raw']);
   if (~exist(fullrawfile))
      error(sprintf('cannot find %s\n', fullrawfile))
   end
   
   
   if (~exist(timeoutfile, 'file') | (cfg.redo))
      
      %read the raw data
      %read the log file and fix rawtimes to logger times
      logfile = [fullrawfile, '.log'];
      logdata = read_logfile(logfile);
      
      if isemptys(logdata)
         fprintf('ERROR: empty log file. skipping:\n %s\n', logfile)
         efid = fopen(errfile,'a');
         fprintf(efid, 'ERROR: empty log file. skipping:\n %s\n', logfile);
         fclose(efid);
         continue
      end         
      
      if length(logdata(1,:)) ~= 5
         fprintf('ERROR: old log file: needs 5 columns\n')
         fprintf('skipping: %s\n', logfile)
         efid = fopen(errfile,'a');
         fprintf(efid,'ERROR: old log file: needs 5 columns:\n') ;
         fprintf(efid, 'skipping: %s\n', logfile);
         fclose(efid);
         continue
      end
      
      instrument_dday = logdata(:,5)';   
      
      if (~isemptys(cfg.log_yearbase))
         d0 = datenum(cfg.log_yearbase, 1,1);
         [Y, M, D, H, Mi, S] = datevec(d0 + logdata(:,1));
         ymdhms = [Y(:), M(:), D(:), H(:), Mi(:), S(:)];
         logdata(:,1) = to_day(yearbase, ymdhms);
         
         % old: rounds seconds
         % logdata(:,1) = ...
         %    to_day(yearbase, to_date(cfg.log_yearbase, logdata(:,1),'n'));
      end


      %logger dday, bestdday, sec_pc_minus_utc
      % get all gps sources
      for ii=1:length(cfg.msg_info(:,1))
         ll = cfg.msg_info{ii,2};
         letters3{ii} = ll(1:3);
      end
      gps_sourcenums = strcmp(letters3, cfg.time_fext)';
      % exit if none
      if (length(find(gps_sourcenums)) == 0)
         fprintf('no navigation present.  exiting\n')
         return
      end

      gps_source = find(gps_sourcenums & strcmp(cfg.msg_info(:,1), posinst));
      if length(gps_source) == 0
         fprintf('did not find a %s message from the %s source\n', ...
                 cfg.time_fext,...
                 posinst)
         return
      elseif length(gps_source) > 1
         fprintf('error?? found more than one %s source  for time\n',...
                 cfg.time_fext)

         efid = fopen(errfile,'a');
         fprintf(efid,...
              'error?? found more than one %s source for time\n',...
                 cfg.time_fext);
         fclose(efid);
         return
      else %%% should have an alternative here, not just take the first one
         trinfo_gps = trinfoc{gps_source};
      end

      [timdata, gga_besttime] = ...
          get_besttime(instrument_dday, logdata, cfg, trinfo_gps, errfile); 
      
      if isemptys(timdata) | ...
            (length(timdata(1,:)) ~= length(instrument_dday)) | ...
             (length(find(isnan(timdata(1,:)))) ~= 0) | ...
            min(diff(timdata(2,:))) <= 0
         
         fprintf('time gridding did not work for this file:\n%s\n', timeoutfile)

         efid = fopen(errfile,'a');
         fprintf(efid,'time gridding did not work for:\n %s\n', timeoutfile);
         fclose(efid);
         continue
      end

      write_bin(timeoutfile, timdata, 'besttime',...
                {'logger_dday', 'bestdday', 'sec_pc_minus_utc'});
      fprintf('writing to %s\n', timeoutfile)
      efid = fopen(errfile,'a');
      fprintf(efid, '%s: writing to time file:\n%s\n\n-------------------\n',...
              nowstr, timeoutfile);
      fclose(efid);
      
      ymdhms = clock;
      
      if (min(diff(timdata(2,:))) <= 0)
         efid = fopen(errfile,'a');
         warning('nonmonotonic ''best'' times  times\n')
         fprintf(efid,'nonmonotonic ''best'' times  times\n');
         paused(2)
         fclose(efid);
      end
      
   else
      fprintf('not making time file: %s\n', timeoutfile)
      efid = fopen(errfile,'a');
      fprintf(efid, 'not making time file: %s\n', timeoutfile);
      fclose(efid);
   end

   logger_dday = timdata(1,:)'; 
   badi = find(isnan(logger_dday));
   if length(badi) > 0
      fprintf('WARNING: NaN exists in logger_dday\n')
   end
   logger_timerange = [logger_dday(1) logger_dday(end)];
   if cfg.verbose
      fprintf('time range is %10.7f - %10.7f\n',...
              logger_timerange(1), logger_timerange(2))
   end

      
   %initialize the gbin file:
   bestgbin = NaN*ones(numbest, length(logger_dday));
   % bestfnames is fieldnames
   bestfnames = {'lon', 'lat', cfg.rules{find(~strcmp(cfg.rules, 'gps'))}};
   best_instnames = cell(length(bestfnames)-1);
   %use tr2ilist 
   
   %read the bindata into an array of structures, one for each rule element
   rule = cfg.gps_rule;
   bindata = [];
   best_instnames{1} = sprintf('position (%s)', cfg.gps_rule{1});
   if cfg.verbose
      fprintf('rule = %s, %s\n', rule{1}, rule{2})
   end
   
   %% fill lon, lat
   if cfg.verbose
      fprintf('getting trinfo from %s and %s\n', rule{1}, rule{2})
   end
   
   trinfo = trinfoc{find(strcmp(cfg.msg_info(:,1), rule{1}) & ...
                         strcmp(cfg.msg_info(:,2), rule{2}))};

   [bindata, binstruct, qcmask] = ...
       tr2ilist(trinfo, 'timerange', logger_timerange);
   if (~isemptys(bindata))
      bindata(qcmask) = NaN;  %% 2004/01/12
      
   end
   
   
   %% get positions on logger_dday grid
   gridded = [timdata(1,:);
              table1([gga_besttime(2,:); bindata(2:end,:)]', timdata(2,:))'];
   best_row = find(strcmp(bestfnames, 'lon'));
   bestgbin(best_row,:) = gridded(r_row(binstruct, 'lon'),:);
   best_row = find(strcmp(bestfnames, 'lat'));
   bestgbin(best_row,:) = gridded(r_row(binstruct, 'lat'),:);


   if cfg.verbose
      figure(2);clf
      subplot(411)
        plot(timdata(1,:),timdata(1,:),'co')
        hold on
        plot(bindata(1,:), bindata(1,:),'r.')
        title('instrument (cyan), serial (red)')
      subplot(412)
        plot(bindata(1,:), bindata(r_row(binstruct, 'lon'),:),'go')
        hold on;title('gbin lon')
        best_row = find(strcmp(bestfnames, 'lon'));
        plot(timdata(1,:), bestgbin(best_row,:) ,'r.') 
      subplot(413)
        plot(bindata(1,:), bindata(r_row(binstruct, 'lat'),:),'go')
        hold on; title('gbin lat')
        best_row = find(strcmp(bestfnames, 'lat'));            
        plot(timdata(1,:), bestgbin(best_row,:) ,'r.')
   end
   %% end of lon, lat
   
   for rulei = 3:length(bestfnames)
      bindata = [];
      rulename = sprintf('%s_rule', bestfnames{rulei});
      rule = getfield(cfg, rulename);
      
      best_instnames{rulei-1} = sprintf('%s (%s)', ...
                                        rulename(1:end-5), rule{1});
      if cfg.verbose
         fprintf('rule = %s, %s\n', rule{1}, rule{2})
      end
      
      if cfg.verbose
         fprintf('getting trinfo from %s and %s\n', ...
                 rule{1}, rule{2})
      end
      
      
      trinfo = trinfoc{find(strcmp(cfg.msg_info(:,1), rule{1}) & ...
                            strcmp(cfg.msg_info(:,2), rule{2}))};
      
      [bindata, binstruct, qcmask] = ...
          tr2ilist(trinfo, 'timerange', logger_timerange);
      
      
      if (~isemptys(bindata))
         bindata(qcmask) = NaN;  %% 2004/01/12
      end

      bin_utc = tablelegs(bindata, binstruct, gga_besttime, errfile, cfg.debug);
      
      if isemptys(bindata) | isemptys(bin_utc)
         fprintf('padding gbin with nans:\n%s\n', timeoutfile)
         efid = fopen(errfile,'a');
         fprintf(efid,'padding gbin with nans:\n%s\n', timeoutfile);
         fclose(efid);

         bestfname = bestfnames{rulei};
         best_row = find(strcmp(bestfnames,bestfname));
         bestgbin(best_row,:) = NaN*timdata(1,:);
      else
      
         gridded = [timdata(1,:);
                    table1([bin_utc; bindata(2:end,:)]', timdata(2,:))'];
         bestfname = bestfnames{rulei};
         best_row = find(strcmp(bestfnames,bestfname));
         bestgbin(best_row,:) = gridded(r_row(binstruct, bestfname),:);
      end
   
   
      if strcmp(bestfname, 'heading') & cfg.verbose
         subplot(414)
         plot(bindata(1,:), bindata(r_row(binstruct, 'heading'),:),'go')
         hold on; title('gbin heading')
         best_row = find(strcmp(bestfnames, 'heading'));            
         plot(timdata(1,:), bestgbin(best_row,:) ,'r.')
      
      end
   
   end
   %% end of heading, etc
   
   write_bin(bestoutfile, bestgbin, 'best', bestfnames);

   
   %(3) if there are requested gbin files to be made, make them
   
   if (isemptys(cfg.mkgbin))
      fprintf('not making additional gbin files\n')
   else
      
      % we either wrote timdata or read from it; now set the time range
      if ~exist(timeoutfile)
         fprintf('time file %s does not exist; skipping\n', timeoutfile)
         continue
      end
      for jj=1:length(cfg.mkgbin(:,1))
         outpath = fullfile(cfg.gbindir, cfg.mkgbin{jj,1});
         outfile =  fullfile(outpath, sprintf('%s.%s.gbin', ...
                                              rawfilebase, cfg.mkgbin{jj,2}));
         if (cfg.verbose)
            fprintf('writing to %s\n', outfile)
         end
         if (~exist(outfile) | cfg.redo)
            trinfo = ...
                trinfoc{find(strcmp(cfg.msg_info(:,1), cfg.mkgbin{jj,1}) & ...
                             strcmp(cfg.msg_info(:,2), cfg.mkgbin{jj,2}))};

            if isemptys(trinfo)
               fprintf('no data found in %s/%s\n', ...
                       cfg.mkgbin{jj,1}, cfg.mkgbin{jj,2});
            else
               [bindata, binstruct, qcmask] = ...
                   tr2ilist(trinfo, 'timerange', logger_timerange);

               if (~isemptys(bindata))
                  bindata(qcmask) = NaN;  %% 2004/01/12

                  if cfg.verbose
                     fprintf('doing gbin for %s/%s\n', ...
                             cfg.mkgbin{jj,1},  cfg.mkgbin{jj,2})
                  end
                  

                  bin_utc = tablelegs(bindata, binstruct, gga_besttime, ...
                                      errfile, cfg.debug);
                  if isemptys(bin_utc)
                     fprintf('empty bin_utc: failed\n')
                     efid = fopen(errfile,'a');
                     fprintf(efid,'empty(bin_utc: failed\n');
                     fclose(efid);
                     badgbin = 1;
                  elseif  min(diff(bin_utc)) <= 0 
                     fprintf('nonmonotonic  times.  skipping\n')
                     efid = fopen(errfile,'a');
                     fprintf(efid,'nonmonotonic times.  skipping\n');
                     fclose(efid);
                     badgbin = 1;
                  elseif isemptys(bindata)
                     fprintf('padding gbin with nans:\n%s\n', timeoutfile)
                     efid = fopen(errfile,'a');
                     fprintf(efid,'padding gbin with nans:\n%s\n', timeoutfile);
                     fclose(efid);
                     badgbin = 1;
                  else
                     badgbin = 0;
                  end
                  
                  if badgbin 
                     bestfname = bestfnames{rulei};
                     best_row = find(strcmp(bestfnames,bestfname));
                     bestgbin(best_row,:) = NaN*timdata(1,:);
                  else
                     gdata = table1([bin_utc; bindata(2:end,:)]', ...
                                    timdata(2,:))';
                  end
                  write_bin(outfile, gdata, cfg.mkgbin{jj,2}, ...
                            binstruct.rows(2:end));
               end
            end
         end
      end
   end
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function besthelp()

fprintf(' Two components need to be specified: "msg_info" and the rules               \n')
fprintf('    for the ancillary  data (eg. position, attitude, soundspeed)             \n')
fprintf('                                                                             \n')
fprintf(' ---------------                                                             \n')
fprintf(' |   msg_info  |                                                             \n')
fprintf(' ---------------                                                             \n')
fprintf('                                                                             \n')
fprintf(' msg_info is a cellarray with                                                \n')
fprintf(' - one row per ancillary serial data stream and                              \n')
fprintf(' - 2 columns :                                                               \n')
fprintf('    The first column is the directory containing the data for that           \n')
fprintf('        instrument.  (the same directory name is used in raw/, rbin/,        \n')
fprintf('        and gbin/ directories)                                               \n')
fprintf('    The second column is the extension in the rbin directory                 \n')
fprintf('                                                                             \n')
fprintf('                              rbin                                   \n')
fprintf('               instrument    extension                              \n')
fprintf('   msg_info = {''gpsnav'' ,   ''gps'',      %% position from pcode \n')
fprintf('               ''posmv'' ,    ''gps'',      %% position from posmv \n')
fprintf('               ''ashtech''    ''gps'',      %% position from ashtech  \n')
fprintf('               ''seapath'',   ''gps'',      %% position from seapath\n')
fprintf('               ''seapath'',   ''gps_sea'',  %% position (with qual) from seapath\n')
fprintf('               ''gyro''  ,    ''hdg'',      %% gyro heading        \n')
fprintf('               ''posmv'' ,    ''pmv'',      %% posmv attitude      \n')
fprintf('               ''ashtech'' ,  ''adu'',      %% ashtech attitude    \n')
fprintf('               ''seapath'',   ''sea'',     %% seapath attitude\n')
fprintf('                                                                             \n')
fprintf('                                                                             \n')
fprintf('                                                                             \n')
fprintf(' ------------                                                                \n')
fprintf(' |   rules  |                                                                \n')
fprintf(' ------------                                                                \n')
fprintf(' Choose only one rule.                                                       \n')
fprintf(' EXAMPLE:                                                                    \n')
fprintf(' best.rules = {''gps'', ''heading'', ''pitch'', ''roll'', ''heave''};        \n')
fprintf(' best.gps_rule    =  {''pcode'',          ''gps''};                          \n')
fprintf(' best.heading_rule = { ''gyro'',           ''hdg''};                         \n')
fprintf(' best.pitch_rule  =   { ''adu2'',          ''adu''};                         \n')
fprintf(' best.roll_rule   =   { ''adu2'',          ''adu''};                         \n')
fprintf(' best.heave_rule  =   { ''adu2'',          ''adu''};                         \n')
fprintf('                                                                             \n')
