function [bindata, binstruct] = ...
          bin_extract(binfpath, fext, logger_timerange, yearbase)
% [bindata, binstruct] = bin_extract(binfpath, fext, logger_timerange, yearbase)
% bindata is nrows X N array read from the list of files specified as
%    binfpath/*fext.bin (bin files were written by serasc2bin.py) 
%    with logger_times matching the boundaries specified by logger_timerange
% binstruct is the second output of read_bin.  file is closed here.
% hardwired: looks for all *.rbin first; it there are none, it will try *.bin
% if include yearbase, be verbose
% see 'serasc2bin.py' for more details of columns
% 
% CONVENIENT but superceded by rbin_tr2ilist
%
% NOTE: uses integer part of first column to make second column of GPS data
%  have full dday
% undocumented option: verbose == 1 is verbose
%

% JH 2001/04/05 
if (~exist('yearbase'))
   verbose = 0;
else
   verbose = 1;
end

berrfile = 'bindata.errs';
fid = fopen(berrfile, 'a');

filelist = dirs(fullfile(binfpath, sprintf('*%s.rbin', fext))); 
if (length(filelist) == 0)
   fprintf('no *.rbin files found: trying *.bin\n')
   filelist = dirs(fullfile(binfpath, sprintf('*%s.bin', fext))); 
end
   
   
bindata = [];

for filei = 1:length(filelist)
   fname = filelist(filei).name;
   if (verbose)
      fname
   end
   
   if (nargout == 2)
      [bdata, binstruct] = read_bin(fullfile(binfpath, fname));
      fclose(binstruct.fid);
      binstruct = rmfield(binstruct, 'fid');
   else
      bdata = read_bin(fullfile(binfpath, fname));
   end
   

   if (isemptys(bdata))
      fprintf(fid, 'no data read from %s\n',fullfile(binfpath, fname));
   else
      
      
      goodi = find(  (bdata(1,:) >= logger_timerange(1)) & ...
                     (bdata(1,:) <= logger_timerange(2)));


      if (~isemptys(goodi))
         %% make a note:
         if (strcmp(fext, 'gps'))
            if (min(diff(bdata(1,:))) <= 0)
               fprintf('gps bdata %s not monotonic\n', ...
                       fullfile(binfpath, fname))
               fprintf(fid, 'bdata %s not monotonic\n', ...
                       fullfile(binfpath, fname));
            end
         end
         if (verbose)
            fprintf('%s logger timerange %4.3f - %4.3f\n', ...
                    fullfile(binfpath, fname), bdata(1,1), bdata(1,end))
            fprintf('accepting indices %d-%d\n', goodi(1), goodi(end))
         end
      end
      
      bindata = [bindata bdata(:,goodi)];

         
      %fix GGA and ATT decimal days to have the integer part match logger dday
      if ( (fext == 'adu') | (fext == 'gps') | (fext == 'pmv'))
         bindata(2,:) = bindata(2,:) + round(bindata(1,:) - bindata(2,:));
      end
      
      if (verbose)
         fprintf('%s - %s',...
                 to_date(yearbase,bdata(1,1)),to_date(yearbase,bdata(1,end)))
         if (~isemptys(goodi))
            fprintf('(goodi: %s - %s)',...
                    to_date(yearbase,bdata(1,goodi(1))),...
                    to_date(yearbase,bdata(1,goodi(end))))
         end
         fprintf('\n')
      end
      
      if (logger_timerange(1) > max(bdata(1,:)))
         filei = length(filelist) +1;
      end
   end
end


if ~exist('bdata')
   if (verbose)
      warning('bdata does not exist: returning no data!!!')
   end
   bindata = [];
   binstruct = [];
   return
end


if (length(bdata(1,:)) == 0)
   if (verbose)
      warning('length of bindata = 0: returning no data!!!')
   end
   bindata = [];
   binstruct = [];
   return
end
   

if exist('fid')
   if fid~=0
      fclose(fid);
   end
end
