function besttime = get_besttime(data, logdata, cfg, trinfo, efid)
% besttime = get_besttime(data, logdata, cfg, trinfo, efid)
% use raw times, logger times, and gga times to get a best time grid
% called by mk_gbin
% trinfo is from tr2ilist
% data is from read(instclass, rawfile)
% logdata is from loading the corresponding log file
%
% this is ugly and should be rewritten

%% JH 2003/07/16
besttime = [];

meas_logger = logdata(:,1)';
offsets = logdata(:,2)';

if (meas_logger(1) > 10^6) %convert unix seconds to dday
   meas_logger = (meas_logger + to_sec(1970, 1, 1, 0, 0, 0) - ...
                               to_sec(cfg.yearbase, 1, 1, 0, 0, 0))/86400;
end
%this will be the same size as data.dday but with fixed meas_logger values
logger_dday = NaN*data.dday;   %need this many; but no more
if (length(data.dday) < length(meas_logger))     %bad checksum: lost data end
   meas_logger = meas_logger(1:length(data.dday));
   offsets = offsets(1:length(data.dday));
end
%index into nb dday,
nominal_offset = median(diff(offsets));

if (isemptys(nominal_offset))
   besttime = [];
   return
end

iinto_dday = (offsets/nominal_offset) + 1;
%except there may be problems with the offsets
badi =  find(   (abs(iinto_dday - floor(iinto_dday)) ~= 0) | ...
                iinto_dday > length(data.dday));
iinto_dday(badi)=[];
meas_logger(badi)=[];

%now only use days from data.dday that have logger_dday
logger_dday(iinto_dday) = meas_logger;
coef = polyfit(data.dday(iinto_dday),  logger_dday(iinto_dday), 1);
loggercorrected_rawdday = polyval(coef, data.dday);
% smoothr instrument times: with a 5-pt boxfilt on the time differences
dt = filt242(diff(loggercorrected_rawdday));
lc_rawddaymean = mean(loggercorrected_rawdday);
filt_loggercorrected_rawdday = loggercorrected_rawdday(1) + cumsum([0 dt]);
%this might only be necessary for small files
flc_rawdday = filt_loggercorrected_rawdday - ...
    mean(filt_loggercorrected_rawdday) + lc_rawddaymean;
flc_rawdday = unscootch(flc_rawdday, loggercorrected_rawdday, cfg.scootch_sec);



if (min(diff(flc_rawdday)) <= 0)
   fprintf('nonmonotonic filtered logger_corrected adcp times\n')
   fprintf(efid,'nonmonotonic filtered logger_corrected adcp times\n');
   besttime = [];
   return
end



%so now we have
% flc_rawdday     = best version of raw dday corrected to logger times
% logger_dday     = logger times for flc_rawdday
% need to match that to gga (or the reverse)


%get the logger time info from the gps files
logger_range = [flc_rawdday(1) flc_rawdday(end)];
%bindata is [logger_dday, fullggadday, lon, lat]

bindata = tr2ilist(trinfo, 'timerange', logger_range);

if (isemptys(bindata))
   berrfid = fopen('bindata.errs','a');
   fprintf(berrfid, 'note added at at %s\n', nowstr);
   fprintf('no data found in meas_logger range %f-%f\n',...
           logger_range(1), logger_range(2))
   fprintf(berrfid,'no data found in meas_logger range %f-%f\n',...
           logger_range(1), logger_range(2))
   fprintf('check bindata.errs\n')
   fclose(berrfid);
   return
end


% we KNOW these are gga timestamps, and we KNOW they may lag behind
% instrument timestamps.  

jumpi = find(diff(bindata(2,:)) < 0);
if length(jumpi) > 1
   fprintf('two negative jumps in gga time\n')
   return
elseif length(jumpi) == 1
   fixjump = (bindata(2,jumpi+1) - bindata(2,jumpi)) -  ...
             mean(diff(bindata(2,jumpi+1:end)));
   bindata(2,1:jumpi) = bindata(2,1:jumpi) + fixjump;
end


%fix bad times due to ?? cpu cycles used up??
okcpu =  fix_timestamps(bindata, efid);
if (isemptys(okcpu))
   fprintf(efid,'busy cpu: failed loop for file\n')
   besttime = [];
   return
else
   bindata(1,:) = okcpu;
end


try
   ggadday = table1([bindata(1,:)' bindata(2,:)'],flc_rawdday)';
catch
   disp('ggadday failed')
   fprintf(efid,'busy cpu: ggadday failed\n')
   besttime = [];
   return
end


if (cfg.timebase == 'log')
   bestdday = flc_rawdday;   %use the filtered, logger_corrected adcp ddays
elseif (cfg.timebase == 'gga')
   %% since gga is a dday fraction we have to add an integer.  guess
   [ggann, ggaxx] = hist(floor(flc_rawdday), min(floor(flc_rawdday))+[-1:3]);
   [maxxx, maxii] = max(ggann);
   bestdday = ggadday + ggaxx(maxii);
else %table from the filtered, logger_corrected adcp ddays to gga ddays
   % WRONG:  bestdday = ggadday;
   fprintf('timebase %s not implimented\n', cfg.timebase)
   return
end

% logger time - gga
sec_pc_minus_utc = ((flc_rawdday - floor(flc_rawdday)) - ggadday)*86400;

besttime =  [flc_rawdday ;  bestdday ;sec_pc_minus_utc];


if (length(besttime(1,:)) ~= length(data.dday))
   warning('mismatched gps times and data.dday')
   keyboard
end

if (cfg.verbose)
   fprintf('logger_range : %s - %s\n',...
           to_date(2001,logger_range(1)),...
           to_date(2001,logger_range(2)))
   subplot(211)
   plot(86400*diff(flc_rawdday))
   title('filtered logger_corrected adcp times')
   yl(0,4)
   subplot(212)
   plot(86400*diff(table1([bindata(1,:)' bindata(2,:)'], flc_rawdday)))
   title('gga times if used')
   yl(0,4)
   drawnow
end

