function mk_allgbin(varargin)
% mk_allgbin(varargin)
% mk_allgbin" will make time2gbin and best2gbin 
%
% do this from adcpr, or where raw/ and rbin/ and grbin/ are 
% starts at the beginning and skips files till it finds a new one to do
%    (could manually set 'redo',1)
% to start later and not see the messages do this:
%
%% varargin                  default 
%
%  instclass                [required]!!  (nb, son)
%  yearbase                 [required]!!
%  
%  --- choose files this way:---
%  startfilei               start with this filenum (partway through the list)
%  glob                      '*.raw'  (eg. could choose specific days)
%
%  --- or this way---
%  filelist                 cellarray of base filenames
%
%
%  redo                      0 (do not process if file exists) 1=rewrite
%  make_allxxx               0 (only fill gbin/best)  (1: do all xxx2gbin)
%  xxx_info                  CELL ARRAY with 4 rows, (one column per xxx)
%                            telling which serially logged data to make rbin
%
%                           rbin       gbin
%              instrument  extension  extension     
%  xxx_info = {'pcode' ,    'gps',     'pos',     %position from pcode
%              'posmv' ,    'gps',     'pos',     %position from posmv 
%              'gyro'  ,    'hdg',     'hdg',     %gyro heading
%              'adu2'  ,    'gps',     'pos',     %position from adu2
%              'posmv' ,    'pmv',     'pmv',     %posmv attitude
%              'adu2'  ,    'adu',     'adu'}     %ashtech attitude
%
%  rawdir                    'raw';
%  rbindir                   'rbin';
%  gbindir                   'gbin';
%  logfpath                  'log'
%  rawfpath                  'raw/adcp_pings'
%  posinst                   'pcode'                %primary position instrument
%  best                      structure of cellarrays for xxx_info
%                            (type mk_allgbin('help','best') for more info
%
%%% JH 2002/08/18: 
%%% vg0202
%%% start in /home/jules/vg0202
%%% "rbin" and "gbin" directories and subdirectories already exist
%%%  rbin files already exist
%%%  new version!! for sontek and Kilomoana logging


cfg.yearbase = [];
cfg.instclass = [];
cfg.startfilei  = 1;
cfg.glob = '*.raw';
cfg.filelist = [];
cfg.redo = 0;
cfg.make_allxxx = 0;
cfg.xxx_info = {'pcode' ,    'gps',     'pos',     %position from pcode
                'posmv' ,    'gps',     'pos',     %position from posmv 
                'gyro'  ,    'hdg',     'hdg',     %gyro heading
                'adu2'  ,    'gps',     'pos',     %position from adu2
                'posmv' ,    'pmv',     'pmv',     %posmv attitude
                'adu2'  ,    'adu',     'adu'}     %ashtech attitude
cfg.rawdir    =    'raw';
cfg.rbindir   =    'rbin';
cfg.gbindir   =    'gbin';
cfg.logfpath  =    'log';
cfg.rawfpath  =    'raw/adcp_pings';
cfg.posinst   =    'pcode' ;
cfg.best      =   [];              %structure of cellarrays for best2gbin
                                   % empty means 'use defaults' (set for KM)
cfg.help = [];


cfg = fillstruct(cfg, varargin,0);

if (~isemptys(cfg.help))
   besthelp
   return
end


if (isemptys(cfg.yearbase))
   help(mfilename)
   fprintf('===>  mk_allgbin: must set yearbase in varargin\n')
   return
end

if (isemptys(cfg.instclass))
   help(mfilename)
   fprintf('===>  mk_allgbin: must set instrument class in varargin\n')
   return
end

%%% check if input directories exist
checkdirs = {cfg.rawdir, cfg.rawfpath};
for dirnum = 1:length(checkdirs)
   if (~exist(checkdirs{dirnum}, 'dir'))
      fprintf('directory %s does not exist\n', checkdirs{dirnum})
      return
   end
end


fprintf('about to make gbin dirs from %s\n', pwd)
%%% check if output diretories exist
mkdirs = {cfg.rbindir, cfg.gbindir, cfg.logfpath};
for dirnum = 1:length(mkdirs)
   dirname = mkdirs{dirnum};
   if (~exist(dirname, 'dir'))
      fprintf('%s directory does not exist, making directory.\n',dirname)
      mkdir(dirname);
   end
end

mkdirs = unique(cfg.xxx_info(:,1));
for dirnum = 1:length(mkdirs)
   dirname = fullfile(cfg.gbindir, mkdirs{dirnum});
   if (~exist(dirname, 'dir'))
      fprintf('%s directory does not exist, making directory:\n%s\n',...
              mkdirs{dirnum}, dirname)
      mkdir(dirname);
   end
end

timedir  = fullfile(cfg.gbindir, 'time');
if (~exist(timedir, 'dir'))
   fprintf('time directory does not exist, making directory:\n%s\n',...
           timedir)
   mkdir(timedir);
end


if (isemptys(cfg.filelist))
   files = dirs(fullfile(cfg.rawfpath, cfg.glob));
   fprintf('found %d files ', length(files))
   fprintf('starting at file number %d\n', cfg.startfilei)
   if (cfg.startfilei > 1)
      files(1:cfg.startfilei) = []
   end
else
   files = cell2struct(cfg.filelist, 'name', 1);
end
   
if (length(files) > 0)
   fprintf('... last file is  %s', files(end).name)
end
fprintf('\n')
paused(1)
   

for filei = 1:length(files)
   rawfile = files(filei).name;
   rawfilebase = rawfile(1:end-4);
   
   fprintf('\nprocessing file %s, %d out of %d\n', rawfile, filei, length(files))

   %%% time is from pcode directory, .gps files
   time2gbin(rawfilebase,...
             'back_to_front',1, ...
             'redo',      cfg.redo,...
             'binfpath',  fullfile(cfg.rbindir, cfg.posinst),...
             'outfpath',  fullfile(cfg.gbindir, 'time'),...
             'rawfpath',  cfg.rawfpath,...
             'logfpath',  cfg.logfpath,...
             'yearbase',  cfg.yearbase, ...
             'instclass', cfg.instclass);
   
   best2gbin(rawfilebase,...
             'redo',     cfg.redo,...
             'rawdir',   cfg.rawdir,...
             'rbindir',  cfg.rbindir,...
             'gbindir',  cfg.gbindir,...
             'logfpath', cfg.logfpath,...
             'best',     cfg.best);

end   

if (cfg.make_allxxx)
  
   %%% all the extras   
   for filei = 1:length(files)
      rawfile = files(filei).name;
      rawfilebase = rawfile(1:end-4);
      fprintf('processing file %s, %d out of %d\n', ...
              rawfile, filei, length(files))
      
      for xxxnum = 1:length(cfg.xxx_info(:,1))
         inst    =     cfg.xxx_info{xxxnum, 1};
         in_fext =     cfg.xxx_info{xxxnum, 2};
         out_fext =    cfg.xxx_info{xxxnum, 3};
         tim_gbinpath = fullfile(cfg.gbindir, 'time');
         xxx_rbinpath = fullfile(cfg.rbindir, inst);
         outfpath    = fullfile(cfg.gbindir, inst);
         
         xxx2gbin(rawfilebase, ...           %position from pcode
                  'redo',     cfg.redo,...
                  'in_fext',      in_fext,     ...
                  'out_fext',     out_fext,    ...
                  'tim_gbinpath', tim_gbinpath,...
                  'outfpath',     outfpath,    ...
                  'xxx_rbinpath',  xxx_rbinpath, ...
                  'logfpath',     cfg.logfpath);
      end

   end
end




function besthelp()


fprintf(' Two components need to be specified: "xxx_info" and the rules               \n')
fprintf('    for the ancillary  data (eg. position, attitude, soundspeed)             \n')
fprintf('                                                                             \n')
fprintf(' ---------------                                                             \n')
fprintf(' |   xxx_info  |                                                             \n')
fprintf(' ---------------                                                             \n')
fprintf('                                                                             \n')
fprintf(' xxx_info is a cellarray with                                                \n')
fprintf(' - one row per ancillary serial data stream and                              \n')
fprintf(' - 3 columns :                                                               \n')
fprintf('    The first column is the directory containing the data for that           \n')
fprintf('        instrument.  (the same directory name is used in raw/, rbin/,        \n')
fprintf('        and gbin/ directories)                                               \n')
fprintf('    The second column is the extension in the rbin directory                 \n')
fprintf('    The third column is the extension in the gbin directory                  \n')
fprintf('       (the second and third column may be different, for instance           \n')
fprintf('       when an instrument (eg. adu2) logs more than one ascii message)       \n')
fprintf('                                                                             \n')
fprintf(' the defaults are for the Kilo Moana. eg:                                    \n')
fprintf('                                                                             \n')
fprintf('                            rbin       gbin                                  \n')
fprintf('               instrument  extension  extension                              \n')
fprintf('   xxx_info = {''pcode'' ,    ''gps'',     ''pos'',     %position from pcode \n')
fprintf('               ''posmv'' ,    ''gps'',     ''pos'',     %position from posmv \n')
fprintf('               ''gyro''  ,    ''hdg'',     ''hdg'',     %gyro heading        \n')
fprintf('               ''adu2''  ,    ''gps'',     ''pos'',     %position from adu2  \n')
fprintf('               ''posmv'' ,    ''pmv'',     ''pmv'',     %posmv attitude      \n')
fprintf('               ''adu2''  ,    ''adu'',     ''adu''}     %ashtech attitude    \n')
fprintf('                                                                             \n')
fprintf('                                                                             \n')
fprintf(' ------------                                                                \n')
fprintf(' |   rules  |                                                                \n')
fprintf(' ------------                                                                \n')
fprintf('                                                                             \n')
fprintf(' Only include the fields with values; they must be IN ORDER                  \n')
fprintf(' best.rules = {''gps'', ''heading'', ''pitch'', ''roll'', ''heave''};        \n')
fprintf('                                                                             \n')
fprintf(' a rule is an Nx2 cellarray with rbin directories in the first column        \n')
fprintf('     and rbin suffix in the second column.                                   \n')
fprintf('                                                          (rule)             \n')
fprintf('                      directory       rbin_suffix        reference_to        \n')
fprintf('                                                                             \n')
fprintf(' best.gps_rule     =  {''pcode'',          ''gps'';       %best              \n')
fprintf('                      ''adu2'',            ''gps''};      %worst case        \n')
fprintf('                                                                             \n')
fprintf(' best.heading_rule = { ''gyro'',           ''hdg'';       %best              \n')
fprintf('                       ''adu2'',           ''adu''};      %worst case        \n')
fprintf('                                                                             \n')
fprintf(' best.pitch_rule  =   { ''adu2'',          ''att''};                         \n')
fprintf(' best.roll_rule   =   { ''adu2'',          ''att''};                         \n')
fprintf(' best.heave_rule  =   { ''adu2'',          ''att''};                         \n')

