function lastname = reset_blkinfo(blkinfo_logfile, numfiles)
% lastname = reset_blkinfo(blkinfo_logfile, numfiles)
%
% read BLKINFO.log, delete the last numfiles from list, rewrite to disk.
% return name of most recent remaining file
% copy that file's [basename]_blkinfo.mat to the generic blkinfo.mat
%    to prepare for subsequent calls to load_raw.

%%% logic is here to specify a cellarray of basenames to delete
%%% or specify a decimal day to delete after.


% read BLKINFO.log 
%       filename        flag              profile#        startdday
[junk1, bfiles, junk2, newblock, junk3, profnum, junk4, startdd] = ...
    textread(blkinfo_logfile, '%s%s%s%d%s%d%s%f');

basenames = bfiles(end-numfiles+1 : end);

if (newblock(1) ~= 1) | (profnum(1) ~= 0)
   fprintf('%s:\n', mfilename)
   fprintf('ERROR: file does not start at the beginning of a database\n')
   lastname = 'error';
end


%%% (1) check for duplicate filename entries
if length(unique(bfiles)) ~= length(bfiles)
   fprintf('%s:\n', mfilename)
   fprintf('ERROR: duplicate filename entries\n')
   lastname = 'error';
   return
end


%% (2) now make sure files are in increasing decimal day order
[tmp, index] = sort(startdd);
if ~isemptys( find(diff(index) ~= 1))
   fprintf('%s:\n', mfilename)
   fprintf('filenames are not in monotonic time order\n')
   lastname = 'error';
   return
end


% (3) now go through and delete  'basenames' names from the original list
% find indices for deletion
deli = [];
for ii=1:length(basenames)
   foundi = find(ismember(bfiles, basenames{ii}));
   if isemptys(foundi)
      fprintf('%s:\n', mfilename)
      fprintf('filename %s is not in the original list of files\n', ...
              basenames{ii})
      lastname = 'error';
      return
   end
   if length(foundi) > 1
      fprintf('%s:\n', mfilename)
      fprintf('insufficient error trapping: \n')
      fprintf('file %s is duplicated in original list of files\n',...
              basenames{ii})
      lastname = 'error';
      return
   end
   deli = [deli; foundi];
end
% make sure deletions are consecutive and at the end
if length(deli) > 1
   if ~isemptys(find(diff(deli)~=1))
      fprintf('%s:\n', mfilename)
      fprintf('ERROR: basenames to be deleted is not consecutive\n')
      lastname = 'error';
      return
   end
end

if ~isemptys(deli) & (deli(end) ~= length(bfiles))
   fprintf('%s:\n', mfilename)
   fprintf('ERROR: basenames to be deleted is not at the end\n')
   lastname = 'error';
   return
end

%% so, we should be ready to rewrite the BLKINFO file
index = [1:length(bfiles)];
index(deli)=[];

fid_blkinfo = fopen(blkinfo_logfile,'w');
for ii=1:length(index)
   fprintf(fid_blkinfo,...
           'filebase= %s newblock= %d profile= %d  start_dday= %10.7f \n',...
           bfiles{ii}, newblock(ii), profnum(ii), startdd(ii));
end
fclose(fid_blkinfo);

lastname = bfiles{ii};


%%% now read the blkinfo.mat file for the last file and rewrite it to blkinfo
load(sprintf('%s_blkinfo.mat', lastname))
save6 blkinfo blkinfo



      