function mask = error_velocity_cutoff(data, editcfg)
% mask = error_velocity_cutoff(data, editcfg)
% who: called by pingedit
% output: returns a mask with 1 where error velocity > editcfg.error_velocity_cutoff
% input: (1) data: structure containing error velocity
%            (error velocity is expected to be in data.enu, in the beam 4 location)
%         (2) editcfg: structure from get_pedefaults, containing 
%              field "error_velocity_cutoff"
%          

% JH. revamped 2004/09/18


if ~isfield(editcfg, 'error_velocity_cutoff') 
   fprintf('no error velocity in editcfg structure\n')
   mask = [];
   return
end


if isfield(data,  'emeas')
   error_velocity = data.emeas;
   no_ev = 0; %found it
else
   if isfield(data, 'enu') 
      if length(size(data.enu) ~= 3)
         no_ev = 1;
      else
         [nbins, nbeams, nprofs] = size(data.enu);
         if length(nbeams) ~= 4
            no_ev = 1;
         else
            error_velocity = squeeze(data.enu(:,4,:))
            no_ev = 0; % found it
         end
      end
   end
end
if no_ev
   fprintf('no error velocity in data structure\n')
   mask = [];
   return
end

mask = (error_velocity > editcfg.error_velocity_cutoff);

