function mask = ocean_velocity_cutoff(data, editcfg)
% mask = ocean_velocity_cutoff(data, editcfg)
% who: called by pingedit
% output: returns a mask with 1 where 
%            ocean velocity      > editcfg.ocean_uv_cutoff
% input: (1) data: structure containing ocean velocities in one of these forms:
%           - velocity fields called "uabs" and "vabs"
%           - velocity fields called "umeas" and "vmeas" AND 
%                  ship spped fields called "uship" and 'vship"
%           - velocity fields called "umeas" and "vmeas" AND nav.txy1
%        (2) editcfg: structure from get_pedefaults, containing 
%              field "editcfg.ocean_uv_cutoff"
%          

% JH. revamped 2004/09/18



if ~isfield(editcfg, 'ocean_uv_cutoff') 
   fprintf('no ocean_uv_cutoff in editcfg structure\n')
   return
end

  
mask = zeros(size(data.cuv));
if isfield(data, 'uabs')
   mask(abs(data.uabs) > editcfg.ocean_uv_cutoff) = 1;
end

if isfield(data, 'vabs')
   mask(abs(data.vabs) > editcfg.ocean_uv_cutoff) = 1;
end

