function [binmask] = outlier_binedit(vel,  stdcutoff, nummodes, binval)
% [binmask] = outlier_binedit(vel,  stdcutoff, binval)
%
% get velocity outliers by bin using mode (highest counts from hist)
% nummodes: number of peaks to center around
% stdcutoff: window (number of stddevs) to use to flag outliers
% % reducing either number will tighten the window and exclude more data


binmask = zeros(size(vel));
avel = abs(vel);

[mm,ss,nn,mmed] = mstdgap(avel(:));

goodi = find(~isnan(avel));

if (length(goodi) < 2)
   return
end

% get the mode (most often hit)
% this assumes that values near zero are associated with bubbles 
%    but a single deep profile might have good data
%%%  watch out for turning though...

% spread out the values depending on range
[histn, histx] = hist(avel(:), max(10, 3*ceil(max(avel(goodi)))));
% turn it into a percent good

% find indices for largest three counts
[sorted_histn, sortedi] = sort(histn);

if nummodes > length(histx)
   fprintf('in outlier_binedit: nummodes excedes length of histogram.\n')
   fprintf('this effectively disables outlier editing\n')
   return
end

modecounts = sorted_histn(end-nummodes+1:end);
modeindex = sortedi(end-nummodes+1:end);
mode_x = histx(modeindex);
   

[rows, cols] = size(avel);
outlier = zeros(nummodes, rows, cols);
for modei = 1:nummodes
   %% get nearby values; calculate a stddev
   closemask = abs(  avel - mode_x(modei)) < 3;  %% 3m/s is  half max ship speed??
   
   if (1==0)
      [nn,xx]=hist(avel(:),[0:.5:15]);
      clf
      hh=plot(xx,nn,'r');lw(hh,2)
      [nn,xx]=hist(avel(closemask), [0:.5:15]);
      hold on
      plot(xx,nn,'g-')
      paused
   end
   
   [mm,ss]=mstdgap(avel(closemask)); 
   %% casting the range in terms of a stddev
   outmask = abs(  avel - mode_x(modei)) > stdcutoff*ss;  
   outlier(modei , :,:) = outmask;
end


binmask(squeeze(sum(outlier))~=0) = binval;

