function mask = spectral_width_cutoff(data, editcfg)
% mask = spectral_width_cutoff(data, editcfg)
% who: called by pingedit
% output: returns a mask with 1 where data.sw excedes editcfg.sw_cutoff
% input:  (1) data: structure containing spectral width
%                (data.sw can be Nbins x Nbeams x Ntimes --or-- Nbins x Ntimes)
%         (2) editcfg:  structure from get_pedefaults, containing 
%                field "spectral_width_cutoff"

if ~isfield(data, 'sw') 
   fprintf('no spectral width in this dataset\n')
   mask = [];
   return
end

if ~isfield(editcfg, 'spectral_width_cutoff') 
   fprintf('no spectral width cutoff in editcfg\n')
   mask = [];
   return
end


if length(size(data.sw)) == 3
   mask = zeros(size(squeeze(data.sw(:,1,:))));
   for beam = 1:4
      mask(squeeze(data.sw(:,beam,:)) > editcfg.spectral_width_cutoff) = 1;
   end
else
   mask = zeros(size(data.sw));
   mask(data.sw > editcfg.pectral_width_cutoff) = 1;
end
