function [txyz, startdd] = get_txyz(uhdascfg, varargin)
% [txyz, startdd] = get_txyz(uhdascfg, varargin )
% txyz is [dday lon lat depth] fom smith-sandwell under subsampled cruisetrack
%
% name                default                      what
% ----------
% ddrange              'all'                : specify ddrange 
%                                           :  [stardd  endd]
% step                  1                   : subsample
%                                           : (recommended for long cruises)
% savename             ''                   : save longrid, latgrid, topo
%                                           : eg. 'topo.mat'
%                                           : default = do not save


cfg.ddrange = 'all';
cfg.step = 1;
cfg.savename = '';
cfg.plotit = 0;
cfg = fillstruct(cfg, varargin);


filelist = dirs(fullfile(uhdascfg.rbindir, 'gpsnav', '*.rbin'),'fullfile',1);
gpsinfo = tr2ilist(filelist, 'timerange', cfg.ddrange, 'step', cfg.step);
[gdata, gstruct] = tr2ilist(gpsinfo);
lon = gdata(3,:);
lat = gdata(4,:);
[lonrange, latrange]=map_range(lon, lat);
[longrid, latgrid, topog] = best_topo(lonrange, latrange);

if ~isempty(cfg.savename)
   fprintf('saving topog to "%s"\n', cfg.savename)
   if strcmp(cfg.savename(end-3:end), '.mat')
      savename = cfg.savename(1:end-4);
   else
      savename = cfg.savename;
   end
   cmd = sprintf('save6 %s longrid latgrid topog', savename);
   eval(cmd)
end

[longridM, latgridM] = meshgrid(longrid,latgrid);
zz = interp2(longridM, latgridM, topog, lon, lat);

txyz = [gdata(r_row(gstruct, 'u_dday'),:)', lon(:) lat(:) zz(:)];


