function cc = get_configs(instclass, varargin)
% cc = get_configs(instclass, varargin)
% 
%    [details here soon]
% * two modes (see below)
% * designed to be sent to write_configs(cc)
%
% varargin (name)             default                  value
%-------------------        ----------           --------------- 
%  yearbase               (from data, req. for NB)    [] (from data)
%======================================================================
% eg.  cc = get_configs(os,'dirbase','../osdata/cramer075/c185',...
%             'file_suffix','LTA')
%
% varargin (name)             default                  value
%-------------------        ----------           --------------- 
% verbose                       0                echo filename
% file_suffix                 'LTA'              (to glob on)
% dir_base                      './'              where to find data
% sort                        'time'              how "dirs" sorts (help dirs)
%
%
%
% or cc=get_configs(os,'filelist', filelist)  % (filelist from "dirs")
%
% varargin (name)             default                         value
%-------------------        ----------                  --------------- 
%
% filelist                     []             from dirs  (NOTE: if filelist 
%                                              is used, no additional
%                                              filelist is generated

%% JH 2003/10/12
%% note, can be generalized, but have to add yearbase for nb

%% JH: 2004/04/02 - generalize to try for second set in all datasets.  

cfg.verbose = 0; 
cfg.file_suffix = 'LTA';
cfg.dirbase = '';
cfg.sort = 'time';
cfg.filelist = [];
cfg.yearbase = [];

cfg = fillstruct(cfg, varargin, 0);

if isemptys(cfg.filelist)
   filelist = dirs(fullfile(cfg.dirbase, sprintf('*.%s', cfg.file_suffix)),...
                   'sort',cfg.sort);
else
   filelist = cfg.filelist;
end


if (isemptys(filelist))
   cfg
   fprintf('EMPTY filelist\n')
   cc = [];
   return
end

[pp,nn,ee]=fileparts(filelist(1).name);
cfg.file_suffix = ee(2:end);

%%%%%%%%%%%%%%%%

cnames = {...
   'num_pings',...
   'ncells', ...
   'cell', ...
   'blank', ...
   'pulse', ...
   'bin1distance',...
   'bt_was_on', ...
   'tr_pressure', ...
   'second_set',...
   'mode'};

cindex.test = [];
for cnum = 1:length(cnames)
   cindex = setfield(cindex, cnames{cnum}, cnum);
end
cindex = rmfield(cindex,'test');

%%%%%%

if isa(instclass, 'os')
   second_set = [0 1];
else
   second_set = 0;
end


NN = length(second_set)*length(filelist);

config = NaN*zeros(NN, length(cnames));
otherinfo.maxdepth         = NaN*zeros(NN, 1);
otherinfo.dday_1           = NaN*zeros(NN, 1); 
otherinfo.dday_end         = NaN*zeros(NN, 1); 
otherinfo.sec_pc_minus_utc = NaN*zeros(NN, 1); 
otherinfo.numens           = NaN*zeros(NN, 1); 
otherinfo.secs_per_ens     = NaN*zeros(NN, 1); 
otherinfo.lon_range        = NaN*zeros(NN, 2); 
otherinfo.lat_range        = NaN*zeros(NN, 2); 
otherinfo.yearbase         = NaN*zeros(NN, 2); 

allerrmsg = cell(NN,1);

%%%%%%%
for ifile = 1:length(filelist)
   file =  fullfile(cfg.dirbase, filelist(ifile).name);
   if ~exist(file)
      fprintf('file %s does not exist!???\n',file)
      continue
   else
      fprintf('%s\n', file)
   end
      
   if cfg.verbose
      fprintf('%s\n', file)
   end
   
   
   for si = 1:length(second_set)
      iconfig = 2*(ifile-1) + si;
      sset = second_set(si);
      if isa(instclass,'os')
         if (isemptys(cfg.yearbase))
            [data, errmsg] = read(instclass,file,'ends',2,'vars','nav',...
                                  'second_set', sset, 'quiet', 1);
         else
            [data, errmsg] = read(instclass,file,'ends',2,'vars','nav',...
                                  'second_set', sset,'quiet', 1, ...
                                  'yearbase', cfg.yearbase);      
         end
      else
         if (isemptys(cfg.yearbase))
            [data, errmsg] = read(instclass,file,'ends',2,'vars','nav');
         else
            [data, errmsg] = read(instclass,file,'ends',2,'vars','nav',...
                                  'yearbase', cfg.yearbase);      
         end
      end         
      
      if isemptys(data)
         continue
      else
         data.config.second_set = sset;
      end
      
      
      for cnum = 1:length(cnames)
         config(iconfig,cnum) = round(getfield(data.config, cnames{cnum}));
      end
      
      otherinfo.yearbase(iconfig) = data.config.yearbase;
      otherinfo.maxdepth(iconfig) = round(data.config.ncells* data.config.cell + ...
                        (data.config.blank + data.config.pulse)/2 + data.config.tr_pressure);
      otherinfo.startdd(iconfig)  = data.dday(1);
      otherinfo.enddd(iconfig)    = data.dday(end);
      
      if isfield(data, 'nav')
         otherinfo.sec_pc_minus_utc(iconfig) = round(data.nav.sec_pc_minus_utc(1));
         lon = rem(data.nav.txy1(2,:)+360, 360);
         lat = data.nav.txy1(3,:);
         goodi = find(~isnan(lon));
         if (~isemptys(goodi))
            otherinfo.lonrange(iconfig,:) = [min(lon(goodi)) ; max(lon(goodi))];
            otherinfo.latrange(iconfig,:) = [min(lat(goodi)) ; max(lat(goodi))];
         else
            otherinfo.lonrange(iconfig,:) = [NaN NaN];
            otherinfo.latrange(iconfig,:) = [NaN NaN];
         end
      else
         otherinfo.sec_pc_minus_utc(iconfig) = NaN;
         otherinfo.lonrange(iconfig,:) = [NaN NaN];
         otherinfo.latrange(iconfig,:) = [NaN NaN];
      end
      
      otherinfo.numens(iconfig)        = data.ens_num(end) - data.ens_num(1) + 1;
      
      if (data.ens_num(end) == data.ens_num(1))
         otherinfo.sec_per_ens(iconfig) = NaN;
      else
         otherinfo.sec_per_ens(iconfig) = ...
             86400*(data.dday(end) - data.dday(1))/(data.ens_num(end)-data.ens_num(1));
      end
   end
end

   
cc.cindex = cindex;
cc.cnames = cnames;
cc.config = config;
cc.otherinfo = otherinfo;
cc.file_suffix = cfg.file_suffix;
cc.dirbase = cfg.dirbase;
cc.filelist = filelist;
cc.errmsg = allerrmsg;

