function qcinfo = get_qcinfo(cfg)
% qcinfo = get_qcinfo(cfg);
% qcinfo is a vector with fields
%   filelist, trinfo, instrument, suffix, ddrange, struct
%
%
% suitable for plugging into tr2ilist
%
% eg. cfg = get_uhdascfg('cfgbase','km0401')
%     qcinfo = get_qcinfo(cfg)
%    rdata = tr2ilist(qcinfo(1).trinfo, 'timerange', [6 7]);
%%%% 
%
% suitable for plugging into plot_qc(qcinfo, field, 'timerange', [6 7])
%
% eg. cfg = get_uhdascfg('cfgbase','km0401')
%     qcinfo = get_qcinfo(cfg)
%     plot_qc(qcinfo, 'heading', [6 7])
%    


qcinfo = [];


for ii=1:length(cfg.msg_info(:,1))
   instname = cfg.msg_info{ii,1};
   suffix     = cfg.msg_info{ii,2};
   search_str = sprintf('*.%s.rbin', suffix);
   
   filelist = dirs(fullfile(cfg.rbindir, instname, search_str),...
                 'fullfile',1);
   trinfo =  tr2ilist(filelist,  'rowname', 'u_dday');
   if isemptys(trinfo)
      continue
   end
   
   qcinfo(ii).filelist = filelist;
   qcinfo(ii).trinfo = trinfo;
   qcinfo(ii).instrument = instname;
   qcinfo(ii).suffix = suffix;
   
   [rdata, rstruct]=read_bin(filelist(1).name,'last',1);
   fclose(rstruct.fid);
   qcinfo(ii).rstruct = rstruct;

   
   fstart = 0;
   fend = length(filelist);;
   %work through looking for the first nonempty file for the first time:
   for jj=1:length(trinfo(:,1))
      if ~isemptys(trinfo{jj,2})
         tr1 = trinfo{jj,2};
         fstart = fstart+1;
         break;
      end
   end
   for jj=length(trinfo(:,1)):-1:1
      if isemptys(trinfo{jj,2})
         fend = fend - 1;
      else
         tr2 = trinfo{jj,2};
         break
      end
   end
   
   qcinfo(ii).ddrange = [tr1(1) tr2(2)];
   fprintf('%d entries from %s/*.%s.rbin\n', fend - fstart + 1, ...
           instname, suffix)
   fprintf('dd range is %f to %f\n\n', tr1(1), tr2(2))
end
