function tdh = plot_avgdh(qcinfo, oname, cruisename);
% tdh = plot_avgdh(qcinfo, oname, cruisename);  
% calls mk_avgdh and makes a plot (tdh is the output from mk_avgdh)
%
% usage example: 
% cfg = get_uhdascfg('cfgbase','km0326');
% qcinfo = get_qcinfo(cfg)
% plot_avgdh(qcinfo, 'posmv', cfg.cruiseid);  
% plot_avgdh(qcinfo, 'adu2', cfg.cruiseid);
%
% NOTE: 
%   cfg.msg_info must contain entries for gyro and the other heading source


gyronum = 0;
othernum = 0;
instruments = {};
for ii=1:length(qcinfo)
   instruments{ii} = qcinfo(ii).instrument;
   if strcmp(instruments{ii}, 'gyro') & ...
         ismember('heading', qcinfo(ii).rstruct.rows)
      gyronum = ii;
   end
   if strcmp(instruments{ii}, oname) & ...
         ismember('heading', qcinfo(ii).rstruct.rows)
      othernum = ii;
   end
end

[gdata, gstruct] = tr2ilist(qcinfo(gyronum).trinfo);
[odata, ostruct] = tr2ilist(qcinfo(othernum).trinfo);

tdh = mk_avgdh(gdata, gstruct, odata, ostruct, oname);

maxsamp = max(tdh.numgood);
badi = find(tdh.numgood < .5*maxsamp);
goodi = find(tdh.numgood > .5*maxsamp);

figure
subplot(311)
plot(tdh.dday, tdh.ang,'k')      
hold on
plot(tdh.dday(badi), tdh.ang(badi),'r.')      
title(sprintf('%s - gyro correction: bad marked', oname))
xl(tdh.dday([1 end]))

subplot(312)
plot(tdh.dday(goodi), tdh.ang(goodi),'g')
hold on
hh=plot(tdh.dday(goodi), tdh.ang(goodi),'k.') ;ms(hh,1)
title(sprintf('%s - gyro correction: good only', oname))
xl(tdh.dday([1 end]))


subplot(313)
plot(tdh.dday, tdh.numgood,'k')      
hold on
plot(tdh.dday(badi), tdh.numgood(badi),'r.')
xl(tdh.dday([1 end]))
title('number of good points')

if exist('cruisename','var')
   subtitle(sprintf('%s %s heading correction', cruisename, oname))
else
   subtitle(sprintf('%s heading correction', oname))
end


orient tall
eval(sprintf('print -dpsc gyro_%s_dh.ps', oname))





