function  plot_qc(qcinfo, fname, ddrange)
%     plot_qc(qcinfo, 'heading',  [6 7])
% 
% eg. cfg = get_uhdascfg('cfgbase','km0401')
%     qcinfo = get_qcinfo(cfg)
%     start_dday = 6
%     plot_qc(qcinfo, 'heading', [start_dday start_dday + 1])
%    

%% assumes first row is consistent dday

if isemptys(qcinfo)
   fprintf('qcinfo is empty.  not running plot_qc\n')
   return
end
   

figure
rownum = zeros(length(qcinfo),1);

for ii=1:length(qcinfo)
   rows = qcinfo(ii).rstruct.rows;
   if ismember(fname, rows)
      rownum(ii) = find(ismember(rows, fname));
   end
end


colors = mk_cmap(length(find(rownum ~= 0)));

plot_these = find(rownum ~= 0);

xx_lim = [];
yy_lim = [];


for ii=1:length(plot_these)
   index = plot_these(ii);
%   rdata = tr2ilist(qcinfo(index).trinfo, 'timerange', ddrange,...
%                    'instrument', qcinfo(index).instrument);
   rdata = tr2ilist(qcinfo(index).trinfo, 'timerange', ddrange);
   if isemptys(rdata)
      continue
   end
   qcmask = qc_rbin(rdata,  qcinfo(index).rstruct);
   badi = find(qcmask(rownum(index),:));
   hold on
   if ~isemptys(badi)
      hh=plot(rdata(1,badi), rdata(rownum(index),badi), 'ro');
      set(hh,'color',.5*colors(ii,:), 'markersize',4)
   end
   hh = plot(rdata(1,:), rdata(rownum(index),:));
   set(hh,'color', colors(ii,:));

   if isemptys(xx_lim)
      xx_lim = rdata(1,[1 end]);
   else
      xx_lim(1) = min(xx_lim(1), rdata(1,1));
      xx_lim(2) = max(xx_lim(1), rdata(1,end));
   end
   
   if isemptys(yy_lim)
      yy_lim = [min(rdata(rownum(index),:)),max(rdata(rownum(index),:))];
   else
      yy_lim(1) = min(yy_lim(1), min(rdata(rownum(index),:)));
      yy_lim(2) = max(yy_lim(1), max(rdata(rownum(index),:)));
   end


end


if isemptys(xx_lim) | isemptys(yy_lim)
   fprintf('no data in ddrange %f to %f\n', ddrange(1), ddrange(2))
else
   dx = .05*diff(xx_lim);
   dy = .05*diff(yy_lim);
   set(gca, 'xlim', xx_lim);
   set(gca, 'ylim', yy_lim+ [-dy dy]);
   
   
   %% make a legend
   
   for ii=1:length(plot_these)
      index = plot_these(ii);
      x = xx_lim(1) + dx/2;
      y = yy_lim(end) - ii*dy;
      hh = plot([x x+dx]', [y y]');
      set(hh,'color',.5*colors(ii,:), 'linewidth', 2);
      hh = text(x + dx*1.5, y, qcinfo(index).instrument);
      click_rgbk(hh,'c',colors(ii,:))   
   end
   
   
   xlabel('logging PG decimal day')
   ylabel(fname)
   title(sprintf('QC plot for %s', fname))
   
   orient landscape
   eval(sprintf('print -dpsc -append %s', fname))
end
