function [data, config] = restruct(instclass, dd, varargin)
% [dd, config] = restruct(instclass, data, varargin)
% index is indices: step, range, or list
% rotate velocities into beam coords, 
% reshape things to output one structure with vel, amp, sw, pg, temperature,etc
% varargin can be  'irange' (index start and end)
%                  'ilist' (indices)
%                  'step' (subsample whole thing by this)
%                  'ddayrange' (ddrange start and end)

cfg.irange =  [];
cfg.ilist =     [];
cfg.ddayrange = [];
cfg.step =   1;
cfg = fillstruct(cfg, varargin);


index = [1:length(dd.dday)]';

if (~isemptys(cfg.irange)),index = [cfg.irange(1):cfg.step:cfg.irange(2)]; end
if (~isemptys(cfg.ilist)),       index = cfg.ilist; end
if (~isemptys(cfg.ddayrange))
   [junk, first_index] = min(abs(dd.dday - cfg.ddayrange(1)));
   [junk, last_index]  = min(abs(dd.dday - cfg.ddayrange(end)));
   index = [first_index:cfg.step:last_index]';
end


if (isfield(dd, 'vel'))
   data.vel1=squeeze(dd.vel(:,1,index));
   data.vel2=squeeze(dd.vel(:,2,index));
   data.vel3=squeeze(dd.vel(:,3,index));
   data.vel4=squeeze(dd.vel(:,4,index));
end

if (isfield(dd, 'amp'))
   data.amp1=squeeze(dd.amp(:,1,index));
   data.amp2=squeeze(dd.amp(:,2,index));
   data.amp3=squeeze(dd.amp(:,3,index));
   data.amp4=squeeze(dd.amp(:,4,index));
end


if (isfield(dd, 'sw'))
   data.sw1=squeeze(dd.sw(:,1,index));
   data.sw2=squeeze(dd.sw(:,2,index));
   data.sw3=squeeze(dd.sw(:,3,index));
   data.sw4=squeeze(dd.sw(:,4,index));
end

data.heading = dd.heading(index);
data.pitch   = dd.pitch(index);
data.roll = dd.roll(index);
data.dday = dd.dday(index);
data.temperature = dd.temperature(index);

if (isfield(dd, 'soundspeed'))
   data.soundspeed = dd.soundspeed(index);
end
if (isfield(dd, 'meas_soundspeed'))
   data.meas_soundspeed = dd.meas_soundspeed(index);
end
if (isfield(dd, 'aduheading'))
   data.aduheading = dd.aduheading(index);
end


data.ens_num = dd.ens_num(index);

if (isfield(dd, 'enu'))
   data.umeas=squeeze(dd.enu(:,1,index));
   data.vmeas=squeeze(dd.enu(:,2,index));
   data.wmeas=squeeze(dd.enu(:,3,index));
end

if (isfield(dd, 'xyze'))
   data.emeas=squeeze(dd.xyze(:,4,index));
end


if (isfield(dd, 'nav'))
   if (isfield(dd.nav, 'txy1'))
      data.mps = ll2mps(dd.nav.txy1(1,index), ...
                        dd.nav.txy1(2,index), dd.nav.txy1(3,index))';
      data.lon = dd.nav.txy1(2,:)';
      data.lat = dd.nav.txy1(3,:)';
   end
end


data.config =  dd.config;
if (isfield(dd,'bt'))
   data.bt.vel   = dd.bt.vel(:,index);
   data.bt.pg  = dd.bt.pg(:,index);
   data.bt.range  = dd.bt.range(:,index);
end


%initialize some default titles and limits 

config.title.vel1            = 'NB BEAM 1 VELOCITY (m/s)';
config.title.vel2            = 'NB BEAM 2 VELOCITY (m/s)';
config.title.vel3            = 'NB BEAM 3 VELOCITY (m/s)';
config.title.vel4            = 'NB BEAM 4 VELOCITY (m/s)';

config.title.sw1            = 'NB BEAM 1 SPECTRAL WIDTH';
config.title.sw2            = 'NB BEAM 2 SPECTRAL WIDTH';
config.title.sw3            = 'NB BEAM 3 SPECTRAL WIDTH';
config.title.sw4            = 'NB BEAM 4 SPECTRAL WIDTH';

config.title.amp1            = 'NB BEAM 1 AMPLITUDE';
config.title.amp2            = 'NB BEAM 2 AMPLITUDE';
config.title.amp3            = 'NB BEAM 3 AMPLITUDE';
config.title.amp4            = 'NB BEAM 4 AMPLITUDE';

config.title.umeas            = 'NB MEAS U (m/s)';
config.title.vmeas            = 'NB MEAS V (m/s)';
config.title.wmeas            = 'NB MEAS W (m/s)';
config.title.emeas            = 'NB ERROR VEL (m/s)';


config.minclipval.vel1 = -6;    config.maxclipval.vel1 =  6;
config.minclipval.vel2 = -6;    config.maxclipval.vel2 =  6;
config.minclipval.vel3 = -6;    config.maxclipval.vel3 =  6;
config.minclipval.vel4 = -6;    config.maxclipval.vel4 =  6;


config.minclipval.amp1 = 10;    config.maxclipval.amp1 =  200;
config.minclipval.amp2 = 10;    config.maxclipval.amp2 =  200;
config.minclipval.amp3 = 10;    config.maxclipval.amp3 =  200;
config.minclipval.amp4 = 10;    config.maxclipval.amp4 =  200;


config.minclipval.sw1 = 20;    config.maxclipval.sw1 =  260;
config.minclipval.sw2 = 20;    config.maxclipval.sw2 =  260;
config.minclipval.sw3 = 20;    config.maxclipval.sw3 =  260;
config.minclipval.sw4 = 20;    config.maxclipval.sw4 =  260;

config.minclipval.umeas = -6;    config.maxclipval.umeas =  6;
config.minclipval.vmeas = -6;    config.maxclipval.vmeas =  6;
config.minclipval.wmeas = -6;    config.maxclipval.wmeas =  6;
config.minclipval.emeas = -6;    config.maxclipval.emeas =  6;


config.comment = '';