function d = fastread(instclass, filename, irange, second_set)
% data = fastread(instclass, filename, irange, second_set)
% for use in mk_rawens

h = mex_osread('open', filename);
c = mex_osread('config', h);
d.config = fill_config(c, second_set);
nread = mex_osread('read', h, irange(1), irange(2));
if second_set
   a = mex_osread('arrays2', h);
else
   a = mex_osread('arrays', h);
end
d.vel = a.vel;
d.cor = a.cor;
d.amp = a.amp;

d.bt = mex_osread('bt', h);


s = mex_osread('scalars', h);
d.temperature = s.temperature;
d.ens_num = s.ens_num;
d.dday = s.dday;         % yearbase is current year <---------
d.heading = s.heading;
d.pitch = s.pitch;
d.roll = s.roll;
d.soundspeed = s.soundspeed;
NC = d.config.ncells;
d.depth =  [d.config.bin1distance + (0:(NC-1)) * d.config.cell]';

mex_osread('close', h);

%---------------------------------------------------------------

function config = fill_config(c, second_set)
% Maybe this can be stripped down much more.
% It now has a bunch of stuff from read_buf that could
% be replaced by simply passing in fixed configuration
% info, such transducer parameters--that is, anything
% that cannot possibly change in a given data set.  We
% should not be extracting it with each data read!

FL = c.FixedLeader(second_set + 1);
config.cell = FL.CellSize / 100;
config.ncells = FL.NCells;
config.nbeams_used = FL.NBeams;
config.npings = FL.NPings;
config.blank = FL.Blank / 100;
config.mode = FL.SPM;
%config.code_reps = FL.NCodeReps;
config.coord_transf = FL.EX;
config.head_align = FL.EA / 100;
config.head_bias = FL.EB / 100;
config.bin1distance = FL.Bin1Dist / 100;
config.pulse = FL.Pulse / 100;
% config.lag = FL.TransLag / 100;
config.ids = c.data_IDs;
config.bt_was_on = any(config.ids == hex2dec('0600'));

config.sysconfig = decode_sysconfig(FL.sysconfig);

  if (config.mode == 1)
      config.pingtype = 'bb';
   elseif (config.mode == 10)
      config.pingtype = 'nb';
   end


   % make these have a more uniform nomenclature
   config.convex  = config.sysconfig.convex;
   if (config.sysconfig.up)
      config.orientation = 'up';
   else
      config.orientation = 'down';
   end
   config.angle = config.sysconfig.angle;
   config.frequency = config.sysconfig.frequency;

  % coordinate system
   instrument = 2^3;
   ship = 2^4;
   earth = 2^3+2^4;
   if (bitand(config.coord_transf, instrument) == 0) & ...
          (bitand(config.coord_transf, ship) == 0)
      config.coordsystem = 'beam';  % no transformation   xxx00xxx
   elseif (bitand(config.coord_transf, earth) == earth)
      config.coordsystem = 'earth';                    %  xxx11xxx
   elseif (bitand(config.coord_transf, instrument) == instrument)
      config.coordsystem = 'xyz';                      %  xxx10xxx
   elseif (bitand(config.coord_transf,ship) == ship)
      config.coordsystem = 'ship';                    %  xxx11xxx
   else
      fprintf('coordinate system error (EX) in %s\n', mfilename)
      return
   end



%---------------------------------------------------

function c = decode_sysconfig(sc)
%% Not quite the same as in read_buf, because it takes
%% an integer instead of the separate bytes.
l = bitand(sc, 256);
m = bitshift(sc, -8);
freqs = [75, 150, 300, 600, 1200, 2400, 38];
freqbits = bitand(l, 7);
c.frequency = freqs(1 + freqbits);
c.up =  bitand(bitshift(l, -7), 1);
c.convex = bitand(bitshift(l, -3), 1);

angles = [15, 20, 30, NaN];
anglebits = bitand(m, 3);
c.angle = angles(1 + anglebits);





