function a = avg(instclass, r, a, p)
% r is a raw data structure; a is initialized for one profile
% p is a structure of parameters that control the averaging
%  and editing (added 2002/03/06)
%
% see load_raw for setup

%JH cut this out of bb/load_raw.m so I could use it to write nb/load_raw
%     (it may be too specific)

%need to get velocities into geographic coordinates before doing this:

if (~isfield(p, 'velpermute'))
   p.velpermute = [1 2 3 4];
end


if (p.verbose)
   [jdata, jconfig] = restruct(instclass, r);
   f1;   rlookit(jdata, jconfig, 1, [3 8]);
end
   

%now set up the editing criteria
%get umeas, vmeas to do the editing in geographic coordinates

%add beam angle here

rr = r;
[rr.enu rr.xyze] = ...
    beam_enu(rr.vel(:, p.velpermute, :), rr.heading + p.h_align, 0, 0,...
             'error', 1, 'angle', p.beamangle);
[rr, rrconfig] = restruct(instclass, rr);
rr.depth = rr.depth(:)';

%% apply pingediting if requested
rr.cuv = rr.umeas + i*rr.vmeas;
if (p.pingedit)
   [instpflag, vprofile, trefl] =  ...
       pingedit(rr, 'cfg', p.editcfg, 'fieldname', 'cuv');
   if (p.verbose)
      fprintf('applying pingedit\n')
   end
else
   instpflag = zeros(size(rr.cuv));
   if (p.verbose)
      fprintf('not applying pingedit\n')
   end
end


if (p.verbose)
   [jdata, jconfig] = restruct(instclass, r);
   rrconfig.var_range = [-.5,.5];
   rr.umeas(logical(instpflag)) = NaN;
   rr.vmeas(logical(instpflag)) = NaN;
   %f2;   rrlookit(rr, rrconfig, [3 8])
   keyboard
   paused
end


%rotate the velocities to E, W, up, errvel
enu =  beam_enu(r.vel(:, p.velpermute, :), r.heading + p.h_align, 0, 0,...
                'error', 1, 'angle', p.beamangle);

%rotate bottom track data to E, W, up, errvel
r.bt.vel = ...
    beam_enu(r.bt.vel(p.velpermute, :), r.heading + p.h_align, 0, 0,...
             'error', 1, 'angle', p.beamangle);

%% get mask
no_vel_mask = logical(instpflag) | isnan(squeeze(enu(:, 1, :)));

ev_mask = abs(enu(:, 4, :)) > p.max_ev;

% Calculate other masks and combine with logical or.
bmask = zeros(size(no_vel_mask));


%%% bad  implimention of  bottom tracking test
%%% (os) if r.config.bt.BP > 0,
if ~(isnan(sum(r.bt.pg(:,1)))) %no bottom tracking: mask
   btd = min(r.bt.range);
   dd = r.depth.' - r.config.tr_depth;
   bmask = mask_below(dd, btd);
else
   for ii = 1:4
       [bm1, b] = bigtarget(squeeze(r.amp(:, 1, :)), ...
            p.bigtarget_amp, p.bigtarget_offset, p.bigtarget_avg);
       bmask = bmask | bottommask(bm1, b, 0.85, 1);
   end
end

bad_mask = squeeze(ev_mask) | bmask;
% The following could probably be made more efficient by
% simply passing the bad mask into refavg, and optimizing
% refavg to use a single set of masks and counts for all
% iterations.


% on all 4 components at once.

% Eliminate from the reference layer levels with insufficient
% samples.
pg = 100 *  sum(~(bad_mask | no_vel_mask), 2)  / a.num_pings;
ipg = find(pg(p.ilayer) > p.min_pg);

if length(ipg) < 2,  % No decent ref layer--just average.
   for ii = 1:4
      uu = squeeze(enu(:, ii, :));
      uu(bad_mask) = NaN;
      a.vel(:,ii) = mstdgap(uu, 2);
   end
else
   for ii = 1:4
      uu = squeeze(enu(:, ii, :));
      uu(bad_mask) = NaN;
      if (p.use_refavg)
         [uu, ulayer] = refavg(uu, p.ilayer(ipg), 5);
         a.vel(:,ii) = uu + mstdgap(ulayer); % simple mean for now
         if (p.verbose)
            fprintf('applying refavg\n')
         end
      else
         a.vel(:,ii) = mstdgap(uu, 2);
         if (p.verbose)
            fprintf('not applying refavg\n')
         end
      end
   end
end


a.amp = round(mean(r.amp, 3));

%sontek doesn't have this
a.pg(:,:) = 0;
a.pg(:, 4) = round(pg);
a.sw = 0*a.pg;


%bottom track
a.bt = mk_bt(r, 1);

%------------------------------
function mask = mask_below(depth, bottom)
np = length(bottom);
nd = length(depth);
depth = depth(:, ones(np, 1));
bottom = bottom(ones(nd, 1), :);

mask = depth > bottom;


