function [nbytes, last_index, config] = get_config(instclass,params, cfg, file)
% read by @son/read.m
% read the file; look for errors
% all "fopen" must have matching "fclose"

% the SON processors write little-endian
fid = fopen(file, 'r', 'ieee-le');  

% tests whether BT is present; tests whether there is a file header
[buf, config] = read_buf(instclass, fid, params);  % First time: read config.

if (~config.sysconfig.getfilehead) %then it is a uh raw file
   params.fileheadlength = 0;
end


proflength = params.profheadlength + 4*config.nbeams_used*config.ncells + 2; % 2 for vel, 1 each std, amp

%nbytes: number of bytes for each record,  INCLUDING 2 bytes for checksum

if (~isfield(config, 'nbytes'))
   fprintf('\n\n ERROR: in get_config: "nbytes" is not defined.\n')
   fprintf('This may be because there is no header in the data: did you add\n')
   fprintf('a varargin pair for ''header'' in read??\n')
   nbytes = [];
   last_index = [];
   config = [];
   return
end


nbytes = config.nbytes + 2;  %used to be "proflength".  Keep names from ../rdi 

% How many profiles are in the file?
fseek(fid, 0, 'eof');
file_nbytes = ftell(fid);
last_index =  fix((file_nbytes - params.fileheadlength)/nbytes);
fclose(fid);                              

if (~cfg.back_to_front)    %start at the front

   fid = fopen(file, 'r', 'ieee-le');  

   % ok to keep reading, since it's the same direction   
   fseek(fid, 0, 'bof');
   keepreading = 1;
   
   last_good_index = 0;
   while keepreading
      buf = [];
      % go to the beginning of the next data record
      fseek(fid, params.fileheadlength + nbytes*last_good_index, 'bof'); 
      try
         [buf] = read_buf(instclass, fid, params, config);
      catch
         if (cfg.verbose)
            fprintf('exited read early: last good index is %d \n', last_index)
         end
         break
      end 
      
      if (~isstruct(buf))
         if (cfg.verbose)
            fprintf('buf returns %s\n',buf)
         end
         break
      end
      last_good_index = last_good_index + 1;
      if (last_good_index == last_index)
         break
      end
   end
   fclose(fid);
   
else     %  cfg.back_to_front is true 
   % the Sontek processors write little-endian
   fid = fopen(file, 'r', 'ieee-le');  

   fseek(fid, 0, 'eof');        % total number of bytes
   len_file = ftell(fid);       % bytes remaining in record up to but not 
   last_index = floor((len_file - params.fileheadlength) / nbytes); 
   % revise the last good index
   for last_good_index = last_index:-1:1
      buf = [];
      fseek(fid, params.fileheadlength + nbytes * (last_good_index - 1), 'bof'); %use ilist
      if (cfg.verbose), last_good_index, end
      try
         [buf] = read_buf(instclass, fid, params, config);
      catch
         if (cfg.verbose)
            fprintf('exited read early at index %d (out of %d):\n', ...
                    last_good_index,  last_index)
         end
      end 
      if (~isstruct(buf))
         if (cfg.verbose)
            fprintf('buf returns %s\n',buf)
         end
      elseif isstruct(buf)
         break
      else
         warning('buf is not struct or string')
      end
   end
   
   % update these things
   fclose(fid);
end

last_index = last_good_index;
