function [buf, config] = read_buf(instclass, fid, params, config)
%
% son from recorder is:
% header (struct1, struct2, struct3)
% bt?? , profile1, checksum
% bt?? , profile2, checksum
% ...
% bt?? , profileN, checksum
%   
% son from binary stream is:
% bt?? , profile1, checksum         
% bt?? , profile2, checksum
% ...
% bt?? , profileN, checksum


out            = params.out;
ofs_bt    = 18;
cs_offset = hex2dec('A596');  %Matlab R13 doesn't like 0xa596


if (nargin == 3)
   config = configinit(instclass);   
end

% offsets from p_offset into profile header
% put these in config:
ofs_nbeams                   = 26;
ofs_sensor_orientation       = 27;
ofs_coord_system             = 29;
ofs_ncells                   = 30;
ofs_cellsize                 = 32;
ofs_blankdistance            = 34;
ofs_avginterval              = 36;
ofs_npings                   = 38;

ofs_nbytes                   = 2;

ofs_profilenumber            = 14;
ofs_profiletime              = 18;
ofs_meanheading              = 40; %ts(:,1)
ofs_meanpitch                = 42; %ts(:,2)
ofs_meanroll                 = 44; %ts(:,3)
ofs_meantemp                 = 46; %ts(:,4)
ofs_meanpress                = 48; %ts(:,5)
ofs_soundspeed               = 56;
ofs_xflux                    = 60; %ts2(:,1)
ofs_yflux                    = 62; %ts2(:,2)
ofs_voltage                  = 75; %ts2(:,5)
ofs_zflux                    = 76; %ts2(:,3)
ofs_compasstemp              = 78; %ts2(:,4)
ofs_noise                    = 68;



if (nargin == 3)
   buf = [];
   
   %initialize config so the fields are in the right order!!
     
   % reading for the first time
   fseek(fid, 0, 'bof');
   struct_indicator = fread(fid, 1, 'uchar');
   if (struct_indicator == 165)  %0xA5, velocity profile header
      config.sysconfig.getfilehead = 0;  % get nbeams, ncells from prof header
      params.fileheadlength = 0;
   else
      config.sysconfig.getfilehead = 1;
   end
   
   
   %this means there will be an empty buffer returned when getting config
   buf = [];
   fseek(fid, 0, 'bof');

   %override if there are transformation values
   config.press_xform = [0 0 0];  %[PressScale, PressOffset, Press_scale_2]
   f_options = [3 1.5 750 500 250];
   c_options = {'beam', 'xyz', 'enu'};
   
   if (config.sysconfig.getfilehead) %ladcp
      
      
      config.sysconfig.sensor         = read_sencfg(instclass, fid);
      config.sysconfig.adpsetup       = read_adpcfg(instclass, fid);
      config.sysconfig.user           = read_usercfg(instclass, fid);
                     
      
      
      % extract specific stuff here:
      if (~isemptys(params.yearbase))
         config.yearbase = params.yearbase;
      else
         config.yearbase        =  config.sysconfig.user.begindeploymentdatetime(1);
      end

      config.nbeams_used     =  config.sysconfig.sensor.nbeams;
      config.ncells          =  config.sysconfig.user.ncells;
      config.cell            =  config.sysconfig.user.cellsize;
      config.blank           =  config.sysconfig.user.blankdistance;
      config.pulse           =  config.sysconfig.adpsetup.pulselength;
      config.lag             =  config.sysconfig.adpsetup.lag;
      
      config.bin1distance = config.blank + (config.cell + config.pulse)/2;

      if config.sysconfig.sensor.pressinstalled
         config.press_xform = ...
             [config.sysconfig.sensor.pressscale 
              config.sysconfig.sensor.pressoffset
              config.sysconfig.sensor.pressscale2];
      end


      %now put these in config.
      % convex or concave
      if (config.sysconfig.sensor.beamgeometry ~= 3)
         fprintf('WARNING: this sontek instrument may not have 4 beams\n')
         config.convex = NaN;
      else
         config.convex = 1;
      end
     
      
      % up or down
      if (config.sysconfig.sensor.sensororientation == 0)
         config.orientation = 'down';
      elseif (config.sysconfig.sensor.sensororientation == 1)
         config.orientation = 'up';      
      else
         config.orientation = 'side';            
         fprintf(' WARNING: this sontek instrument is pointing sideways!\n')
      end
      % angle
      config.angle = config.sysconfig.sensor.slantangle;
      % compass offset
      config.head_align = config.sysconfig.sensor.compassoffset;
      % frequency
      config.frequency = f_options(config.sysconfig.sensor.adptype+1);
      % coordinate system
      config.coordsystem = c_options{config.sysconfig.user.coordsystem+1};

   else   %%  (raw data logging)

      fseek(fid, ofs_nbeams, 'bof');        config.nbeams_used        = fread(fid, 1, 'char');
      fseek(fid, ofs_coord_system, 'bof'); 
        config.coordsystemnum = fread(fid, 1, 'char');
        config.coordsystem = c_options{config.coordsystemnum +1};
      fseek(fid, ofs_ncells, 'bof');        config.ncells            = fread(fid, 1, 'uint16');
      fseek(fid, ofs_cellsize, 'bof');      config.cell              = fread(fid, 1, 'uint16')/100;
      fseek(fid, ofs_blankdistance, 'bof'); config.blank             = fread(fid, 1, 'uint16')/100;
      %fseek(fid, ofs_avginterval, 'bof');   config.avg_interval     = fread(fid, 1, 'uint16');
      fseek(fid, ofs_npings, 'bof');        config.num_pings         = fread(fid, 1, 'uint16');

      fseek(fid, ofs_profiletime, 'bof');    
      if (isemptys(params.yearbase))
         config.yearbase = params.yearbase;
      else
         yearbase =  fread(fid, 1, 'int16');       % year
         config.yearbase = yearbase;
      end
      
      fseek(fid, ofs_sensor_orientation, 'bof'); 
      sensor_orientation = fread(fid, 1, 'char');
      if (sensor_orientation == 0)
         config.orientation = 'down';
      elseif (sensor_orientation == 1)
         config.orientation = 'up';      
      else
         config.orientation = 'side';            
         fprintf(' WARNING: this sontek instrument is pointing sideways!\n')
      end

      config.pulse = config.cell;  %% HARDWIRED for Kilo Moana
      config.bin1distance = config.blank + (config.cell + config.pulse)/2;
      config.head_align = 0;               
      config.head_bias = 0;            
      config.convex  = 1;                    
      config.angle = 32;                     
      config.frequency = 130;  % as per Daryl Slocum, 2003/12/12
   end
   
   %assume no data structures other than possibly Bottom track
   %check whether BT is on
   wt_okchecksum = 0;
   bt_okchecksum = 0;

   %assume NO BT first:
   % DO  NOT include checksum;
   %number of bytes per data record (% 2 for vel, 1 each std, amp)  
   wt_prof_nbytes = params.profheadlength + 4*config.ncells*config.nbeams_used;
   %%% test checksum 
   fseek(fid,  params.fileheadlength, 'bof');
   buf = fread(fid, wt_prof_nbytes, 'uint8');
   test_cs = rem(sum(buf) + cs_offset, 65536);
   cs = fread(fid, 1, 'uint16');
   if cs == test_cs,
      wt_okchecksum = 1;
   end
   
   % DO  NOT include checksum;
   bt_prof_nbytes = wt_prof_nbytes + ofs_bt;
   % now try including BT
   fseek(fid, 0, 'bof');
   buf = fread(fid, bt_prof_nbytes, 'uint8');
   test_cs = rem(sum(buf) + cs_offset, 65536);
   cs = fread(fid, 1, 'uint16');
   if cs == test_cs,
      bt_okchecksum = 1;
   end

   if ((~wt_okchecksum) & (~bt_okchecksum))
      buf = [];
      fprintf('FATAL: checksum failed for both watertrack and bottomtrack\n')
      return
   elseif wt_okchecksum
      config.nbytes = wt_prof_nbytes;
      config.bt_was_on = 0;
   elseif bt_okchecksum,
      config.nbytes = bt_prof_nbytes;
      config.bt_was_on = 1;
   else
      buf = [];
      fprintf('FATAL: checksum test passed with and without BT on\n')
      return
   end
   
   
   return
end

%otherwise, continue to get data
   
NC = config.ncells;
NB = config.nbeams_used;
ofs_vel   = params.profheadlength;
ofs_amp   = params.profheadlength + 3*NB*NC;
ofs_std   = params.profheadlength + 2*NB*NC;

%current position, at the start of the profile header
p_offset = ftell(fid);


%%% test checksum 
buf = fread(fid, config.nbytes, 'uint8');
test_cs = rem(sum(buf) + cs_offset, 65536);
cs = fread(fid, 1, 'uint16');
if cs ~= test_cs,
   buf = 'checksum failed';
   return
end

% read things from profile header
% start again where the profile header starts
fseek(fid, p_offset, 'bof');
fseek(fid, p_offset +  ofs_npings, 'bof');         buf.num_pings      = fread(fid, 1, 'uint16');
fseek(fid, p_offset +  ofs_profilenumber, 'bof');  buf.ens_num        = fread(fid, 1, 'int32');
fseek(fid, p_offset +  ofs_profiletime, 'bof');    

ymdhms = [NaN NaN NaN NaN NaN NaN];
ymdhms(1)  = fread(fid, 1, 'int16');       % year
ymdhms(3)  = fread(fid, 1, 'int8');        % day
ymdhms(2)  = fread(fid, 1, 'int8');        % month
ymdhms(5)  = fread(fid, 1, 'int8');        % minute
ymdhms(4)  = fread(fid, 1, 'int8');        % hour
ymdhms(6) =  fread(fid, 1, 'int8')*0.01+ fread(fid, 1, 'int8');   %sec100*0.01 + second

%make yearday
if isemptys(config.yearbase) %if it wasn't set before, set it now.
   config.yearbase = ymdhms(1); %override with correct value
end
buf.dday = to_day( config.yearbase, ymdhms);


fseek(fid, p_offset + ofs_profilenumber, 'bof');      buf.ens_num = fread(fid, 1, 'ulong');

if out.ts,
   fseek(fid, p_offset +  ofs_meanheading, 'bof'); buf.heading    = fread(fid, 1, 'int16')*0.1;
   fseek(fid, p_offset +  ofs_meanpitch, 'bof');   buf.pitch      = fread(fid, 1, 'int16')*0.1;
   fseek(fid, p_offset +  ofs_meanroll, 'bof');    buf.roll       = fread(fid, 1, 'int16')*0.1;
   fseek(fid, p_offset +  ofs_meantemp, 'bof');    buf.temperature= fread(fid, 1, 'int16')*0.01;
   fseek(fid, p_offset +  ofs_meanpress, 'bof');   press_counts  = fread(fid, 1, 'uint16')*0.01;
   buf.tr_pressure = config.press_xform(1) + ...
       config.press_xform(2)*press_counts + ...
       config.press_xform(3)*press_counts.^2;
   fseek(fid, p_offset +  ofs_soundspeed, 'bof');  buf.soundspeed = fread(fid, 1, 'uint16')*0.1;
end


if (out.einfo), %compass, power, and noise info
   fseek(fid, p_offset +  ofs_xflux, 'bof');       buf.xflux          = fread(fid, 1, 'int16')*0.01;
   fseek(fid, p_offset +  ofs_yflux, 'bof');       buf.yflux          = fread(fid, 1, 'int16')*0.01;
   fseek(fid, p_offset +  ofs_zflux, 'bof');       buf.zflux          = fread(fid, 1, 'int16')*0.01;
   fseek(fid, p_offset +  ofs_compasstemp, 'bof'); buf.ctemperature   = fread(fid, 1, 'int16')*0.1;
   fseek(fid, p_offset +  ofs_voltage, 'bof');     buf.voltage        = fread(fid, 1, 'int8')*0.2;
   fseek(fid, p_offset + ofs_noise, 'bof');        buf.noise = fread(fid, [1,4], 'uint8');
end


%now we're past the profile header
if (out.bt)
   if (config.bt_was_on) %bt is next
      fs = fseek(fid, p_offset + params.profheadlength, 'bof');
      buf.bt = read_bt(instclass, fid, config); %range and velocity only.
      buf.bt.pg = repmat(buf.bt.status, 4, 1);  % for avg.m (temporary)
   else
      if out.bt,          %output NaNs
         buf.bt.status  = -1;
         buf.bt.vel     = NaN*zeros(config.nbeams_used,1);
         buf.bt.pg      = NaN*zeros(config.nbeams_used,1); % for avg.m (temporary)
         buf.bt.range   = buf.bt.vel;
      end
   end
end


% now output the other things requested
if out.vel,
   fs = fseek(fid, p_offset + config.bt_was_on*ofs_bt + ofs_vel, 'bof');
   buf.vel = fread(fid, [NC,NB], 'int16') * 1e-3;
end


% "amp" is counts
if out.amp,
   fs = fseek(fid, p_offset + config.bt_was_on*ofs_bt + ofs_amp, 'bof');
   buf.amp = fread(fid, [NC,NB], 'uint8');
end

