function  profhdr = read_prfhdr(instclass, fid, out)
%profhdr = read_profhdr(instclass, fid)     %% read it all
%profhdr = read_profhdr(instclass, fid, out)    %%output only what is requested
%
% currently available:
% out.ts =  [0 or 1]            heading, pitch, roll, temo, soundspeed, pressure (counts)
% out.einfo = [0 1]             xflux, yflux, zflux, voltage, noise, compasstemp

if (nargin == 2)
   out.ts = 0;
   out.einfo = 0;
end

%offsets in case we need them
ofs.npings                   = 38;
ofs.profilenumber            = 14;
ofs.profiletime              = 18;

if out.ts,
   ofs.meanheading           = 50;
   ofs.meanpitch             = 51;
   ofs.meanroll              = 52;
   ofs.meantemp              = 46;
   ofs.soundspeed            = 56;
   ofs.rawmeanpressure       = 48;
end

if (out.einfo), %compass, power, and noise info
   ofs.xflux                 = 60;
   ofs.yflux                 = 62;
   ofs.zflux                 = 76;
   ofs.compasstemp           = 78;
   ofs.voltage               = 75;
   ofs.noise                 = 68;
end


if ((nargin < 2) | (nargin > 3))
   help(mfilename)
   error('2 arguments reads it all, third argument specifies a subset.')
end

vv = 0;
start = ftell(fid);

if nargin == 2
   %% read everything                                                                                           %   byte
   %                                                                                                               value  offset
   profhdr.syncchar                = fread(fid, 1, 'uchar');          %  0xa5                                   %   1        0
   profhdr.datatype                = fread(fid, 1, 'uchar');          %  0x10                                   %   1        1
   profhdr.nbytes                  = fread(fid, 1, 'int16');          %  number of bytes in profile header      %   2        2
   profhdr.serialnumber            = fread(fid, [1 10], 'char');      % serialnumber                            %   1*10     4
   if (vv),  ftell(fid),    end                               %  %%%% ------------offset 14                     %           
   %                                                                                                            %           
   profhdr.profilenumber           = fread(fid, 1, 'int32');          %  from start of data collection          %   4        14
   profhdr.profiletime = read_date(instclass, fid, ftell(fid), 'DateTimeType');   %  at start of avg interval   %   8        18
   profhdr.nbeams                  = fread(fid, 1, 'char');           %  2/3/4                                  %   1        26
   profhdr.sensororientation       = fread(fid, 1, 'char');           %  0-down; 1-up; 2-side                   %   1        27
   profhdr.tempmode                = fread(fid, 1, 'char');           %  0-user value  1- measured              %   1        28
   profhdr.coordsystem             = fread(fid, 1, 'char');           %  0-beam; 1-xyz; 2-enu                   %   1        29
   if (vv),  ftell(fid),    end                               %%%%%%%% -------------  offset 30                 %           
   %                                                                                                            %           
   profhdr.ncells                  = fread(fid, 1, 'uint16');         %  # of depth cells in profile            %   2        30
   profhdr.cellsize                = fread(fid, 1, 'uint16')/100;     %  in cm     (now m)                      %   2        32
   profhdr.blankdistance           = fread(fid, 1, 'uint16')/100;     %  in cm     (now m)                      %   2        34
   profhdr.avginterval             = fread(fid, 1, 'uint16');         %  in s                                   %   2        36
   profhdr.npings                  = fread(fid, 1, 'uint16');         %  averaged in profile                    %   2        38
   profhdr.meanheading             = fread(fid, 1, 'int16')/10;       %  in 0.1  deg      (now deg)             %   2        40
   profhdr.meanpitch               = fread(fid, 1, 'int16')/10;       %  in 0.1  deg      (now deg)             %   2        42
   profhdr.meanroll                = fread(fid, 1, 'int16')/10;       %  in 0.1  deg      (now deg)             %   2        44
   profhdr.meantemp                = fread(fid, 1, 'int16')/100;          %  in 0.01 deg                        %   2        46
   profhdr.rawmeanpressure         = fread(fid, 1, 'uint16');         %  in counts (0-65535)                    %   2        48
   profhdr.stdheading              = fread(fid, 1, 'uchar')/10;       %  in 0.1  deg      (now deg)             %   1        50
   profhdr.stdpitch                = fread(fid, 1, 'uchar')/10;       %  in 0.1  deg      (now deg)             %   1        51
   profhdr.stdroll                 = fread(fid, 1, 'uchar')/10;       %  in 0.1  deg      (now deg)             %   1        52
   profhdr.stdtemp                 = fread(fid, 1, 'uchar')/10;       %  in 0.1  deg      (now deg)             %   1        53
   profhdr.stdpress                = fread(fid, 1, 'int16');          %  in counts                              %   2        54
   profhdr.soundspeed              = fread(fid, 1, 'int16')/10;       %  in 0.1  m/s   (now m/s)                %   2        56
   profhdr.spare2                  = fread(fid, [1  2], 'uchar');     %  reserved, set to 0                     %   1*2      58
   %                                                                  % the following was originally a "status" %           
   %                                                                  %           array of 20 uchar             %           
   profhdr.xflux                   = fread(fid, 1, 'int16')/100;      % in 10s of nanotesla (now microtesla)    %   2        60
   profhdr.yflux                   = fread(fid, 1, 'int16')/100;          % in 10s of nanotesla (now microtesla)%   2        62
   profhdr.status1                 = fread(fid, [1 4], 'uchar');      %  contains what? status1[4];             %   1*4      64
   profhdr.noise                   = fread(fid, [1 4], 'uchar');      %noise[4];                                %   1*4      68
   profhdr.status2                 = fread(fid, [1 3], 'uchar');      %status2[3];                              %   1*3      72
   %                                                                                                            %           
   profhdr.voltage                 = fread(fid, 1, 'int8')/5;         %  in 0.2 v  (now volts)                  %   1        75
   profhdr.zflux                   = fread(fid, 1, 'int16')/100;      %  in 10s of nanotesla (now microtesla);  %   2        76
   profhdr.compasstemp             = fread(fid, 1, 'int16')/10;       %  in 0.1 deg (now deg)                   %   2        78
   if (vv),  ftell(fid),    end                                %%%%%%%%%% at the end                            %       
                                                                                                                % (total = 80)
                                                                                                                
                                                                                                                
else
   
   start = ftell(fid);
   
   fseek(fid, start + ofs.npings, 'bof');
   profhdr.npings                                    = fread(fid, 1, 'uint16');
   
   fseek(fid, start + ofs.profilenumber, 'bof');
   profhdr.profilenumber                             = fread(fid, 1, 'int32');
   
   fseek(fid, start + ofs.profiletime, 'bof');
   profhdr.profiletime                               = read_date(instclass, fid, ftell(fid), 'DateTimeType'); 
   
   if out.ts,
      fseek(fid, start + ofs.meanheading, 'bof');
      profhdr.meanheading                               = fread(fid, 1, 'int16')/10;
      
      fseek(fid, start + ofs.meanpitch, 'bof');
      profhdr.meanpitch                                 = fread(fid, 1, 'int16')/10;
      
      fseek(fid, start + ofs.meanroll, 'bof');
      profhdr.meanroll                                  = fread(fid, 1, 'int16')/10;
      
      fseek(fid, start + ofs.meantemp, 'bof');
      profhdr.meantemp                                  = fread(fid, 1, 'uint16')/100;
      
      fseek(fid, start + ofs.soundspeed, 'bof');
      profhdr.soundspeed                                = fread(fid, 1, 'int16')/10;
      
      fseek(fid, start + ofs.rawmeanpressure, 'bof');
      profhdr.rawmeanpressure                           = fread(fid, 1, 'uint16');
   end
   
   if (out.einfo), %compass, power, and noise info
      fseek(fid, start + ofs.xflux, 'bof');
      profhdr.xflux                                     = fread(fid, 1, 'int16')/100;
      
      fseek(fid, start + ofs.yflux, 'bof');
      profhdr.yflux                                     = fread(fid, 1, 'int16')/100;
      
      fseek(fid, start + ofs.zflux, 'bof');
      profhdr.zflux                                     = fread(fid, 1, 'int16')/100;
      
      fseek(fid, start + ofs.compasstemp, 'bof');
      profhdr.compasstemp                               = fread(fid, 1, 'int16')/10;
      
      fseek(fid, start + ofs.voltage, 'bof');
      profhdr.voltage                                   = fread(fid, 1, 'int8')/5;
      
      fseek(fid, start + ofs.noise, 'bof');
      profhdr.noise                                     = fread(fid, 1, 'uchar');
   end
end

   