function s_config = read_sencfg(instclass, fid)
%s_config = read_sencfg(instclass, fid)

if (nargin ~= 2)
   help(mfilename)
   error('requires 2 arguments')
end

vv = 0;

s_config.configtype           = fread(fid, 1, 'char');          %  type: 0x10
s_config.configver            = fread(fid, 1, 'char');          %  version   0x02                
s_config.nbytes               = fread(fid, 1, 'int16');         %  bytes in configuration        
s_config.datenums             = read_date(instclass, fid, ftell(fid), 'DateTimeType');  %  date created or last modified 
s_config.softwarevernum       = fread(fid, 1, 'uchar');         %  ver number of adp firmware    
s_config.dspsoftwarevernum    = fread(fid, 1, 'uchar');         %  ver number of dsp firmware    
s_config.boardrev             = fread(fid, 1, 'char');          %  electronics board revision    
if (vv), ftell(fid), end                                           %%%-------   offset 15

s_config.serialnumber         = fread(fid, [1 10], 'char');          %  serialnumber[10]
s_config.adptype              = fread(fid, 1, 'char');          %  0-3; 1-1.5; 2-750; 3-500;     
                                                                %      4-250;                   
s_config.nbeams               = fread(fid, 1, 'char');          %  2 or 3 or 4                   
s_config.beamgeometry         = fread(fid, 1, 'char');          %  0- 2 beams; 1- 3 beams        
                                                                %  2- 4 beams, 1 vertical        
                                                                %  3- 4 beams, janus             
if ( s_config.beamgeometry == 3)
   s_config.nbeams = 4;
else
   fprintf('WARNING: this software is only configured to deal with 4 beams\n')
   return
end
                                                                
if (vv), ftell(fid), end                                           %%% ------ offset 28

s_config.slantangle           = fread(fid, 1, 'int16')/10;      %  in 0.1 deg now deg)
s_config.sensororientation    = fread(fid, 1, 'char');          %  0-down; 1-up; 2-side          
s_config.compassinstalled     = fread(fid, 1, 'char');          %  0-no;   1-yes                 
s_config.recorderinstalled    = fread(fid, 1, 'char');          %  0-no;   1-yes                 
s_config.tempinstalled        = fread(fid, 1, 'char');          %  0-no;   1-yes                 
s_config.pressinstalled       = fread(fid, 1, 'char');          %  0-no;   1-yes                 
s_config.spare1               = fread(fid, 1, 'char');          % spare[1]   %  set to all 0s
if (vv), ftell(fid), end                                           %%% offset 36   

s_config.xformmat             = fread(fid, [1 16], 'int16');         %  from beam to xyz veloc. [16]  
s_config.compassoffset        = fread(fid, 1, 'int16');         %  degress to east of north      
if (vv), ftell(fid), end                                           %%% offset 70: --------

s_config.pressscale           = 1e-8*fread(fid, 1, 'int32');    %  nanobar*10^(-8) per count= dbar?
s_config.pressoffset          = 1e-5*fread(fid, 1, 'int32');    %  microbar*10^(-5) = dbar?
s_config.powersavemove        = fread(fid, 1, 'char');          %  if 1 adp turns off after 5 min
                                                                %    idle in command mode        
s_config.extsensorinstalled   = fread(fid, 1, 'char');          %  0-no;   1-yes                 
s_config.parossensorinstalled = fread(fid, 1, 'char');          %  0-no;   1-yes                 
s_config.ctdsensorinstalled   = fread(fid, 1, 'char');          %  0-no;   1-yes                 
s_config.pressscale2          = 1e-12*fread(fid, 1, 'int16');   % 10^12*pico dbar per count = dbar
s_config.spare2               = fread(fid, [1 12], 'char');          %      spare2[12] set to all 0s 

if (vv), ftell(fid), end                                           %%% offset 96: --------

