function [enu, xyze] = son_beam_enu(vel, varargin);
% function [enu, xyze] = son_beam_enu(vel, varargin);
%           (vel, H, P, R, [name,value]...)
%      or   (vel, HPR,     [name,value]...)
%  
%   varargin name value pairs are:
%     name               default                     what
%   'orientation'        'down'      %'up' or 'down' (transducers are facing ??)
%     'angle'             30         % transducer angle from vertical
%     'error'             0          % include error velocity
%  'instrument'          'son'       % use generic transformation matrix)
%                                    % 'kmson' (only for Kilo Moana Sontek)
%   
%      key/value pairs are optional.

% 99/10/22 modified for orientation and angle inputs.
% 2002/09/11 JH added fillstruct


cfg.angle = 30;
cfg.orientation = 'down';
cfg.instrument = 'son';
cfg.error = 0;


% parse varargin
% First 1 or 3 arguments should be either the hpr array from
% adp.m, or separate arrays of heading, pitch, and roll.
startstrings = 0*[1:length(varargin)];
for argi = 1:length(startstrings)
   startstrings(argi) = isstr(varargin{argi});
end
startstrings = find(startstrings);  % varargin name,value pairs start here
% now split off name,value pairs from earlier arguments
if (isemptys(startstrings))
   firstpart = varargin;
   varargin = [];
else
   firstpart = varargin(1:startstrings(1) - 1);
   varargin(1:startstrings(1)-1) = [];
end
%now deal with first part
if length(firstpart) == 1,
   hpr = firstpart{1};
   Pitch = hpr(:,2);
   Roll = hpr(:,3);
   Heading = hpr(:,1);
elseif length(firstpart) == 3,
   Heading = firstpart{1};
   Pitch = firstpart{2};
   Roll = firstpart{3};
else
   help('beam_enu')
   error('Cannot parse input arguments.')
end


cfg = fillstruct(cfg, varargin, 0);

%now deal with inputs
beam_angle = cfg.angle*pi/180;
%start with defaults; change if necessary
orientation_up = 0;
if (strcmp(cfg.orientation, 'up'))
   orientation_up = 1;
elseif (~strcmp(cfg.orientation, 'down'))
   help(mfilename)
   error(['Invalid value of orientation: ', cfg.orientation]);   
end


%%%%%%%%%%%%%


if orientation_up,
   orient_string = 'up';
else
   orient_string = 'down';
end

xyze = son_beam_xyz(vel, 'angle', cfg.angle, 'orientation', orient_string, ...
                'instrument', cfg.instrument);


if (length(size(xyze)) == 2)
   enu = son_xyz_enu(xyze(1:3, :), Heading, Pitch, Roll);
   if cfg.error,
      enu = [enu;  xyze(4,:)];
   end
else % Nbins x Nbeams x Nprofs
   enu = son_xyz_enu(xyze(:, 1:3, :), Heading, Pitch, Roll);
   if cfg.error,
      enu = cat(2, enu, xyze(:,4,:));
   end
end



