% =============================================================================
%
%  How to get data from uhdas logging:
%  Make a  structure with appropriate values, use it to call
%
%     update_gbin :  (to update gbin files) (navigation and attitude) 
%     getsome_adcp:  (navigated adcp data, unedited, but with geographic 
%                            coordinates)
%     (plotting programs)
%%%
%
% Making the configuration structure: examples
% (1)  get the defaults, then override defaults and/or specify new values:
%
%      cfg=get_uhdascfg; 
%
% (2)  get path information from a files [cfgbase]_cfg.m and it's partner with
%      processing parameters [cfgbase]_proc.m, then either of these methods:
%
%    (2a)     
%
%    cfg = get_uhdascfg('uhdas_dir', '/home/data/km0401', 'cfgbase', 'os38b');
%                 % expects uhdas_dir/proc/config/[cfgbase]_cfg.m
%    (2b)
%
%    cfg = get_uhdascfg('cfgpath', '/home/adcp/config', 'cfgbase', 'cruise');
%
%  NOTE: "get_uhdascfg" starts with the defaults, 
%        then updates existing fields with new values from [cfgbase]_*.m
% 
%
%
%  cfg = get_uhdascfg; 
%  % must have the following fields for the following uses:
%
%
%  Updating "gbin" files: ("help update_gbin" or "help mk_gbin")
%
%    field                         value                  
%  -------                       ----------              
%
%  instclass                    :  os                           
%  instname                     : 'os38'                        
%  whichones                    : 'none'                         
%  redo                         : 1                             
%  gbindir                      : '../../gbin/os38'             
%  posinst                      : 'posmv'                       
%  path_ext                     : {'time','best';
%                                  'time','tim'};               
%  msg_info                     : {'posmv', 'gps',  'gps';        
%                                  'gyro',   'hdg',  'hdg';    
%                                  'adu2',   'adu',  'adu';    
%                                  'posmv',  'pmv',  'pmv'};   
%  best.rules                   : {'gps','heading','pitch','roll','heave'};
% best.gps_rule                 :{'posmv', 'gps'}; 
% best.heading_rule             :{'gyro',  'hdg'};
% best.pitch_rule               :{'posmv', 'pmv'};
% best.roll_rule                :{'posmv', 'pmv'};
% bbest.heave_rule              :{'posmv', 'pmv'};
% logfpath   : '../proclog'                   
%
%
% Reading navigated data ("help getsome_adcp") 
% Processing single-ping data ("help load_raw", "help loaduh")
% Plotting a chunk of navigated data ("help os_plotbbnb") % uhdas-specific
%            
%
%    field     value                        getsome_adcp           plotting
%                                                      (processing)
%  -------  ----------                      ----------  ----------
%                                                                  
%  uhdas_dir  : '/home/adcp/cruise'            *          *
%  instclass  :  os                            *          *           * 
%  pingtype   : 'first'                        *          *           *
%  instname   : 'os38'                         *          *           *
%  procdir    : [uhdas_dir]/proc/os38b'                   *
%  rawdir     : [uhdas_dir]/raw/os38'          *          *
%  gbindirbase: [uhdas_dir]/gbin/os38'         *          *
%  logfpath   : '../proclog'                   *          *
%  params.yearbase = 2004                      *          *
%  params.ducer_depth = 7                      *          *
%  params.h_align = 45                         *          *
%  
%  showfig    : 0                              *                        
%  interval   : 600                            *        
%  ddrange    : []                             *        
%
%  figdir     :'/home/adcp/www/figures'                               *
%
%
%===========================================================================
%
% example from km0401;
%
% UNDERWAY: (uses defaults of 'uhdas_dir' = '/home/adcp/cruise' and
%                             'cdfbase' = 'cruise'
% cfg =get_uhdascfg('cfgbase','cruise');
%
% nbcfg = cfg;
% nbcfg.params = get_params(nbcfg.allparams, nbcfg.instname, 'nb');
%
% bbcfg = cfg;
% bbcfg.params = get_params(bbcfg.allparams, bbcfg.instname, 'bb');
%
% % update gbin files (ping type irrelavaant at the moment)
% update_gbin(cfg,'whichones','last') 
%
% %% plot last ensemble
% [nbdd, nbconfig]=getsome_adcp(nbcfg,'ddrange',300);
% [bbdd, bbconfig]=getsome_adcp(bbcfg,'ddrange',300);
% 
%  os_plotbbnb(bbdd, bbconfig, nbdd, nbconfig, nbcfg, ...
%            'print2disk', 1, 'outdir', cfg.figdir);
%
%
% LATER:
% cfg =get_uhdascfg('uhdas_dir', '/home/data/km0401', 'cfgbase','os38b');
%
% %perhaps making gbin files from scratch:
% update_gbin(cfg,'whichones','last') 
%
% %%% click to choose time range
% [bbdd, bbconfig]=getsome_adcp(cfg); 
%
% %%% manually-specified dday range
% [bbdd, bbconfig]=getsome_adcp(cfg, 'ddrange', ddrange); 
%