function [ddrange, gpsinfo] = mk_rbinmap(cfg, varargin)
% [ddrange, gpsinfo] = mk_rbinmap(cfg, varargin)
% use gbin positions to create a topo map for clicking
%

cfg.gpsinfo = [];
cfg.max_xypts = 2000;

cfg = fillstruct(cfg, varargin);

if isempty(cfg.gpsinfo)
   filelist = dirs(fullfile(cfg.rbindir, 'gpsnav', '*.rbin'),'fullfile',1);
   gpsinfo = tr2ilist(filelist);
else
   gpsinfo = cfg.gpsinfo;
end

numpts = 0;
for ii=1:length(gpsinfo(:,1))
   numpts = numpts + gpsinfo{ii,4};
end


step = ceil(numpts/cfg.max_xypts);
now = to_day(2004,clock);
gdata = tr2ilist(gpsinfo, 'step', step);
lon = gdata(3,:);
lat = gdata(4,:);
clev = [ -3000 -2000 -1000 -500 0];  % topo contour levels
[lonrange, latrange]=map_range(lon, lat);
[longrid, latgrid, topog] = best_topo(lonrange, latrange);

clf
contour6(longrid, latgrid, topog, clev)
hold on
colormap(mk_cmap(length(clev), 'topo'));
axis('equal')
contourf(longrid, latgrid, topog, [0 0])

%%%
hold on
plot(lon, lat,'g.')

hh=gcf
if isemptys(cfg.interval)
   fprintf('click 2 points to delimit data extraction time range\n')
   [xx,yy]=ginput(2);
else
   fprintf('click 1 point to start data extraction time range\n')
   figure(hh)
   [xx,yy]=ginput(1);
end
   
   
[junk, ii]=min( (xx(1) - lon).^2 + (yy(1) - lat).^2);
startdd = gdata(1,ii);
if length(xx) == 2
   [junk, ii]=min( (xx(2) - lon).^2 + (yy(2) - lat).^2);
   enddd = gdata(1,ii);
else
   enddd = startdd+cfg.interval/86400;
end

ddrange = [startdd enddd];


