function  vels = nb_plotens(nbdd, nbconfig, plotcfg, varargin)
% makes a plot (nb150_lastens.png) from nb150 data
%   cfg = get_uhdascfg(varargin); 
%   update_gbin(cfg, 'last', 1)
%   [nbdd, nbconfig] = getsome_adcp(cfg,'pingtype','nb');
%   vels = nb_plotnb(nbdd, nbconfig, cfg, varargin)
%
% additional values that can be set by name,value pairs are:
%  name                      what                               default
% dubious                 put a 'dubious data' label              0
% otherheading           cellarray with instrument and extention  {}
%                                 eg. {'posmv','pmv'}


cfg = plotcfg;
if ~isfield(cfg, 'dubiousflag')
   cfg.dubiousflag = 0;
end
if ~isfield(cfg, 'otherheading')
   cfg.otherheading = '';
end

cfg = fillstruct(cfg, varargin);



if isemptys(nbdd)
   nbOK = logical(0);
else
   nbOK = logical(1);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nbplots_script;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nbOK
   ylims=[0,max(nbavg_edit.depth)];
elseif bbOK
   ylims=[0,max(bbavg_edit.depth)];
else
   clf
   return
end

if nbOK, nbdd.secs = 86400*(nbdd.dday - nbdd.dday(1)); end


%%%%%%% left side: amp  %%%%%%%%%
clf

%% make soemthing change with time if no data
mask = zeros(size(nbdd.amp2));
if max(nbdd.amp2(:)) < 20
   cc = clock;
   nowday = to_day(cfg.yearbase, cc);
   [nr, nc] = size(nbdd.amp2);
   hourstripes = 1:round(nr/25): nr;
   hour = cc(4) + 1;
   mask(hourstripes(hour),:) = 1;
   ensstripes =  1:round(nc/13): nc;
   ens = ceil(cc(5)/12);
   mask(:,ensstripes(ens)) = 1;
   
end

cc = aplotit(nbdd,  nbconfig, 'fn', 'amp2', 'abc_name', 'secs', ...
             'flagged', mask,...
             'ord_name', 'depth', 'index', [1:2: length(nbdd.dday)],...
             'var_range',[0 200], 'cur_ax', subplot(1,3,1),...
             'ylim', [0 max(nbdd.depth)],...
             'comment', 'NB150', 'astitle',1);
   

%%%%%%%%%%%% center: velocity  %%%%%%%%%%


subplot(1,3,2)

if nbOK
   hh=plot(nbavg_edit.uabs(nb_goodi), nbdd.depth(nb_goodi), 'r');lw(hh,2)
   hold on
   hh=plot(nbavg_noedit.uabs(nb_ngoodi), nbdd.depth(nb_ngoodi), 'r.');
   hh=plot(nbavg_edit.vabs(nb_goodi), nbdd.depth(nb_goodi), 'b');lw(hh,2)
   hold on
   hh=plot(nbavg_noedit.vabs(nb_ngoodi), nbdd.depth(nb_ngoodi), 'b.');
   
end   


ra(gca)
yl(ylims)
set(gca,'yticklabel', [])
%% do something reaosable with the xlimits
grid on
xlims = [-100 100];

uvvals = [nbavg_edit.uabs(nb_goodi)' nbavg_edit.vabs(nb_goodi)'];


if ( (min(uvvals) < xlims(1)) | (max(uvvals) > xlims(2)))
   xticks = best_ticks([min(uvvals) max(uvvals)]);
   dxticks = mean(diff(xticks));
   xlims  = [xticks(1)-dxticks  xticks(end)+dxticks];
   xl(xlims)
end
xl(xlims)
title('ocean velocity, cm/s')
if nbOK
   hh=text(xlims(1)+10,200,'NB U (east)');click_rgbk(hh,'c','r','s',12);
   hh=text(xlims(1)+10,250,'NB V (north)');click_rgbk(hh,'c','b','s',12);
   hh=text(mean(xlims),360, 'heading correction:');
   click_rgbk(hh,'c','k','s',12,'ha','c');
   hh=text(mean(xlims),390,sprintf(' %2.1f deg', mstdgap(dhdeg)));
   click_rgbk(hh,'c','k','s',12, 'ha','c');
end


%percent good
subplot(133)
nbpg_col = 'k';
bbpg_col = .5*[.5 1 0];

if nbOK
   %nb
   hh=plot(max(2,nbavg_edit.pg(:,4)),nbdd.depth);lw(hh,2)
   set(hh,'color',nbpg_col)
   ra(gca)
   hold on
   hh=plot(max(2,nbavg_noedit.pg(:,4)),nbdd.depth,'.');
   set(hh,'color',nbpg_col)
   yl(ylims)
end


set(gca,'yaxislocation','right', 'tickdir', 'out')
xl(0,100)
yl(ylims)
grid on
hold on
title('percent good')

if nbOK
   hh=text(10,1025,'NB PG');       click_rgbk(hh,'c',nbpg_col,'s',12);
   hh=text(10,1075,'- (edited)');click_rgbk(hh,'c',nbpg_col,'s',12);
   hh=text(10,1125,'. (unedited)');click_rgbk(hh,'c',nbpg_col,'s',12);
end

if cfg.dubiousflag
   aa = axes('position', [0 0 1 1]);
   axis('off')
   h=text(0.5, 0.6, 'DUBIOUS', 'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center','rotation',30);
   
   h=text(0.5, 0.4, 'DATA',    'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center', 'rotation',30);
end


%%%%%%%%%%%%%% label and print %%%%%%%%%%%%
minutes = (nbdd.corr_dday(end)-nbdd.corr_dday(1))*(24*60);
   
hh=subtitle(sprintf('(%s) %2.1f minutes of NB150 pings, ending at %s UTC\n',...
                 cfg.cruiseid, minutes, ...
                 to_date(nbdd.config.yearbase,nbdd.corr_dday(end),'s')));
pos = get(hh,'position');
set(hh,'position', pos - [0 .05 0]);
