function vels = os_plotamps(bbdd, bbconfig, nbdd, nbconfig, plotcfg, varargin)
% make a vector plot (to 100mn, and 100-200m) in one panel
%  and a magnitude + direction plot in the other panel
%
%   cfg = get_uhdascfg(varargin); 
%   update_gbin(cfg, 'last', 1)
%   [bbdd, bbconfig] = getsome_adcp(cfg,'pingtype','bb');
%   [nbdd, nbconfig] = getsome_adcp(cfg,'pingtype','nb');
%   vels = os_plotbbnb(bbdd, bbonfig, nbdd, nbconfig, cfg, varargin)
%
% additional values that can be set by name,value pairs are:
%  name                      what                               default
% dubious                 put a 'dubious data' label              0
% otherheading           cellarray with instrument and extention  {}
%                                 eg. {'posmv','pmv'}


cfg = plotcfg;
cfg.dubiousflag = 0;
cfg.otherheading = '';

cfg = fillstruct(cfg, varargin);


if isemptys(bbdd)
   bbOK = logical(0);
else
   bbOK = logical(1);
end

if isemptys(nbdd)
   nbOK = logical(0);
else
   nbOK = logical(1);
end

vels = [];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

osplots_script;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nbOK
   ylims=[0,max(nbdd.depth)];
elseif bbOK
   ylims=[0,max(bbdd.depth)];
else
   clf
   return
end


maxvel = 0;
maxamp = 0;
if bbOK
   cuv = bbmasks.uabs + i*bbmasks.vabs;
   cuvrot = uv2fp(cuv, repmat(bbmasks.heading, length(bbdd.depth), 1));
   bbfabs = 100*mstdgap(real(cuvrot)')';
   
   bbamp = 0;
   for ii=1:4
      bbamp = bbamp + squeeze(bbmasks.cleaned_amp(:,ii,:));
   end
   bbamp = mstdgap(bbamp'/4)';
   maxamp = max(maxamp, max(bbamp));
end
if nbOK
   cuv = nbmasks.uabs + i*nbmasks.vabs;
   cuvrot = uv2fp(cuv, repmat(nbmasks.heading, length(nbdd.depth), 1));
   nbfabs = 100*mstdgap(real(cuvrot)')';
   
   nbamp = 0;
   for ii=1:4
      nbamp = nbamp + squeeze(nbmasks.cleaned_amp(:,ii,:));
   end
   nbamp = mstdgap(nbamp'/4)';
   maxamp = max(maxamp, max(nbamp));
end


if isempty(maxamp)
   bbOK = [];
   nbOK=[];
end


if bbOK, bbdd.secs = 86400*(bbdd.dday - bbdd.dday(1)); end
if nbOK, nbdd.secs = 86400*(nbdd.dday - nbdd.dday(1)); end


%%%%%%% left side: amp %%%%%%%%%

subplot(121)
if bbOK ,  
   plot(bbamp, bbdd.depth,'g'); 
   hold on 
   hh=text(50, ylims(2) - 200 , 'bb amplitude');
   click_rgbk(hh,'c','g');
end
if nbOK ,  
   plot(nbamp, nbdd.depth,'k'); 
   hold on  
   hh=text(50, ylims(2) - 50, 'nb amplitude');
   click_rgbk(hh,'c','k');
end
grid on
xl(0,1.1*maxamp)
yl(ylims)
title('amplitude')
xlabel('RSSI counts')
ra(gca)


%%%%%%%%% right side forward %%%%%%%%%%

subplot(122)


allvels = [];
if bbOK ,  
   plot(bbfabs, bbdd.depth,'g'); 
   hold on 
   allvels = [allvels; [bbfabs, bbdd.depth]];
   hh=text(mstdgap(bbfabs), 1100, 'bb');
   click_rgbk(hh,'c','g','ha','center','s',18);
end
if nbOK ,  
   plot(nbfabs, nbdd.depth,'k'); 
   hold on  
   allvels = [allvels; [nbfabs, nbdd.depth]];
   hh=text(mstdgap(nbfabs), 1200, 'nb');
   click_rgbk(hh,'c','k','ha','center','s',18);
end
grid on

xlims = [min(allvels(:,1)), max(allvels(:,1))] + [-10 10];
xl(xlims)
yl(ylims)
title('ocean velocity (fwd direction)')
xlabel('cm/s')
ra(gca)



%%%%%%%%%%%% center: velocity  %%%%%%%%%%


if cfg.dubiousflag
   aa = axes('position', [0 0 1 1]);
   axis('off')
   h=text(0.5, 0.6, 'DUBIOUS', 'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center','rotation',30);
   
   h=text(0.5, 0.4, 'DATA',    'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center', 'rotation',30);
end




%%%%%%%%%%%%%% label and print %%%%%%%%%%%%

if bbOK
   minutes = (bbdd.corr_dday(end)-bbdd.corr_dday(1))*(24*60);
else
   minutes = (nbdd.corr_dday(end)-nbdd.corr_dday(1))*(24*60);
end   
   
subtitle(sprintf('%2.1f minutes of OS pings, ending at %s UTC\n',...
                 minutes, ...
                 to_date(bbdd.config.yearbase,bbdd.corr_dday(end),'s')))


