function  vels = os_plotens(bbdd, bbconfig, nbdd, nbconfig, plotcfg, varargin)
% makes a combined plot (like lastens) from osb and osn data
%   cfg = get_uhdascfg(varargin); 
%   update_gbin(cfg, 'last', 1)
%   [bbdd, bbconfig] = getsome_adcp(cfg,'pingtype','bb');
%   [nbdd, nbconfig] = getsome_adcp(cfg,'pingtype','nb');
%   vels = os_plotbbnb(bbdd, bbonfig, nbdd, nbconfig, cfg, varargin)
%
% additional values that can be set by name,value pairs are:
%  name                      what                               default
% dubious                 put a 'dubious data' label              0
% otherheading           cellarray with instrument and extention  {}
%                                 eg. {'posmv','pmv'}

cfg = plotcfg;
if ~isfield(cfg, 'dubiousflag')
   cfg.dubiousflag = 0;
end
if ~isfield(cfg, 'otherheading')
   cfg.otherheading = '';
end

cfg = fillstruct(cfg, varargin);

if isemptys(bbdd)
   bbOK = logical(0);
else
   bbOK = logical(1);
end

if isemptys(nbdd)
   nbOK = logical(0);
else
   nbOK = logical(1);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

osplots_script;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nbOK
   ylims=[0,max(nbavg_edit.depth)];
elseif bbOK
   ylims=[0,max(bbavg_edit.depth)];
else
   clf
   return
end

if bbOK, bbdd.secs = 86400*(bbdd.dday - bbdd.dday(1)); end
if nbOK, nbdd.secs = 86400*(nbdd.dday - nbdd.dday(1)); end



%%%%%%% left side: amp  %%%%%%%%%
clf

if bbOK & nbOK
   cc = aplotit(bbdd,  bbconfig, 'fn', 'amp2', 'abc_name', 'secs', ...
                'ord_name', 'depth',...
                'ylim', [0 max(bbdd.depth)],...                
                'var_range', [20 180],'cur_ax', subplot(2,3,1),...
                'comment', 'BB', 'astitle',1);
   
   cc = aplotit(nbdd,  nbconfig, 'fn', 'amp2', 'abc_name', 'secs', ...
                'ord_name', 'depth',...
                'ylim', [0 max(nbdd.depth)],...                
                'var_range',[0 200], 'cur_ax', subplot(2,3,4),...
                'comment', 'NB', 'astitle',1);
   
elseif  ( bbOK & (~nbOK))
   
   cc = aplotit(bbdd,  bbconfig, 'fn', 'amp2', 'abc_name', 'secs', ...
                'ord_name', 'depth',...
                'ylim', [0 max(bbdd.depth)],...                
                'var_range', [20 180],'cur_ax', subplot(1,3,1),...
                'comment', 'BB', 'astitle',1);
   
elseif ( (~bbOK) & nbOK)
   
   cc = aplotit(nbdd,  nbconfig, 'fn', 'amp2', 'abc_name', 'secs', ...
                'ord_name', 'depth',...
                'ylim', [0 max(nbdd.depth)],...                
                'var_range',[0 200], 'cur_ax', subplot(1,3,1),...
                'comment', 'NB', 'astitle',1);
else
   return
end


%%%%%%%%%%%% center: velocity  %%%%%%%%%%


subplot(1,3,2)

if nbOK
   hh=plot(nbavg_edit.uabs(nb_goodi), nbdd.depth(nb_goodi), 'r');lw(hh,2)
   hold on
   hh=plot(nbavg_noedit.uabs(nb_ngoodi), nbdd.depth(nb_ngoodi), 'r.');
   hh=plot(nbavg_edit.vabs(nb_goodi), nbdd.depth(nb_goodi), 'b');lw(hh,2)
   hold on
   hh=plot(nbavg_noedit.vabs(nb_ngoodi), nbdd.depth(nb_ngoodi), 'b.');
   
end   


ucol = [1 0 1] * .5; %dark magenta
vcol = [0 1 1] * .5; %dark cyan

if bbOK
   hh=plot(bbavg_edit.uabs(bb_goodi), bbdd.depth(bb_goodi), ...
           'color', ucol);lw(hh,2)
   hold on
   hh=plot(bbavg_noedit.uabs(bb_ngoodi), bbdd.depth(bb_ngoodi), 'o');ms(hh,4)
   set(hh,'color',ucol)
   hh=plot(bbavg_edit.vabs(bb_goodi), bbdd.depth(bb_goodi), ...
           'color', vcol);lw(hh,2)
   hh=plot(bbavg_noedit.vabs(bb_ngoodi), bbdd.depth(bb_ngoodi), 'o');ms(hh,4)
   set(hh,'color',vcol)
   
end

ra(gca)
yl(ylims)
set(gca,'yticklabel', [])
%% do something reaosable with the xlimits
grid on
xlims = [-100 100];

if nbOK
   uvvals = [nbavg_edit.uabs(nb_goodi)' nbavg_edit.vabs(nb_goodi)'];
elseif bbOK
   uvvals = [bbavg_edit.uabs(bb_goodi)' bbavg_edit.vabs(bb_goodi)'];   
end


if ( (min(uvvals) < xlims(1)) | (max(uvvals) > xlims(2)))
   xticks = best_ticks([min(uvvals) max(uvvals)]);
   dxticks = mean(diff(xticks));
   xlims  = [xticks(1)-dxticks  xticks(end)+dxticks];
   xl(xlims)
end
xl(xlims)
title('ocean velocity, cm/s')
if bbOK
   hh=text(xlims(1)+10,600,'BB U (east)');click_rgbk(hh,'c', ucol, 's',12);
   hh=text(xlims(1)+10,670,'BB V (north)');click_rgbk(hh,'c', vcol, 's',12);
   hh=text(mean(xlims),750, 'heading correction:');
   click_rgbk(hh,'c','k','s',12,'ha','c');
   hh=text(mean(xlims),820,sprintf(' %2.1f deg', mstdgap(dhdeg)));
   click_rgbk(hh,'c','k','s',12, 'ha','c');
   
end
if nbOK
   hh=text(xlims(1)+10,1100,'NB U (east)');click_rgbk(hh,'c','r','s',12);
   hh=text(xlims(1)+10,1200,'NB V (north)');click_rgbk(hh,'c','b','s',12);
   hh=text(mean(xlims),1350, 'heading correction:');
   click_rgbk(hh,'c','k','s',12,'ha','c');
   hh=text(mean(xlims),1420,sprintf(' %2.1f deg', mstdgap(dhdeg)));
   click_rgbk(hh,'c','k','s',12, 'ha','c');
end


%percent good
subplot(133)
nbpg_col = 'k';
bbpg_col = .5*[.5 1 0];

if nbOK
   %nb
   hh=plot(max(2,nbavg_edit.pg(:,4)),nbdd.depth);lw(hh,2)
   set(hh,'color',nbpg_col)
   ra(gca)
   hold on
   hh=plot(max(2,nbavg_noedit.pg(:,4)),nbdd.depth,'.');
   set(hh,'color',nbpg_col)
   yl(ylims)
end

if bbOK
   % bb
   hh=plot(max(2,bbavg_edit.pg(:,4)),bbdd.depth);lw(hh,2)
   set(hh,'color',bbpg_col)
   ra(gca)
   hold on
   hh=plot(max(2,bbavg_noedit.pg(:,4)),bbdd.depth,'o');ms(hh,4)
   set(hh,'color',bbpg_col)
end


set(gca,'yaxislocation','right', 'tickdir', 'out')
xl(0,100)
yl(ylims)
grid on
hold on
title('percent good')
if bbOK
   hh=text(10,525,'BB PG');        click_rgbk(hh,'c',bbpg_col,'s',12);
   hh=text(10,575,'- (edited)'); click_rgbk(hh,'c',bbpg_col,'s',12);
   hh=text(10,625,'o (unedited)'); click_rgbk(hh,'c',bbpg_col,'s',12);
end

if nbOK
   hh=text(10,1025,'NB PG');       click_rgbk(hh,'c',nbpg_col,'s',12);
   hh=text(10,1075,'- (edited)');click_rgbk(hh,'c',nbpg_col,'s',12);
   hh=text(10,1125,'. (unedited)');click_rgbk(hh,'c',nbpg_col,'s',12);
end

if cfg.dubiousflag
   aa = axes('position', [0 0 1 1]);
   axis('off')
   h=text(0.5, 0.6, 'DUBIOUS', 'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center','rotation',30);
   
   h=text(0.5, 0.4, 'DATA',    'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center', 'rotation',30);
end


%%%%%%%%%%%%%% label and print %%%%%%%%%%%%

if bbOK
   minutes = (bbdd.corr_dday(end)-bbdd.corr_dday(1))*(24*60);
else
   minutes = (nbdd.corr_dday(end)-nbdd.corr_dday(1))*(24*60);
end   

if bbOK
   hh=subtitle(sprintf('(%s) %2.1f minutes of OS38  BB pings, ending at %s UTC\n',...
                    cfg.cruiseid, minutes, ...
                    to_date(bbdd.config.yearbase,bbdd.corr_dday(end),'s')));
   pos = get(hh,'position');
   set(hh,'position', pos - [0 .05 0]);
else
   hh=subtitle(sprintf('(%s) %2.1f minutes of OS38 NB pings, ending at %s UTC\n',...
                    cfg.cruiseid, minutes, ...
                    to_date(nbdd.config.yearbase,nbdd.corr_dday(end),'s')));
   pos = get(hh,'position');
   set(hh,'position', pos - [0 .05 0]);
end   
