function vels = os_plotvec(bbdd, bbconfig, nbdd, nbconfig, plotcfg, varargin)
% make a vector plot (to 100mn, and 100-200m) in one panel
%  and a magnitude + direction plot in the other panel
%
%   cfg = get_uhdascfg(varargin); 
%   update_gbin(cfg, 'last', 1)
%   [bbdd, bbconfig] = getsome_adcp(cfg,'pingtype','bb');
%   [nbdd, nbconfig] = getsome_adcp(cfg,'pingtype','nb');
%   vels = os_plotbbnb(bbdd, bbonfig, nbdd, nbconfig, cfg, varargin)
%
% additional values that can be set by name,value pairs are:
%  name                      what                               default
% dubious                 put a 'dubious data' label              0
% otherheading           cellarray with instrument and extention  {}
%                                 eg. {'posmv','pmv'}
% nightview               background color is dark               0

cfg = plotcfg;
cfg.dubiousflag = 0;
cfg.otherheading = '';
cfg.nightview = 0;   %else white background 

cfg = fillstruct(cfg, varargin);

if cfg.nightview
   color = [.25 0 0];
else
   color = [1 1 1];
end


if isempty(bbdd)
   bbOK = logical(0);
else
   bbOK = logical(1);
end

if isempty(nbdd)
   nbOK = logical(0);
else
   nbOK = logical(1);
end

vels = [];
kts = 52;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

osplots_script;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nbOK
   ylims=[0,max(nbdd.depth)];
elseif bbOK
   ylims=[0,max(bbdd.depth)];
else
   clf
   return
end



if bbOK
   
   bbi100 = find(bbdd.depth <=130 & bbgoodmask);
   bbi200 = find(bbdd.depth >=100 & bbdd.depth<=200 & bbgoodmask);
   bbavg = bbavg_edit;
   bbmag = abs(bbavg.uabs + i*bbavg.vabs);
   bbkts = bbmag/kts;
   bbdir = rem(360 + 90 - 180*angle(bbavg.uabs + i*bbavg.vabs)/pi,360);

   uvvals = bbkts(bb_goodi);
   vels.bbkts = bbkts;
   vels.bbdir = bbdir;
   
end
if nbOK
   nbi100 = find(nbdd.depth<=130 & nbgoodmask);
   nbi200 = find(nbdd.depth>=100 & nbdd.depth<=200 & nbgoodmask);
   nbavg = nbavg_edit;
   nbmag = abs(nbavg.uabs + i*nbavg.vabs);
   nbkts = nbmag/kts;
   nbdir = rem(360 + 90 - 180*angle(nbavg.uabs + i*nbavg.vabs)/pi, 360);;

   uvvals = nbkts(nb_goodi);
   vels.nbkts = nbkts;
   vels.nbdir = nbdir;

end


if cfg.nightview
   hh = axes('position',[0 0 1 1]);
   set(gca,'xticklabel',[],'yticklabel',[])
   x = [0 1];
   y = [0 1];
   hh=fill([x fliplr(x) x(1)], [y(1) y fliplr(y)],'r');
   set(hh,'facecolor', [.2 0 0 ]) 
end



if bbOK, bbdd.secs = 86400*(bbdd.dday - bbdd.dday(1)); end
if nbOK, nbdd.secs = 86400*(nbdd.dday - nbdd.dday(1)); end

if isempty(nb_goodi),   nbOK = 0;  end
if isempty(bb_goodi),   bbOK = 0;  end

if (~bbOK) & (~nbOK)
   return
end



%%%%%%% left side: vector %%%%%%%%%
if nbOK
   utop = nbavg.uabs(nb_goodi(1));
   vtop = nbavg.vabs(nb_goodi(1));
   ztop = nbdd.depth(nb_goodi(1));
   ztopstr = num2str(round(ztop));
   
   nbuabs100 = mstdgap(nbavg.uabs(nbi100));
   nbvabs100 = mstdgap(nbavg.vabs(nbi100));
   
   nbuabs200 = mstdgap(nbavg.uabs(nbi200));
   nbvabs200 = mstdgap(nbavg.vabs(nbi200));
   nb_goodi = find(nbavg.pg(:,4) > 20);
end


if bbOK
   utop = bbavg.uabs(bb_goodi(1));
   vtop = bbavg.vabs(bb_goodi(1));
   ztop = bbdd.depth(bb_goodi(1));
   ztopstr = num2str(round(ztop));
   
   bbuabs100 = mstdgap(bbavg.uabs(bbi100));
   bbvabs100 = mstdgap(bbavg.vabs(bbi100));

   bbuabs200 = mstdgap(bbavg.uabs(bbi200));
   bbvabs200 = mstdgap(bbavg.vabs(bbi200));
   bb_goodi = find(bbavg.pg(:,4) > 20);
end


if bbOK & ~nbOK 
   uabs100 = bbuabs100;
   vabs100 = bbvabs100;
   uabs200 = bbuabs200;
   vabs200 = bbvabs200;
   lon = bbavg.lastlon;
   lat = bbavg.lastlat;
   maxmag = max(abs(uvvals));
elseif nbOK & ~bbOK
   uabs100 = nbuabs100;
   vabs100 = nbvabs100;
   uabs200 = nbuabs200;
   vabs200 = nbvabs200;
   lon = nbavg.lastlon;
   lat = nbavg.lastlat;
   maxmag = max(abs(uvvals));
else
   uabs100 = (bbuabs100 + nbuabs100)/2;
   vabs100 = (bbvabs100 + nbvabs100)/2;
   uabs200 = (bbuabs200 + nbuabs200)/2;
   vabs200 = (bbvabs200 + nbvabs200)/2;
   lon = nbavg.lastlon;
   lat = nbavg.lastlat; 
   bbmaxmag = max(abs(uvvals));
   nbmaxmag = max(abs(uvvals));
   maxmag = max(abs(bbmaxmag(1)), abs(nbmaxmag(1)));
end

maxmag = max(maxmag, .6)

axes('position', [ 0.1300    0.300    0.3270    .3270])
y = (lat + maxmag*[-1 1]);
x = (lon + maxmag*[-1 1]);
if cfg.nightview
   hh=fill([x fliplr(x) x(1)], [y(1) y fliplr(y)],'r');
   set(hh,'facecolor', color) ;
   set(gca,'xcolor',[.8 0  0])
   set(gca,'ycolor',[.8 0  0])
   hold on
end

   
squiver(lon, lat, uabs200, vabs200, 'vcolor', 'b', ...
        'msize', 1, 'mcolor', color,...
        'lthick',2,'scale',.01, 'head', .2);
hold on
squiver(lon,  lat, uabs100, vabs100, 'vcolor', 'g', ...
        'msize', 1, 'mcolor', color,...
        'lthick',2,'scale',.01, 'head', .2);
squiver(lon,  lat, utop, vtop, 'vcolor', 'r', ...
        'msize', 1, 'mcolor', color,...
        'scale',.01, 'head', .2);
axis('equal')
xl(x)
yl(y)
lltick(gca,'x','lon','y','lat')
squiver(lon-.90*maxmag,  lat-.9*maxmag, kts, 0, 'vcolor', 'k', ...
        'msize', 1, 'mcolor', color,...
        'lthick',2,'scale',.01, 'head', .15);
hh=text(lon-.9*maxmag,  lat-.8*maxmag, 'key: 1kt');


hh=text(lon-.9*maxmag,  lat+.9*maxmag, ...
        sprintf('shallowest (%sm)', ztopstr));
click_rgbk(hh,'c','r');
hh=text(lon-.9*maxmag,  lat+.75*maxmag ,...
        '50-100m average');
click_rgbk(hh,'c','g');
hh=text(lon-.9*maxmag,  lat+.6*maxmag, ...
        '100m-200m average');
click_rgbk(hh,'c','b');



%%%%%%%%%%%% center: velocity  %%%%%%%%%%

axes('position', [ 0.5426    0.2100    0.1562    0.6150])

xlims = [0 1.1*maxmag];
if ( (min(uvvals) < xlims(1)) | (max(uvvals) > xlims(2)))
   xticks = best_ticks([0 max(uvvals)]);
   dxticks = mean(diff(xticks));
   xlims  = [0  xticks(end)+dxticks];
   xl(xlims)
end


if cfg.nightview
   y = ylims;
   x = xlims
   hh=fill([x fliplr(x) x(1)], [y(1) y fliplr(y)],'r');
   set(hh,'facecolor', [.25 0 0 ]) ;
   set(gca,'xcolor',[.8 0  0])
   set(gca,'ycolor',[.8 0  0])
   hold on
end


if nbOK
   hh=plot(nbmag(nb_goodi)/kts, nbdd.depth(nb_goodi), 'color', .3*[1 1 1]);
   lw(hh,2)
   hold on
   hh=plot(nbmag(nbi100)/kts, nbdd.depth(nbi100),'g');lw(hh,2)
   hh=plot(nbmag(nbi200)/kts, nbdd.depth(nbi200),'b');lw(hh,2)
end   


if bbOK
   hh=plot(bbmag(bb_goodi)/kts, bbdd.depth(bb_goodi), 'k');
   hold on
   hh=plot(bbmag(bbi100)/kts, bbdd.depth(bbi100),'g');lw(hh,2)
   hh=plot(bbmag(bbi200)/kts, bbdd.depth(bbi200),'b');lw(hh,2)
end


hh=plot(abs(utop + i*vtop)/kts, ztop,'r.');ms(hh,20)   



ra(gca)
set(gca,'yticklabel', [])
%% do something reaosable with the xlimits
grid on
yl(ylims)
xl(xlims)
title('ocean speed ')
xlabel('speed, kts')




%%%%%%%%%%%%%%%%% direction %%%%%%%%%%%
axes('position', [ 0.7488    0.2100    0.1562    0.6150]);


if cfg.nightview
   y = ylims;
   x = [0 360];
   hh=fill([x fliplr(x) x(1)], [y(1) y fliplr(y)],'r');
   set(hh,'facecolor', [.25 0 0 ]) ;
   set(gca,'xcolor',[.8 0  0])
   set(gca,'ycolor',[.8 0  0])
   hold on
end


plot([90 90]', [0 1500]', 'color', .5*[1 1 1])
hold on
plot([180 180]', [0 1500]', 'color', .5*[1 1 1])
plot([270 270]', [0 1500]', 'color', .5*[1 1 1])


if nbOK
   hh=plot(nbdir(nb_goodi) ,  nbdd.depth(nb_goodi), 'o')
   set(hh, 'color', .3*[1 1 1]);
   ms(hh,4)
   lw(hh,2)
   hold on
   hh=plot(nbdir(nbi100) ,  nbdd.depth(nbi100), 'go'); ms(hh,4)
   hh=plot(nbdir(nbi200) ,  nbdd.depth(nbi200), 'bo'); ms(hh,4)
   hh=plot(nbdir(1) , nbdd.depth(1) , 'ro'); ms(hh,4)
end   


if bbOK
   hh=plot(bbdir(bb_goodi),  bbdd.depth(bb_goodi), 'k.');
   hold on
   hh=plot(bbdir(bbi100),  bbdd.depth(bbi100), 'g.');
   hh=plot(bbdir(bbi200),  bbdd.depth(bbi200), 'b.');
   hh=plot(bbdir(1) , bbdd.depth(1) , 'r.'); ms(hh,10)

end

ra(gca)
yl(ylims)
set(gca, 'xtick', [0 90 180 270])
xl(0,360)

set(gca,'yaxislocation','right')
lltick(gca,'x','none')
xlabel('direction')
title('direction')
ylabel('meters')


if cfg.dubiousflag
   aa = axes('position', [0 0 1 1]);
   axis('off')
   h=text(0.5, 0.6, 'DUBIOUS', 'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center','rotation',30);
   
   h=text(0.5, 0.4, 'DATA',    'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center', 'rotation',30);
end






%%%%%%%%%%%%%% label and print %%%%%%%%%%%%

if bbOK
   minutes = (bbdd.corr_dday(end)-bbdd.corr_dday(1))*(24*60);
else
   minutes = (nbdd.corr_dday(end)-nbdd.corr_dday(1))*(24*60);
end   
   
subtitle(sprintf('%2.1f minutes of OS pings, ending at %s UTC\n',...
                 minutes, ...
                 to_date(bbdd.config.yearbase,bbdd.corr_dday(end),'s')))


