function vels = os_plotvec(dd, config, plotcfg, varargin)
% make a vector plot in specified ranges
%  and a magnitude + direction plot in the other panel
%
%   plotcfg = get_uhdascfg(varargin); 
%   update_gbin(cfg, 'last', 1)
%   [dd, config] = getsome_adcp(cfg,'pingtype','nb');
%   vels = os_plotvec(dd, config, plotcfg, varargin)
%
% additional values that can be set by name,value pairs are:
%  name                      what                               default
% dubious                 put a 'dubious data' label              0
% otherheading           cellarray with instrument and extention  {}
%                                 eg. {'posmv','pmv'}
% nightview               background color is dark               0

cfg = plotcfg;
cfg.dubiousflag = 0;
cfg.otherheading = '';
cfg.nightview = 0;   %else white background 
cfg.depthranges = [0 100
                   100 200
                   200 300];  %% up to three

cfg = fillstruct(cfg, varargin);

if cfg.nightview
   color = [.25 0 0];
else
   color = [1 1 1];
end

vels = [];
kts = 52;
depthranges = cfg.depthranges;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isemptys(cfg.otherheading)
   if ~iscell(cfg.otherheading)
      fprintf('otherheading must be a cellarray \')
      fprintf('  with instrument and extension, such as:\n')
      fprintf(' {''posmv'',''pmv''} \n')
      return
   elseif length(cfg.otherheading) ~= 2
      fprintf('otherheading must be a cellarray \')
      fprintf('  with instrument and extension, such as:\n')
      fprintf(' {''posmv'',''pmv''} \n')
      return
   end      
   otheri = find(ismember(cfg.path_ext(:,1),  cfg.otherheading{1}) & ...
                 ismember(cfg.path_ext(:,2),  cfg.otherheading{2}));
   h_path = cfg.path_ext{otheri,1};
   h_ext  = cfg.path_ext{otheri,2};
   
   %% bbdd has a struct like bbdd.posmv with:
   %%    bbdd.posmv.pmv.gdata (data from read_bin)
   %%    bbdd.posmv.pmv.gstruct (structure from read_bin)
   inststruct = getfield(dd, h_path);

   try
      extstruct  = getfield(inststruct, h_ext);
      hdata = getfield(extstruct, 'gdata');
      hstruct = getfield(extstruct, 'gstruct');
      %% get quality info
      qcmask = qc_rbin(hdata, hstruct);
      hdata = hdata(r_row(hstruct, 'heading'),:);
      hmask = qcmask(r_row(hstruct, 'heading'),:);
      
      dhdeg = mstdgap(hdata(~hmask) - dd.heading(~hmask));
      mean_headcor_rad = pi*dhdeg/180;
   catch
      mean_headcor_rad = 0;
      dhdeg = 0;
   end
else
   mean_headcor_rad = 0;
   dhdeg = 0;
end



%%%%%%%%%%5%% edit bb pings and average  %%%%%%%%%%%%%%%
   
params = cfg.params;
params.editcfg = get_pedefaults(config.instclass, config.pingtype);
params.use_refavg = 1;
params.pingedit = 'all';

fprintf('rotating measured velocities by exp(-i*%f) deg\n', mstdgap(dhdeg))
cuv = exp(-i*mean_headcor_rad)*(dd.umeas + i* dd.vmeas);
dd.umeas = real(cuv);
dd.vmeas = imag(cuv);
ushipM = repmat(dd.uship, length(dd.depth),1);
vshipM = repmat(dd.vship, length(dd.depth),1);

a = cutadcp(dd,1);
[avg_edit, masks] = avg(os, dd, a, params);
uabs = 100*(ushipM + real(dd.umeas.*dotmmask(masks.instpflag)));
vabs = 100*(vshipM + real(dd.vmeas.*dotmmask(masks.instpflag)));
avg_edit.uabs = mstdgap(uabs')';
avg_edit.vabs = mstdgap(vabs')';

% not pingedited
params.pingedit = 'none';
[avg_noedit] = avg(os, dd, a, params);
uabs = 100*(ushipM + real(dd.umeas));
vabs = 100*(vshipM + real(dd.vmeas));
avg_noedit.uabs = mstdgap(uabs')';
avg_noedit.vabs = mstdgap(vabs')';


vels.uabs = avg_noedit.uabs;
vels.vabs = avg_noedit.vabs;

goodi  = find(avg_edit.pg(:,4) > 20);
ngoodi = find(avg_noedit.pg(:,4) > 20);

goodmask = avg_edit.pg(:,4) > 20;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ylims=[0,max(dd.depth)];

cuv   = NaN*ones(size(depthranges(:,1)));

for depthi = 1:length(depthranges(:,1))
   ii = find(dd.depth >= depthranges(depthi,1) & ...
             dd.depth <= depthranges(depthi,2) & ...
             goodmask );
   if isempty(ii)
      cuv(depthi) = NaN+i*NaN;
   else
      cuv(depthi) = mstdgap(avg_edit.uabs(ii) + i*avg_edit.vabs(ii));
      % reset to show what was used
      depthranges(depthi,1) = dd.depth(ii(1));
      depthranges(depthi,2) = dd.depth(ii(end));
   end
end
uvmag = abs(cuv);
uvdir = rem(360+90-180*angle(cuv)/pi, 360);
uvkts = uvmag/kts;


if cfg.nightview
   hh = axes('position',[0 0 1 1]);
   set(gca,'xticklabel',[],'yticklabel',[])
   x = [0 1];
   y = [0 1];
   hh=fill([x fliplr(x) x(1)], [y(1) y fliplr(y)],'r');
   set(hh,'facecolor', [.2 0 0 ]) 
end


dd.secs = 86400*(dd.dday - dd.dday(1));


%%%%%%% left side: vector %%%%%%%%%
for depthi = 1:length(depthranges(:,1))
   if isnan(cuv(depthi))
      labelstr{depthi} = 'no good data';
   else
      labelstr{depthi} = sprintf('%dm to %dm average', ...
                                 floor(depthranges(depthi,1)),...
                                 ceil(depthranges(depthi,2)));
   end
end   

lon = avg_edit.lastlon;
lat = avg_edit.lastlat;
maxmag = max(abs(cuv));

maxmag = max(maxmag, .6)

if isempty(maxmag)
   return
end

axes('position', [ 0.1300    0.300    0.3270    .3270])
y = (lat + maxmag*[-1 1]);
x = (lon + maxmag*[-1 1]);
if cfg.nightview
   hh=fill([x fliplr(x) x(1)], [y(1) y fliplr(y)],'r');
   set(hh,'facecolor', color) ;
   set(gca,'xcolor',[.8 0  0])
   set(gca,'ycolor',[.8 0  0])
   hold on
end

   

colors = ['rgb'];
for depthi = 1:length(depthranges)
   squiver(lon, lat, real(cuv(depthi)), imag(cuv(depthi)), ...
   'vcolor', colors(depthi), ...
      'msize', 1, 'mcolor', color,...
      'lthick',2,'scale',.01, 'head', .2);
   hold on
end

axis('equal')
xl(x)
yl(y)
lltick(gca,'x','lon','y','lat')
squiver(lon-.90*maxmag,  lat-.9*maxmag, kts, 0, 'vcolor', 'k', ...
        'msize', 1, 'mcolor', color,...
        'lthick',2,'scale',.01, 'head', .15);
hh=text(lon-.9*maxmag,  lat-.8*maxmag, 'key: 1kt');


for depthi = 1:length(depthranges(:,1))
   dzfrac = .15
   hh=text(lon-.9*maxmag,  lat+(.4+ depthi*dzfrac)*maxmag, labelstr{depthi})
   click_rgbk(hh,'c',colors(depthi));
end


%%%%%%%%%%%% center: velocity  %%%%%%%%%%

%%%%%%%%%%%%%
return
%%%%%%%%%%%%%

stuck here



axes('position', [ 0.5426    0.2100    0.1562    0.6150])

xlims = [0 1.1*maxmag];
if ( (min(uvvals) < xlims(1)) | (max(uvvals) > xlims(2)))
   xticks = best_ticks([0 max(uvvals)]);
   dxticks = mean(diff(xticks));
   xlims  = [0  xticks(end)+dxticks];
   xl(xlims)
end


if cfg.nightview
   y = ylims;
   x = xlims
   hh=fill([x fliplr(x) x(1)], [y(1) y fliplr(y)],'r');
   set(hh,'facecolor', [.25 0 0 ]) ;
   set(gca,'xcolor',[.8 0  0])
   set(gca,'ycolor',[.8 0  0])
   hold on
end


if nbOK
   hh=plot(nbmag(nb_goodi)/kts, nbdd.depth(nb_goodi), 'color', .3*[1 1 1]);
   lw(hh,2)
   hold on
   hh=plot(nbmag(nbi100)/kts, nbdd.depth(nbi100),'g');lw(hh,2)
   hh=plot(nbmag(nbi200)/kts, nbdd.depth(nbi200),'b');lw(hh,2)
end   


if bbOK
   hh=plot(bbmag(bb_goodi)/kts, bbdd.depth(bb_goodi), 'k');
   hold on
   hh=plot(bbmag(bbi100)/kts, bbdd.depth(bbi100),'g');lw(hh,2)
   hh=plot(bbmag(bbi200)/kts, bbdd.depth(bbi200),'b');lw(hh,2)
end


hh=plot(abs(utop + i*vtop)/kts, ztop,'r.');ms(hh,20)   



ra(gca)
set(gca,'yticklabel', [])
%% do something reaosable with the xlimits
grid on
yl(ylims)
xl(xlims)
title('ocean speed ')
xlabel('speed, kts')



%%%%%%%%%%%%%%%%% direction %%%%%%%%%%%
axes('position', [ 0.7488    0.2100    0.1562    0.6150]);


if cfg.nightview
   y = ylims;
   x = [0 360];
   hh=fill([x fliplr(x) x(1)], [y(1) y fliplr(y)],'r');
   set(hh,'facecolor', [.25 0 0 ]) ;
   set(gca,'xcolor',[.8 0  0])
   set(gca,'ycolor',[.8 0  0])
   hold on
end


plot([90 90]', [0 1500]', 'color', .5*[1 1 1])
hold on
plot([180 180]', [0 1500]', 'color', .5*[1 1 1])
plot([270 270]', [0 1500]', 'color', .5*[1 1 1])


if nbOK
   hh=plot(nbdir(nb_goodi) ,  nbdd.depth(nb_goodi), 'o')
   set(hh, 'color', .3*[1 1 1]);
   ms(hh,4)
   lw(hh,2)
   hold on
   hh=plot(nbdir(nbi100) ,  nbdd.depth(nbi100), 'go'); ms(hh,4)
   hh=plot(nbdir(nbi200) ,  nbdd.depth(nbi200), 'bo'); ms(hh,4)
   hh=plot(nbdir(1) , nbdd.depth(1) , 'ro'); ms(hh,4)
end   


if bbOK
   hh=plot(bbdir(bb_goodi),  bbdd.depth(bb_goodi), 'k.');
   hold on
   hh=plot(bbdir(bbi100),  bbdd.depth(bbi100), 'g.');
   hh=plot(bbdir(bbi200),  bbdd.depth(bbi200), 'b.');
   hh=plot(bbdir(1) , bbdd.depth(1) , 'r.'); ms(hh,10)

end

ra(gca)
yl(ylims)
set(gca, 'xtick', [0 90 180 270])
xl(0,360)

set(gca,'yaxislocation','right')
lltick(gca,'x','none')
xlabel('direction')
title('direction')
ylabel('meters')


if cfg.dubiousflag
   aa = axes('position', [0 0 1 1]);
   axis('off')
   h=text(0.5, 0.6, 'DUBIOUS', 'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center','rotation',30);
   
   h=text(0.5, 0.4, 'DATA',    'units','normalized', 'color',.5*[1 1 1],...
          'fontsize',74,'horizontalalignment','center', 'rotation',30);
end






%%%%%%%%%%%%%% label and print %%%%%%%%%%%%

if bbOK
   minutes = (bbdd.corr_dday(end)-bbdd.corr_dday(1))*(24*60);
else
   minutes = (nbdd.corr_dday(end)-nbdd.corr_dday(1))*(24*60);
end   
   
subtitle(sprintf('%2.1f minutes of OS pings, ending at %s UTC\n',...
                 minutes, ...
                 to_date(bbdd.config.yearbase,bbdd.corr_dday(end),'s')))


