function params = get_params(allparams, instname, pingtype)
% params = get_params(allparams, instname, pingtype)
% eg. cruise_proc; params = get_params(allparams, 'os38', 'nb')
% for use with uhdas processing; in config directory after 
%   adcptree.py cruiseid /home/noio/programs uhdas
% allparams is like
%      allparams.son125.son.h_align = -1.3; 
%      allparams.son125.son.ducer_depth  = 7;  
%      [etc]
%      allparams.os38.bb.h_align = -45; 
%      allparams.os38.bb.ducer_depth  = 7;  
%      [etc] 


if isemptys(allparams) | isemptys(instname) | isemptys(pingtype)
   params.h_align = [];
   params.velscale = 1;
   params.interval = 300;
   params.ilayer = 1:20;
   params.beamangle = NaN;   %unknown -- must override
   params.instrument = '';
   params.ducer_depth = 0 ;
   cc=clock;
   params.yearbase = cc(1);
   return
end


% extract correct params from allparams.instname.inststr
if ~isfield(allparams, instname)
   fprintf('%s is not a field of allparams\n', instname)
   allparams
   return
end
instparams = getfield(allparams, instname);

if ~isfield(instparams, pingtype)
   fprintf('%s is not a subfield of allparams\n', pingtype)
   allparams
   return
end
params = getfield(instparams, pingtype);
 