function [iM, iMc, rawbaselist] = get_iM(rawdir, rawbaselist, ifilestart)
% [iM, iMc] = get_iM(rawdir, rawbaselist, ifilestart)
% 
%% iM, index matrix, contains columns (which are in a cellarray iMc)
%  ilsit (index into rawbaselist for files to use)
%   
%% ifile         (index into rawbaselist)
%% inst_dday    (instrument dday from log file)
%% ensnum        (ensemble number, from log file)
%% ifilej        (index into file(ifile))
%%
%% 
%
% might get big.... one entry for each ping for all files in the list


iMc{1} = 'ifile';
iMc{2} = 'inst_dday';
iMc{3} = 'ensnum';
iMc{4} = 'ifilej';


if isempty(rawbaselist)
   iM=[];
   return
end

fprintf('getting total ensemble counter list: please wait...\n')
fprintf('starting at %d of %d files\n', ifilestart, length(rawbaselist))

skip_rawfile_mask = zeros(size(rawbaselist));
numpts = 0;
for ifile=1:length(rawbaselist);
   if ifile >= ifilestart
      rawlogfile = fullfile(rawdir, [rawbaselist{ifile}, '.raw.log']);
      if ~exist(rawlogfile) %bugs create (empty) data files with no log file
         skip_rawfile_mask(ifile) = 1;
      else
         % get raw logfile info
         logdata = load(rawlogfile);
         if (~isempty(logdata))
            numpts = numpts + length(logdata(:,1));
         end
      end
   end
end

%% eliminate those which have no log file
rawbaselist(find(skip_rawfile_mask)) = [];


numpts = numpts + 1000;  % for good measure

iM = NaN*zeros(numpts, 4);

lastcount = 0;
for ifile=1:length(rawbaselist)
   if ifile >= ifilestart
      rawlogfile = fullfile(rawdir, [rawbaselist{ifile}, '.raw.log']);
      % get raw logfile info
      [rlog_udday, rlog_offsets, rlog_bytes, rlog_ensnums, rlog_instdday] = ...
          textread( rawlogfile, '%f%d%d%d%f');
      if ~isempty(rlog_udday)
         index = [1:length(rlog_udday)] + lastcount;
         iM(index,1) = ifile;
         iM(index,2) = rlog_instdday;
         iM(index,3) = rlog_ensnums;
         iM(index,4) = [1:length(rlog_udday)]';
         lastcount = index(end);
      end
   end
end

badi = find(isnan(iM(:,1)));
iM(badi,:)=[];




