function [all_ensinfo, last_ensinfo, newblock, cursor] = ...
    get_next_ensinfo(rawdir, rawbaselist, iM, iMc, cursor, params)
% [all_ensinfo, last_ensinfo, newblock] = ...
%                get_next_ensinfo(iM, rawdir, rawbaselist, last_ensinfo, params)
%
% ensinfo is a structure array with one entry per call to read, fieldnames:
% rawfilebase, index, pingtype, ensnums, 


if isemptys(rawbaselist)
   ensinfo = struct;
   return
end


%%%  set some variables
% use this as a criterion for being "close enough" to accept an ensemble
have_an_ensemble    = 0;
all_ensinfo         = [];
ifile_col      = find(ismember(iMc, 'ifile'));
inst_dday_col  = find(ismember(iMc, 'inst_dday'));
ensnum_col     = find(ismember(iMc, 'ensnum'));
ifilej_col     = find(ismember(iMc, 'ifilej'));
second_set_col = find(ismember(iMc, 'second_set'));

 
% initialize cursor
cursor = cursor + 1;

while ~have_an_ensemble

   % quit if this is the end
   if cursor >= length(iM(:,1))
      all_ensinfo = [];
      last_ensinfo = [];
      newblock = [];
      return
   end
   
   % try again
   startdd = iM(cursor, inst_dday_col);
   enddd   = startdd + params.interval/86400;
   inst_dday = iM(:, inst_dday_col);
   ii = cursor : max(find(inst_dday <= enddd));
   
   % test: if isempty, return
   if isempty(ii)
      cursor = ii(end)+1;
      continue
   end
   
   % test: if ensemble numbers jumped, reset cursor and return
   ensjumpi = find(diff(iM(ii, ensnum_col))<=0);
   if length(ensjumpi) > 0
      cursor = ii(ensjumpi(end))+1;
      continue
   end
   
   
   % else this is supposed to be good:
   filenums = unique(iM(ii, ifile_col));
   all_ensinfo = [];

   % set up all_ensinfo
   for filenumi = 1:length(filenums)
      ifile = filenums(filenumi);
      jj=find(iM(ii, ifile_col) == ifile);
      iijj = ii(jj);
      
      ifilej = iM(iijj, ifilej_col);
      ensinfo.firstens    = iM(iijj(1),  ensnum_col);
      ensinfo.lastens     = iM(iijj(end),ensnum_col);
      ensinfo.ilist       = ifilej;
      ensinfo.ifile       = ifile;
      ensinfo.rawfilebase = rawbaselist{ifile};
      ensinfo.secs_read   = round(86400*...
          (iM(ii(jj(end)), inst_dday_col) - iM(iijj(1), inst_dday_col)));
      ensinfo.second_set  = iM(iijj, second_set_col);
      
      all_ensinfo = [all_ensinfo; ensinfo];
   end      
   
   
   if iM(ii(1), ensnum_col) <  iM(ii(1)-1, ensnum_col)
      newblock = 1;
   else
      newblock = 0;
   end
   
   cursor = ii(end);

   last_ensinfo.rawfilebase = ensinfo.rawfilebase;
   last_ensinfo.lastens     = ensinfo.lastens;
   last_ensinfo.ifile       = ensinfo.ifile;
   
   have_an_ensemble = 1;
   
end

