function pria = get_pria(rawdir, rawbaselist, iM, iMc, params);
% pria = get_pria(rawdir, rawbaselist, iM, iMc, params);
%
% "pria"  - Ping Requested Is Available
% pria is the same length as iM(:,1), containing a mask with 
%    0 = not available
%    1 = first set 
%    2 = second set (os only)
%
% automatically returns ones for nb or bb; must test for os


ll = length(iM(:,1));


if isa(params.instclass, 'bb')
   if strcmp(params.pingtype, 'bb')
      pria = ones(ll,1);
   else
      pria = zeros(ll,1);
   end
elseif isa(params.instclass, 'nb')
   if strcmp(params.pingtype, 'nb')
      pria = ones(ll,1);
   else
      pria = zeros(ll,1);
   end
elseif isa(params.instclass, 'os')
   
   if isempty(iM)
      pria = [];
      return
   end
   
   pria = NaN*zeros(size(iM(:,1)));
   
   ens_col = find(ismember(iMc, 'ensnum'));
   ensnums = iM(:,ens_col);
   ensjumpi = find(diff(ensnums) <= 0);
   seg_starti = [1; ensjumpi+1];
   seg_endi   = [ensjumpi ; length(ensnums)];
   

   ifile_col = find(ismember(iMc, 'ifile'));

   fprintf('getting pingtype compatibility list: please wait...')
   
   lastcount = 0;
   for segi = 1:length(seg_starti)
      rawfilebase = rawbaselist{iM(seg_starti(segi), ifile_col)}
      pingtypes = get_pingtypes(params.instclass, rawdir, rawfilebase, params);

      index = (seg_starti(segi) : seg_endi(segi));
      
      if iscell(pingtypes.available)
         if ismember(params.pingtype, pingtypes.available)
            pingmatches = pingtypes.second_set+1;
         else
            pingmatches = 0;
         end
      else
         if strcmp(params.pingtype, pingtypes.available)
            pingmatches = pingtypes.second_set+1;
         else
            pingmatches = 0;
         end
      end
      
      pria(index) = pingmatches;
      
      lastcount = index(end);
         
   end
   
else
   fprintf('instrument class not supported\n')
   params.instclass
   pria = [];
   return
end
   
      





