function [a, rr] = mk_rawens(rawdir, ensinfo, params)
%%%


rr = [];

for iseg = 1:length(ensinfo)
   rawfilebase = ensinfo(iseg).rawfilebase ;
   rawfile = fullfile(rawdir, [rawfilebase, '.raw']); 
   ilist = ensinfo(iseg).ilist;

   if (isa(params.instclass, 'os'))
      %% testing fastread:
      if isfield(params, 'fastread') & params.fastread
         r = fastread(params.instclass, rawfile,  ilist([1 end]),...
                  ensinfo(iseg).second_set);
         %% adding more stuff for compatibility...
         r.nav.txy1 = NaN*ones(3,length(r.dday));
         r.nav.txy2 = NaN*ones(3,length(r.dday));
         r.nav.seconds_pc_minus_utc = NaN*ones(1,length(r.dday));
         [pp, r.info.rawfilebase] = fileparts(rawfile);
         r.info.ilist = ilist;
         r.pg = NaN * r.vel;
         r.num_pings = ones(size(r.dday));;
         r.tr_pressure = zeros(size(r.dday));;
         r.config.yearbase = params.yearbase;
         
         %%%%% end compatibility klooge 
      else        
         r = read(params.instclass, rawfile,  ...
                  'yearbase',  params.yearbase, ...
                  'ilist', ilist , 'vars', 'all',...
                  'second_set', ensinfo(iseg).second_set);
      end
   else
      r = read(params.instclass, rawfile,  'yearbase',  params.yearbase, ...
               'beamangle', params.beamangle,...
               'ilist', ilist , 'vars', 'all');
   end         

   if isempty(r)
      a=[];
      rr=[];
      fprintf('cannot find pingtype %s\n')   
      return
   end
   
   r.vel = r.vel*params.velscale;
   
   
   %% now get navigation
   try
      aux_struct = get_serfields(r, 'gbindir', params.gbindir, ...
                                    'path_ext', params.path_ext);
   catch
      fprintf('could not match serial data to raw data\n')
      fprintf('problem with gbin for %s???\n', rawfile')
      a=[];
      rr=[];
      return
   end
      

   if (isemptys(aux_struct))
      fprintf('could not complete auxilliary (nav) fill\n')
      r = [];   
      fprintf(fid_log,'could not complete auxilliary (nav) fill\n')
      fclose(fid_log);  fid_log=0;
      return;
   end
   aux_fnames = fieldnames(aux_struct);
   
   
   for fnamei = 1:length(aux_fnames);
      fname = aux_fnames{fnamei};
      r = setfield(r, fname, getfield(aux_struct, fname));
   end
   
   rr = catadcp(rr, r);
   
end


% set the stage for averaging
rr.corr_dday = rr.nav.txy1(1,:);  %txy1 and txy2 are the same for single pings
a = cutadcp(rr,1);
a.num_pings = length(rr.dday);


a  = avg(params.instclass, rr, a, params);


