function hfig = file_chooser2(varargin)
% hfig =  file_chooser2(cfg)
% called by file_chooser
%%
% returns figure handle which is parsed and then closed by file_chooser


%normal, user-defined variables
cfg.startpath = '.';
cfg.glob      = '*';
cfg.promptstring = 'choose a file';
cfg.OKcallback  = [];

%add this so we can get back here at the end; saved in userdata
cfg.orig_pwd = pwd;

% for callbacks: programmer uses these to access parts of this code.
% NOTE: only set up to use one at a time
cfg.ok_hfig = [];          %called by 'OK': return the current string, exit
                           % if called, send in figure handle
cfg.getfile = [];          %called by "...": call uigetfile
                           % if called, send in current cfg values
cfg.exit    = [];          %chdir back to original and exit
                           

cfg = fillstruct(cfg, varargin, 0);


%%%%%

if (isempty(cfg.ok_hfig) & isempty(cfg.getfile) & isempty(cfg.exit))
   %% check to see whether there is anything matching this wildcard
   %% expansion
   filestruct = dirs(fullfile(cfg.startpath,cfg.glob));
   globstr = [];
   if (length(filestruct) == 1) %there is only one; use it
      globstr = fullfile(cfg.startpath, filestruct(1).name);
   end
   
   %figure and figure name
   hfig = figure('Color',[0.92 0.92 0.97], ...
                 'MenuBar','none', ...
                 'Name','          Choose a file', ...
                 'NumberTitle','off', ...
                 'PaperPosition',[18 180 576 432], ...
                 'PaperUnits','points', ...
                 'Position',[335 391 363 151], ...
                 'Tag', 'file_chooser_figure',...
                 'Userdata', cfg,...
                 'ToolBar','none');


   %store cfg in userdata
   set(hfig, 'userdata', cfg);
   
   
   % title
   uicontrol('Parent',hfig, ...
             'Units','points', ...
             'BackgroundColor',[0.92 0.92 0.97], ...
             'FontSize',20, ...
             'ListboxTop',0, ...
             'Position',[27.9 98.099 273.6 28.8], ...
             'String','File Selection', ...
             'Style','text');

   if (~isempty(cfg.OKcallback))
      uicontrol('Parent',hfig, ...
                'Units','points', ...
                'callback', 'file_chooser2(''ok_hfig'',0);',...
                'BackgroundColor',[0.7 0.7 0.7], ...
                'ListboxTop',0, ...
                'Position',[220.5 14.4 66.509 22.5], ...
                'Callback',cfg.OKcallback, ...
                'String','OK', ...
                'Tag','btnOK');
   else
      uicontrol('Parent',hfig, ...
                'Units','points', ...
                'callback', 'file_chooser2(''ok_hfig'',0);',...
                'BackgroundColor',[0.7 0.7 0.7], ...
                'ListboxTop',0, ...
                'Position',[220.5 14.4 66.509 22.5], ...
                'String','OK', ...
                'Tag','btnOK');
   end      
      
      
      
      
   uicontrol('Parent',hfig, ...
             'Units','points', ...
             'BackgroundColor',[0.7 0.7 0.7], ...
             'Callback','file_chooser2(''exit'',1);', ...
             'ListboxTop',0, ...
             'Position',[44.099 14.4 66.509 22.5], ...
             'String','Cancel', ...
             'Tag','btnCancel');

   %fill in the wildcard
   if (isempty(globstr))  %i.e. not initialized earlier
      globstr = fullfile(cfg.startpath, cfg.glob);
   end
   
   uicontrol('Parent',hfig, ...
             'Units','points', ...
             'BackgroundColor',[1 1 1], ...
             'FontSize',12, ...
             'HorizontalAlignment','right', ...
             'ListboxTop',0, ...
             'Position',[10.8 51.3 278.1 19.8], ...
             'Style','edit', ...
             'String', globstr, ...
             'Tag','txtboxinpath');

   %
   uicontrol('Parent',hfig, ...
             'Units','points', ...
             'BackgroundColor',[0.92 0.92 0.97], ...
             'FontSize',14, ...
             'FontWeight','demi', ...
             'ForegroundColor',[1 0 0], ...
             'ListboxTop',0, ...
             'Position',[40 72 254.9 18.9], ...
             'String', cfg.promptstring, ...
             'Style','text');

   
   uicontrol('Parent',hfig, ...
                    'Units','points', ...
                    'BackgroundColor',[0.702 0.702 0.702], ...
                    'Callback','file_chooser2(''getfile'',0);', ...
                    'FontWeight','bold', ...
                    'ListboxTop',0, ...
                    'Position',[289.6 52.2 28.8 20.7], ...
                    'String','. . .');
end


if (~isempty(cfg.getfile)) %file_chooser was called by ellipses
   getfile;
end


if (~isempty(cfg.exit))
   hfig = findobj('tag','file_chooser_figure');
   chdir(cfg.orig_pwd);
   close(hfig);
end


if (~isempty(cfg.ok_hfig))  %file_chooser was called by the OK button.
   chdir(cfg.orig_pwd);
   hfig = findobj('tag','file_chooser_figure');
   set(hfig, 'tag', 'file_chooser_done')
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

function getfile()
%getfile()


%update cfg from userdata
hfig = findobj('tag','file_chooser_figure');
cfg = get(hfig, 'userdata');

cph = findobj(hfig,'tag','txtboxinpath');
cpstr = get(cph,'string');      


uiprompt =  sprintf('Select a file of the form %s', cfg.glob);
% if a file was chosen before from uigetfile:
if (isfield(cfg, 'uifile') & isfield(cfg, 'uipath'))
   uistr = cfg.uifile;
   chdir(cfg.uipath)
elseif (exist(cpstr, 'file')) %use the file from initialization if exists
   [cppath, cpfile] = fileparts(cpstr);
   chdir(cppath)
   uistr = sprintf('%s', cfg.glob);
else %use globbing, chdir if possible 
   if (exist(cfg.startpath))
      chdir(cfg.startpath)
   end
   uistr = sprintf('%s', cfg.glob);
end

[uifile, uipath] = uigetfile(uistr, uiprompt);
           
if uipath ~= 0   % not cancelled 
   fullname = strcat(uipath, uifile);
   set(cph,'string', fullname);
else
   set(cph, 'enable', 'on');  % keep avail. if cancelled & reset
   set(cph, 'string', cpstr);
end

chdir(cfg.orig_pwd);
cfg.uipath = uipath;
cfg.uifile = uifile;
set(hfig, 'userdata', cfg);

