

import os, time, signal, sys
import Tkinter, Pmw

class Checker:
   def __init__(self, flagfile = None,
                              stopflagfile = None,
                              action = None,
                              timeout = 1):
      self.action = action
      self.flagfile = flagfile
      self.stopflagfile = stopflagfile
      self.timeout = timeout
      self.nloops = int(timeout/0.2)
      self.pid = os.getpid()
      p, self.command = os.path.split(sys.argv[0])

   def pid_from_flagfile(self):
     if self.flagfile is not None:
        pid = open(self.flagfile).readline().strip()
        return pid
     return None

   def check(self):
      plist = self.list_processes()
      if not plist:
         return
      if self.action == 'replace':
         self.end_processes()
         return
      if self.action == 'keep_old':  # keep old one if it matches flagfile
         if len(plist) == 1 and plist[0] == self.pid_from_flagfile:
            sys.exit()
         else:                  # something is wrong; run the new one
            self.end_processes()
            return
      root = Tkinter.Tk()
      Pmw.initialise(root, size=24, fontScheme='pmw1')
      root.withdraw()
      self.root = root
      self.M = Pmw.MessageDialog(root,
                  title = '%s duplicate process checker' % self.command,
                  message_text = 'Duplicate process(es):\n%s' % str(plist),
                  buttons=('Kill Earlier Duplicate(s)',
                              'Recheck', 'Cancel Latest'),
                  command=self.choices)
      self.M.wm_geometry('900x400+10+10')
      self.M.focus_set()
      root.mainloop()

   def choices(self, button):
      if button is None or button == 'Cancel Latest':
         self.exit()
      if button == 'Kill Earlier Duplicate(s)':
         self.end_processes()
         return
      if button == 'Recheck':
         plist = self.list_processes()
         if not plist:
            self.finished()
         else:
            self.M.configure(message_text =
                           'Duplicate process(es):\n%s' % str(plist))
            return
      if button == 'OK':
         if self.n_left:
            self.exit()
         else:
            self.finished()

   def list_processes(self):
      ps = os.popen('ps -C python,%s -o pid,args --no-headers' % self.command)
      procs = ps.readlines()
      procs = [l.strip() for l in procs if l.find(self.command) > 0]
      pidcmds = [l.split(None,1) for l in procs]
      pidcmds = [p for p in pidcmds if int(p[0]) != self.pid]
      return pidcmds

   def end_processes(self):
      plist = self.list_processes()
      msg = 'Ending %d process(es)' % len(plist)
      self.M.configure(message_text = msg, buttons=tuple())
      if self.stopflagfile is not None and not os.path.exists(self.stopflagfile):
         open(self.stopflagfile, 'w').close()
         for ii in range(self.nloops):
            time.sleep(0.2)
            plist = self.list_processes()
            if len(plist) == 0:
               break
      for p in plist:
         try:
            os.kill(int(p[0]), signal.SIGTERM)
         except:
            pass
      time.sleep(0.2)
      plist = self.list_processes()
      for p in plist:
         try:
            os.kill(int(p[0]), signal.SIGKILL)
         except:
            pass
      plist = self.list_processes()
      self.n_left = len(plist)
      if self.n_left:
         msg = 'Failed to kill:\n %s' % str(plist)
      else:
         msg = 'Successfully ended duplicate processes'
      self.M.configure(message_text = msg, buttons = ('OK',))
      try:
         os.remove(self.stopflagfile)
      except:
         pass
      try:
         os.remove(self.flagfile)
      except:
         pass

   def finished(self, dummy = None):
      self.root.destroy()

   def exit(self, dummy = None):
      sys.exit()

class SingleFunction(Checker):
   def __init__(self, flagdir = None, action = None, args = (), kwargs = {}):
      Checker.__init__(self, action = action)
      self._args = args
      self._kwargs = kwargs
      self.name = os.path.splitext(self.command)[0]
      if flagdir is None:
         self.flagfile = None
         self.stopflagfile = None
      else:
         self.flagfile = os.path.join(flagdir, self.name + '.running')
         self.stopflagfile = os.path.join(flagdir, self.name + '.stop')
         try: os.remove(self.stopflagfile)
         except: pass
      self.check()

   def start(self):
      ''' Write out the flagfile, and execute the run method.
      The flagfile has two lines: the pid, and the start time.
      '''
      self.start_time = time.gmtime()
      tstr = time.strftime('%Y/%m/%d %H:%M:%S', self.start_time)
      ffstr = '%s\n%s\n' % (str(self.pid), tstr)
      try:
         if self.flagfile:
            open(self.flagfile, 'w').write(ffstr)
            while not os.path.exists(self.flagfile):
               time.sleep(0.1)  # in case there is a delay
         self.run(*self._args, **self._kwargs)
      finally:
         self.cleanup()

   def run(self):
      ''' Override this.
      If it is looping, then one can touch the flagfile by including:
      os.utime(self.flagfile, None)
      '''
      raise NotImplementedError

   def cleanup(self):
      if self.flagfile:
         try: os.remove(self.flagfile)
         except: pass
      if self.stopflagfile:
         try: os.remove(self.stopflagfile)
         except: pass



if __name__ == "__main__":

   Checker().check()
   print "Sleeping..."
   time.sleep(10)


